//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   10 Jan 06  Andy Frank  Creation
//

using compiler

**
** Run the C# compiler to produce an exe or dll.
**
class CompileCs : Task
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  **
  ** Initialize the .NET environment fields for csc.exe.
  **
  new make(BuildScript script)
    : super(script)
  {
    // netHomeDir
    netHomeProp := Sys.env["fan.build.netHome"]
    try
    {
      netHomeDir = netHomeProp.toUri.toFile
      if (!netHomeDir.exists || !netHomeDir.isDir) throw Err.make
    }
    catch
    {
      throw fatal("Missing or invalid URI for fan.build.netHome: $netHomeProp")
    }

    // derived files
    cscExe = netHomeDir + `csc.exe`
  }

//////////////////////////////////////////////////////////////////////////
// Run
//////////////////////////////////////////////////////////////////////////

  **
  ** Run the csc task
  **
  override Void run()
  {
    log.message("CompileCs")

    try
    {
      // build command
      cmd := [cscExe.osPath]

      // default paramaters
      cmd.add("/nologo")
      cmd.add("/fullpaths")
      //cmd.add("/debug:full")

      // /out:output
      if (output != null)
      {
        cmd.add("/out:$output.osPath")
      }

      // /target:target
      if (target != null)
      {
        cmd.add("/target:$target")
      }

      // /r:<libs>
      if (libs != null)
      {
        s := libs.join(";") |File f->Str| { return f.osPath }
        cmd.add("/r:$s")
      }

      // src files/dirs
      src.each |File f|
      {
        if (f.isDir)
          cmd.add((f + `*.cs`).osPath)
        else
          cmd.add(f.osPath)
      }

      log.verbose(cmd.join(" "))
      r := Process.make(cmd).run
      if (r != 0) throw Err.make
    }
    catch (Err err)
    {
      throw fatal("CompileCs failed")
    }
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  ** Home directory for .NET installation
  ** configured via Sys.env["fan.build.netHome"]
  File netHomeDir

  ** C# compiler executable: {netHomeDir}/csc.exe
  File cscExe

  ** Output file created by the compiler.
  File output

  ** Output target file
  Str target

  ** List of dll libraries to link in
  File[] libs

  ** List of source files or directories to compile
  File[] src := File[,]

}