//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   26 Dec 05  Brian Frank  Creation
//   19 Aug 06  Brian Frank  Ported from Java to Fan
//

**
** FMethod is the read/write fcode representation of sys::Method.
**
class FMethod : FSlot
{

  FMethodVar[] params()
  {
    return vars.findAll |FMethodVar v->Bool| { return v.isParam }
  }

  Void write(OutStream out)
  {
    super.writeCommon(out)
    out.writeI2(ret)
    out.writeI2(inheritedRet)
    out.write(paramCount)
    out.write(localCount)
    vars.each |FMethodVar var| { var.write(out) }
    FUtil.writeBuf(out, code)
    super.writeFacets(out)
  }

  FMethod read(InStream in)
  {
    super.readCommon(in)
    ret = in.readU2
    inheritedRet = in.readU2
    paramCount = in.readU1
    localCount = in.readU1
    vars = FMethodVar[,];
    (paramCount+localCount).times |,| { vars.add(FMethodVar.make.read(in)) }
    code = FUtil.readBuf(in)
    super.readFacets(in)
    return this
  }

  Int ret              // type qname index
  Int inheritedRet     // type qname index
  FMethodVar[] vars    // parameters and local variables
  Int paramCount       // number of params in vars
  Int localCount       // number of locals in vars
  Buf code             // method executable code

}

**************************************************************************
** FMethodVar
**************************************************************************

**
** FMethodVar models one parameter or local variable in a FMethod
**
class FMethodVar : FConst
{

  Bool isParam()  { return flags & Param != 0 }

  Void write(OutStream out)
  {
    out.writeI2(name)
    out.writeI2(typeRef)
    out.write(flags)
    FUtil.writeBuf(out, def)
  }

  FMethodVar read(InStream in)
  {
    name    = in.readU2
    typeRef = in.readU2
    flags   = in.readU1
    def     = FUtil.readBuf(in)
    return this
  }

  Int name      // name index
  Int typeRef   // typeRef index
  Int flags     // method variable flags
  Buf def       // default expression or null (only for params)

}