//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   3 Sep 05  Brian Frank  Creation
//   7 Oct 06  Brian Frank  Port from Java to Fan
//

**
** WritePod writes the FPod to a zip file.
**
class WritePod : CompilerStep
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  new make(Compiler compiler)
    : super(compiler)
  {
  }

//////////////////////////////////////////////////////////////////////////
// Run
//////////////////////////////////////////////////////////////////////////

  **
  ** Not used, use write instead
  **
  override Void run() { throw UnsupportedErr.make }

  **
  ** Run the step and return pod file written
  **
  File write()
  {
    dir  := compiler.input.outDir
    fpod := compiler.fpod
    podFile := dir + "${fpod.podName}.pod".toUri
    location = Location.makeFile(podFile)

    log.message("WritePod", podFile.toStr)

    // create output directory
    dir.create

    Zip zip := null
    try
    {
      // open zip store
      zip = Zip.write(podFile.out)

      // write fpod data structures into zip file
      fpod.write(zip)

      // write resource files
      compiler.resFiles.each |File f| { writeRes(zip, f) }

      // if including fandoc write it out too
      if (compiler.input.includeDoc) writeTypeDocs(zip)

      // if including source write it out too
      if (compiler.input.includeSrc) writeSrc(zip)
    }
    catch (CompilerErr e)
    {
      throw e
    }
    catch (Err e)
    {
      throw errReport(CompilerErr.make("Cannot write", location, e))
    }

    // close file
    if (zip != null) zip.close
    return podFile
  }

//////////////////////////////////////////////////////////////////////////
// Resource
//////////////////////////////////////////////////////////////////////////

  private Void writeRes(Zip zip, File file, Uri path := null)
  {
    input := compiler.input
    if (path == null)
    {
      path = file.uri
      path = path - input.homeDir.uri
    }

    try
    {
      out := zip.writeNext(path, file.modified)
      file.in.pipe(out)
      out.close
    }
    catch (Err e)
    {
      throw errReport(CompilerErr.make("Cannot write resource file '$path'", location, e))
    }
  }

//////////////////////////////////////////////////////////////////////////
// Doc
//////////////////////////////////////////////////////////////////////////

  private Void writeTypeDocs(Zip zip)
  {
    compiler.types.each |TypeDef t|
    {
      if (!t.isSynthetic) writeTypeDoc(zip, t)
    }
  }

  private Void writeTypeDoc(Zip zip, TypeDef t)
  {
    try
    {
      out := zip.writeNext("doc/${t.name}.fandoc".toUri)
      writeDoc(out, t.qname, t.doc)
      t.slotDefs.each |SlotDef s|
      {
        writeDoc(out, s.qname, s.doc)
      }
      out.close
    }
    catch (Err e)
    {
      throw errReport(CompilerErr.make("Cannot write fandoc '$t.name'", t.location, e))
    }
  }

  **
  ** FDoc is used to read/write a fandoc text file.  The fandoc file
  ** format is an extremely simple plan text format with left justified
  ** type/slot qnames, followed by the fandoc content indented two spaces.
  **
  private static Void writeDoc(OutStream out, Str key, Str[] doc)
  {
    if (doc == null) return
    out.printLine(key)
    doc.each |Str line| { out.print("  ").printLine(line) }
    out.printLine
  }

//////////////////////////////////////////////////////////////////////////
// Src
//////////////////////////////////////////////////////////////////////////

  private Void writeSrc(Zip zip)
  {
    compiler.srcFiles.each |File f|
    {
      writeRes(zip, f, "src/$f.name".toUri)
    }
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  private Location location
}