//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   02 Jul 06  Andy Frank  Creation
//

using web

**
** FandocPod.
**
class FandocPod : FandocWeblet
{

//////////////////////////////////////////////////////////////////////////
// Weblet
//////////////////////////////////////////////////////////////////////////

  override Void get()
  {
    pod := (Pod)req.stash["pod"]

    doHeader("$pod.name - Fandoc")

    sorter := |Type a, Type b -> Int| { return a.name <=> b.name }
    filter := |Type t -> Bool|
    {
      return !t.isSynthetic && (t == Test.type || !t.fits(Test.type))
    }

    types   := pod.types.rw.sort(sorter).findAll(filter)
    classes := types.findAll |Type t -> Bool| { return !t.isMixin }
    mixins  := types.findAll |Type t -> Bool| { return t.isMixin }

    out.div("class='header'")
    out.table("width='100%' cellspacing='0' cellpadding='0'")
    out.tr

    out.td
    doBreadcrumb(pod)
    out.tdEnd

    out.td("style='text-align: right;'")
    doSearch() // TODO - WTF
    out.tdEnd

    out.trEnd
    out.tableEnd
    out.divEnd

    out.div("class='content'")

    out.div("style='float:right;'")
    out.a(`#`, "onclick='showAllSlots(); return false;'")
    out.w("Show All Types").aEnd.nl
    out.divEnd

    out.h1("$pod.name Pod")
    writeTable("Mixins", mixins)
    writeTable("Classes", classes)
    out.divEnd

    doFooter
  }

  **
  ** Write out the table for these types.
  **
  Void writeTable(Str title, Type[] types)
  {
    if (types.isEmpty) return

    out.div("class='slotTable'")
    out.h1(title)
    out.table("width='100%' cellspacing='0' cellpadding='0'")
    flip := true
    types.each |Type t|
    {
      shw := t.isPublic
      cls := ""
      doc := (t.doc != null) ? firstSentance(t.doc) : null

      if (shw)
      {
        cls = (flip) ? "rowEven" : "rowOdd"
        flip = !flip
      }
      else cls = "internal"

      if (shw) out.tr
      else out.tr("class='hidden'")

      out.td("class='$cls col2'")
      out.a(typeUri(t)).w(t.name).aEnd
      out.tdEnd

      out.td("width='100%' class='$cls col3'")
      if (doc != null)
        out.w(safe(doc))
      else
        out.w("&nbsp;") // b/c IE sucks
      out.tdEnd

      out.trEnd
    }
    out.tableEnd
    out.divEnd
  }

}