//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   05 Apr 06  Andy Frank  Creation
//

using fantext
using web

**
** FandocWeblet.
**
class FandocWeblet : Weblet, FandocUri
{

//////////////////////////////////////////////////////////////////////////
// Weblet
//////////////////////////////////////////////////////////////////////////

  virtual override Void get() { route }

//////////////////////////////////////////////////////////////////////////
// FandocUri - TEMP
//////////////////////////////////////////////////////////////////////////

  override WebReq _req() { return req }
  override WebRes _res() { return res }

//////////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////////

  **
  ** Write out the common start for each page.
  **
  Void doHeader(Str title)
  {
    res.headers["Content-Type"] = "text/html"

    out.docType

    out.html
    out.head
    out.w("<meta http-equiv='Content-Type' content='text/html; charset=UTF-8' />\n")
    out.title(title)
    out.css(req.prefixUri + `res/fandoc.css`)
    out.js(req.prefixUri + `res/fandoc.js`)
    out.headEnd
    out.body
  }

  **
  ** Write out the breadcrumb.
  **
  Void doBreadcrumb(Pod pod := null, Type type := null, Bool src := false)
  {
    out.a(indexUri).w("All Pods").aEnd.nl
    if (pod != null)
    {
      out.w(" &#187; ");
      out.a(podUri(pod)).w("$pod.name").aEnd.nl

      if (type != null)
      {
        out.w(" &#187; ");
        out.a(typeUri(type)).w("$type.name").aEnd.nl

        if (src) out.w(" &#187; Source");
      }
    }
  }

  **
  ** Write out search field.
  **
  Void doSearch()
  {
    action := req.prefixUri + "search".toUri

    out.div("class='search'")
    out.form("method='get' action='$action'")
    out.w("Search ").textField("name='q'").nl
    out.formEnd
    out.divEnd
  }

  **
  ** Write out the common footer for each page.
  **
  Void doFooter()
  {
    out.bodyEnd
    out.htmlEnd
  }

  **
  ** Escape illegal chars.
  **
  Str safe(Str s)
  {
    s = s.replace("&", "&amp;")
    s = s.replace("<", "&lt;")
    s = s.replace(">", "&gt;")
    return s
  }

  **
  ** Return the first sentence from this text.
  **
  Str firstSentance(Str s)
  {
    a := s.index(". ")
    b := s.index(".\n")
    i := -1

    if (a != null && b == null)      i = a
    else if (a == null && b != null) i = b
    else if (a != null && b != null) i = a.min(b)

    return (i < 0) ? s : s[0..i]
  }

  **
  ** Write out the fantext for this text - if an exception
  ** is thrown, write the original text.
  **
  Void fantext(Str text)
  {
    try { FantextParser.makeForStr(text).translate(out) }
    catch { out.w(text) }
  }

}