//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   4 Jul 06  Brian Frank  Creation
//
package fan.sys;

import fanx.fcode.*;

/**
 * Facets encapsulate meta-data as a map of Str name/value pairs.
 */
public final class Facets
  extends FanObj
{

//////////////////////////////////////////////////////////////////////////
// FCode
//////////////////////////////////////////////////////////////////////////

  public static Facets make(FPod fpod, Object spec1, Object spec2, FFacet[] facets)
  {
    Facets r = make();
    for (int i=0; i<facets.length; ++i)
    {
      FFacet f = facets[i];
      if ((f.flags & FConst.Reflective) != 0)
      {
        String name = fpod.name(f.name);
        try
        {
          String val  = f.data.utf();
          r.set(Str.make(name), Str.make(val));
        }
        catch (Exception e)
        {
          String spec = String.valueOf(spec1);
          if (spec2 != null) spec += "." + spec2;
          System.out.println("ERROR: Invalid facet value: " + spec + "/" + name);
        }
      }
    }
    return r.ro();
  }

//////////////////////////////////////////////////////////////////////////
// Constructors
//////////////////////////////////////////////////////////////////////////

  public static Facets make()
  {
    return new Facets(new Map(Sys.StrType, Sys.StrType));
  }

  public Facets(Map map)
  {
    this.map = map;
  }

//////////////////////////////////////////////////////////////////////////
// Identity
//////////////////////////////////////////////////////////////////////////

  public final Bool equals(Obj that)
  {
    if (that instanceof Facets)
    {
      return map.equals(((Facets)that).map);
    }
    return Bool.False;
  }

  public final Int hash() { return map.hash(); }

  public final Str toStr() { return map.toStr(); }

  public final Type type() { return Sys.FacetsType; }

//////////////////////////////////////////////////////////////////////////
// Map
//////////////////////////////////////////////////////////////////////////

  public Bool isEmpty() { return map.isEmpty(); }

  public Int size() { return map.size(); }

  public List keys() { return map.keys(); }

//////////////////////////////////////////////////////////////////////////
// Getters
//////////////////////////////////////////////////////////////////////////

  public Str get(Str key) { return (Str)map.get(key); }
  public Str get(Str key, Str def)
  {
    Str val = (Str)map.get(key);
    if (val == null) return def;
    return val;
  }

  public Bool getBool(Str key) { return getBool(key, null); }
  public Bool getBool(Str key, Bool def)
  {
    Str val = (Str)map.get(key);
    if (val == null) return def;
    return val.toBool();
  }

  public Int getInt(Str key) { return getInt(key, null); }
  public Int getInt(Str key, Int def)
  {
    Str val = (Str)map.get(key);
    if (val == null) return def;
    return val.toInt();
  }

  public Float getFloat(Str key) { return getFloat(key, null); }
  public Float getFloat(Str key, Float def)
  {
    Str val = (Str)map.get(key);
    if (val == null) return def;
    return val.toFloat();
  }

//////////////////////////////////////////////////////////////////////////
// Setters
//////////////////////////////////////////////////////////////////////////

  public Facets set(Str key, Str val)
  {
    map.set(key, val);
    return this;
  }

  public Facets setBool(Str key, Bool val)
  {
    return set(key, val.toStr());
  }

  public Facets setInt(Str key, Int val)
  {
    return set(key, val.toStr());
  }

  public Facets setFloat(Str key, Float val)
  {
    return set(key, val.toStr());
  }

  public Str remove(Str key)
  {
    return (Str)map.remove(key);
  }

  public void clear()
  {
    map.clear();
  }

//////////////////////////////////////////////////////////////////////////
// Readonly
//////////////////////////////////////////////////////////////////////////

  public final Bool isRW()
  {
    return map.isRW();
  }

  public final Bool isRO()
  {
    return map.isRO();
  }

  public final Facets rw()
  {
    if (isRW().val) return this;
    return new Facets(map.rw());
  }

  public final Facets ro()
  {
    if (isRO().val) return this;
    return new Facets(map.ro());
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  private Map map;

}