//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   3 Sep 05  Brian Frank  Creation
//
package fanx.compiler;

/**
 * CompileException - instances should always be created
 * via CompilerStep.err().
 */
public class CompileException
  extends RuntimeException
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  CompileException(String msg, ILocation location, Throwable cause)
  {
    super(format(msg, location));
    this.msg      = msg;
    this.location = location;
    this.cause    = cause;
  }

  CompileException(String msg, ILocation location)
  {
    this(msg, location, null);
  }

  CompileException(String msg, Object file, Throwable cause)
  {
    this(msg, new Location(file), cause);
  }

  CompileException(String msg, Object file)
  {
    this(msg, new Location(file), null);
  }

  CompileException(String msg, Throwable cause)
  {
    this(msg, null, cause);
  }

  CompileException(String msg)
  {
    this(msg, null, null);
  }

//////////////////////////////////////////////////////////////////////////
// Utils
//////////////////////////////////////////////////////////////////////////

  public static String format(String msg, ILocation loc)
  {
    if (loc == null || loc.file() == null) return msg;
    return format(loc) + ": " + msg;
  }

  public static String format(ILocation loc)
  {
    StringBuilder s = new StringBuilder(loc.file().toString());
    if (loc.line() > 0)
    {
      s.append('(').append(loc.line());
      if (loc.line() > 0) s.append(',').append(loc.col());
      s.append(')');
    }
    return s.toString();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public final String msg;
  public final ILocation location;
  public final Throwable cause;

}