//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler;

import java.util.*;
import fan.sys.*;
import fanx.compiler.ast.*;

/**
 * NameSpace is used to resolve identifiers to types, methods, and fields.
 * It encapsulates the ShimPod/ShimTypes from the actual committed Pod/Types
 * so that it all looks one single namespace.
 */
public class NameSpace
  extends CompilerSupport
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public NameSpace(Compiler compiler)
  {
    super(compiler);
  }

//////////////////////////////////////////////////////////////////////////
// Init
//////////////////////////////////////////////////////////////////////////

  public void add(ShimType type)
  {
    shimTypes.put(type.name().val, type);
  }

//////////////////////////////////////////////////////////////////////////
// Resolve Type
//////////////////////////////////////////////////////////////////////////

  /**
   * Given a type name, resolve the name to a single Type based
   * on the current namespace and imports defined.  If the name
   * maps to more than one potential type, then log errors.  Return
   * null if no dice.
   */
  public Type resolveType(CompilationUnit curUnit, ILocation loc, TypeRef ref)
  {
    // short circuit if already resolved
    if (ref.type != null) return ref.type;

    // handle special type refs
    // TODO - I think short circuit above handles all cases now
    //if (ref instanceof ListTypeRef)   return resolveListType(curUnit, loc, (ListTypeRef)ref);

    String name = ref.id;

    // check this namespace
    Type type = (Type)shimTypes.get(name);

    // check imports
    for (int i=0; i<curUnit.imports.size(); ++i)
    {
      Type dup = null;

      Import imp = (Import)curUnit.imports.get(i);
      dup = imp.pod.findType(name, false);

      if (dup != null)
      {
        if (type == null)
          type = dup;
        else
          throw err("Ambiguous type for " + name + ": " + type.qname() + " & " + dup.qname(), loc);
      }
    }

    return type;
  }

  /*
  private Type resolveListType(CompilationUnit curUnit, ILocation loc, ListTypeRef ref)
  {
    Type of = resolveType(curUnit, loc, ref.of);
    return (of != null) ? of.toListOf() : null;
  }
  */

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  HashMap shimTypes = new HashMap();   // by name

}