//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.asm;

import java.io.*;
import java.util.*;
import fan.sys.*;
import fanx.fcode.*;
import fanx.util.*;
import fanx.compiler.*;
import fanx.compiler.Compiler;
import fanx.compiler.ast.*;

/**
 * FacetAsm provides support for assembling the facets table for types and slots.
 */
public class FacetAsm
  implements FConst
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public FacetAsm(FPod fpod)
  {
    this.fpod = fpod;
    this.facets = new ArrayList();
  }

//////////////////////////////////////////////////////////////////////////
// Access
//////////////////////////////////////////////////////////////////////////

  public FFacet[] list()
  {
    return (FFacet[])facets.toArray(new FFacet[facets.size()]);
  }

//////////////////////////////////////////////////////////////////////////
// Named Adds
//////////////////////////////////////////////////////////////////////////

  public void sourceFile(String source)
  {
    utf(SourceFile, Reflective, source);
  }

  public void lineNumber(int line)
  {
    utf(LineNumber, Reflective, String.valueOf(line));
  }

//////////////////////////////////////////////////////////////////////////
// Generic Adds
//////////////////////////////////////////////////////////////////////////

  public void utf(String name, int flags, String data)
  {
    Box buf = new Box();
    buf.utf(data);
    add(name, flags, buf);
  }

  public void add(String name, int flags, Box data)
  {
    facets.add(new FFacet(fpod.name(name), flags, new FBuf(data)));
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  FPod fpod;
  ArrayList facets;

}