//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import java.io.*;
import fanx.util.*;

/**
 * AstWriter is used to debug dump the abstract syntax tree.
 */
public class AstWriter
  extends PrintWriter
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  /**
   * Construct for specified base Writer.
   */
  public AstWriter(Writer out)
  {
    super(out);
  }

  /**
   * Construct for specified base OutputStream.
   */
  public AstWriter(OutputStream out)
  {
    super(out);
  }

  /**
   * Construct for System.out with initial indent
   */
  public AstWriter(int indent)
  {
    super(System.out);
    this.indent = indent;
  }

  /**
   * Construct for System.out
   */
  public AstWriter()
  {
    super(System.out);
  }

//////////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////////

  /**
   * Write and then return this.
   */
  public AstWriter w(Object o)
  {
    if (nl) { print(StrUtil.getSpaces(indent*2)); nl = false; }
    print(o);
    return this;
  }

  /**
   * Write newline and then return this.
   */
  public AstWriter nl()
  {
    w("\n");
    nl = true;
    flush();
    return this;
  }

  /**
   * Increment the indentation
   */
  public AstWriter indent()
  {
    indent++;
    return this;
  }

  /**
   * Decrement the indentation
   */
  public AstWriter unindent()
  {
    indent--;
    if (indent < 0) indent = 0;
    return this;
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public int indent = 0;
  public boolean nl;
}