//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import fanx.compiler.*;

/**
 * ExprStmt models a stand along expression statement.
 */
public class ExprStmt
  extends Stmt
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public ExprStmt(ILocation location, Expr expr)
  {
    super(location);
    this.expr = expr;
  }

  public ExprStmt(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// Stmt
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return EXPR;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    expr = walk(v, expr);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public void tree(AstWriter out)
  {
    out.w("ExprStmt").nl();
    out.indent();
    expr.tree(out);
    out.unindent();
  }

  public void pretty(AstWriter out, boolean endOfStmt)
  {
    expr.pretty(out);
    if (endOfStmt) out.w(";").nl();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Expr expr;

}