//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import java.util.*;
import fan.sys.*;
import fanx.util.*;
import fanx.compiler.*;

/**
 * FieldDef models a field definition.
 */
public class FieldDef
  extends SlotDef
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public FieldDef(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// SlotDef
//////////////////////////////////////////////////////////////////////////

  public Slot slot() { return field; }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  public Node walk(Visitor v)
  {
    v.enterFieldDef(this);
    type = walk(v, type);
    init = walk(v, init);
    get  = walk(v, get);
    set  = walk(v, set);
    v.exitFieldDef(this);
    return v.visit(this);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public void tree(AstWriter out)
  {
    out.w("FieldDef " + name + " [" + StrUtil.flagsToString(flags) + "]").nl();
    out.indent();

    out.w("type: ");
    if (type == null) out.w("null").nl();
    else type.tree(out);

    out.w("init: ");
    if (init == null) out.w("null").nl();
    else init.tree(out);

    out.w("get [" + StrUtil.flagsToString(getFlags) + "]: ");
    if (get == null) out.w("null").nl();
    else get.tree(out);

    out.w("set [" + StrUtil.flagsToString(setFlags) + "]: ");
    if (set == null) out.w("null").nl();
    else set.tree(out);

    out.unindent();
  }

  public void pretty(AstWriter out)
  {
    out.w("" + StrUtil.flagsToString(flags));
    if (type != null) out.w(type).w(" ");
    out.w(name);
    if (init != null) { out.w(" := "); init.pretty(out); }
    out.nl();
    if (get != null || set != null)
    {
      out.w("{").nl().indent();
      if (get != null) { out.w("get").nl(); get.pretty(out); }
      if (set != null) { out.w("set").nl(); set.pretty(out); }
      out.unindent().w("}").nl();
    }
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public TypeRef type;          // field type (null if inferred from init)
  public Field field;           // resolved finalized field
  public Expr init;             // init expression or null
  public Block get;             // getter block (null to auto-generate default)
  public Block set;             // setter block (null to auto-generate default)
  public int getFlags;          // getter flags
  public int setFlags;          // setter flags
  public MethodDef getMethod;   // synthetic getter MethodDef
  public MethodDef setMethod;   // synthetic setter MethodDef

}