//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import java.util.*;
import fan.sys.*;
import fanx.compiler.*;

/**
 * FieldExpr is used for a field variable access.
 */
public class FieldExpr
  extends Expr
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public FieldExpr(ILocation location, Field field, Expr target)
  {
    super(location);
    this.field  = field;
    this.name   = field.name().val;
    this.type   = field.fieldType();
    this.target = target;
  }

  public FieldExpr(ILocation location, Type type, String name)
  {
    super(location);
    this.type = type;
    this.name = name;
  }

  public FieldExpr(ILocation location, String name)
  {
    super(location);
    this.name = name;
  }

//////////////////////////////////////////////////////////////////////////
// Expr
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return FIELD;
  }

  public Int asTableSwitchCase()
  {
    // an Enum literal may be uses as an int literal
    // TODO: this is a little hackish - first we have to split the code
    // based on if the enum is in my own compilation unit and look it up
    // in the AST;  if already compiled we try to use reflection on the
    // field this is actually guaranteed to be only accessing a field
    // in the range?
    if (type != null && type.isEnum().val)
    {
      if (type instanceof ShimType)
      {
        ClassDef cls = ((ShimType)type).def;
        EnumDef enumDef = cls.enumDef(name);
        if (enumDef != null)
          return Int.make(enumDef.ordinal);
      }
      else
      {
        Obj obj = field.get(null);
        if (obj instanceof fan.sys.Enum)
          return ((fan.sys.Enum)obj).ordinal();
      }
    }

    return super.asTableSwitchCase();
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    target = walk(v, target);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    StringBuilder s = new StringBuilder();
    if (target != null) s.append(target).append('.');
    if (field != null) s.append(field.parent()).append(':');
    s.append(name);
    return s.toString();
  }

  public void tree(AstWriter out)
  {
    out.w("FieldExpr " + name).nl();
    out.indent();
    if (target != null) { out.w("target: "); target.tree(out); }
    out.unindent();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Expr target;          // target of field access
  public String name;          // unresolved name of field to get
  public Field field;          // resolved Field
  public boolean useAccessor;  // use getter/setter

}