//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   28 Jan 06  Brian Frank  Creation
//
package fanx.compiler.ast;

import fanx.compiler.*;

/**
 * For loop
 */
public class ForStmt
  extends Stmt
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public ForStmt(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// Stmt
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return FOR;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    init = walk(v, init);
    condition = walk(v, condition);
    update = walk(v, update);
    block = walk(v, block);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    StringBuilder s = new StringBuilder("for (");
    if (init != null) s.append(init);
    s.append("; ");
    if (condition != null) s.append(condition);
    s.append("; ");
    if (update != null) s.append(update);
    s.append(")");
    return s.toString();
  }

  public void tree(AstWriter out)
  {
    out.w("ForStmt").nl();
    out.indent();
    if (init != null) { out.w("init: "); init.tree(out); }
    if (condition != null) { out.w("condition: "); condition.tree(out); }
    if (update != null) { out.w("update: "); update.tree(out); }
    out.w("block: "); block.tree(out);
    out.unindent();
  }

  public void pretty(AstWriter out)
  {
    out.w("for (");
    if (init != null) init .pretty(out, false);
    out.w("; ");
    if (condition != null) condition.pretty(out);
    out.w("; ");
    if (update != null) update.pretty(out);
    out.w(")").nl();
    block.pretty(out);
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Stmt init;
  public Expr condition;
  public Expr update;
  public Block block;

}