//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import fanx.compiler.*;

/**
 * Return models a if or if/else statement.
 */
public class IfStmt
  extends Stmt
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public IfStmt(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// Stmt
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return IF;
  }

  public boolean isMethodExit()
  {
    if (falseBlock == null) return false;
    return trueBlock.lastIsMethodExit() && falseBlock.lastIsMethodExit();
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    condition = walk(v, condition);
    trueBlock = walk(v, trueBlock);
    falseBlock = walk(v, falseBlock );
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    return "if (" + condition + ")";
  }

  public void tree(AstWriter out)
  {
    out.w("IfStmt").nl();
    out.indent();
    out.w("condition: "); condition.tree(out);
    out.w("trueBlock: "); trueBlock.tree(out);
    if (falseBlock != null) { out.w("falseBlock: "); falseBlock.tree(out); }
    out.unindent();
  }

  public void pretty(AstWriter out)
  {
    out.w("if (");
    condition.pretty(out);
    out.w(")").nl();
    trueBlock.pretty(out);
    if (falseBlock != null)
    {
      out.w("else").nl();
      falseBlock.pretty(out);
    }
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Expr condition;      // test expression
  public Block trueBlock;     // block to execute if condition true
  public Block falseBlock;    // else clause or null

}