//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import fan.sys.*;
import fanx.util.*;
import fanx.compiler.*;

/**
 * LiteralExpr puts an Bool, Int, Float, Str, Duration, Uri, Type,
 * or null constant onto the stack.
 */
public class LiteralExpr
  extends Expr
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public LiteralExpr(ILocation location, int id, Object value)
  {
    super(location);
    this.id    = id;
    this.value = value;
  }

//////////////////////////////////////////////////////////////////////////
// Access
//////////////////////////////////////////////////////////////////////////

  public boolean asBool()  { return ((Boolean)value).booleanValue(); }
  public long    asInt()   { return ((Long)value).longValue(); }
  public double  asFloat() { return ((Double)value).doubleValue(); }
  public String  asStr()   { return (String)value; }
  public Type    asType()  { return (Type)value; }

//////////////////////////////////////////////////////////////////////////
// Expr
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return id;
  }

  public Int asTableSwitchCase()
  {
    if (id == INT_LITERAL)
      return Int.make(((LiteralExpr)this).asInt());

    return super.asTableSwitchCase();
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    if (id == STR_LITERAL) return "\"" + StrUtil.asCode((String)value) + "\"";
    if (id == NULL_LITERAL) return "null";
    if (id == DURATION_LITERAL) return value + "ns";
    return String.valueOf(value);
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public int id;
  public Object value;   // Boolean, Long (Int/Duration), Double, String, Type, or null

}