//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   7 Dec 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import java.util.*;
import fanx.compiler.*;

/**
 * MapLiteralExpr supports creating a Map populated by zero or
 * more key/value pair expressions.
 */
public class MapLiteralExpr
  extends Expr
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public MapLiteralExpr(ILocation location, MapTypeRef mapType)
  {
    super(location);
    this.mapType = mapType;
  }

//////////////////////////////////////////////////////////////////////////
// Expr
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return MAP_LITERAL;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    mapType = (MapTypeRef)walk(v, mapType);
    keys = walk(v, keys);
    vals = walk(v, vals);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public void tree(AstWriter out)
  {
    out.w("MapLiteralExpr ").w(this).nl();
    out.indent();
    if (mapType != null) { out.w("mapType: "); mapType.tree(out); }
    for (int i=0; i<keys.size(); ++i) { out.w("keys[" + i + "] "); ((Expr)keys.get(i)).tree(out); }
    for (int i=0; i<vals.size(); ++i) { out.w("vals[" + i + "] "); ((Expr)vals.get(i)).tree(out); }
    out.unindent();
  }

  public String toString()
  {
    StringBuilder s = new StringBuilder();
    if (mapType != null) s.append(mapType);
    s.append("[");
    if (keys.size() == 0) s.append(':');
    for (int i=0; i<keys.size(); ++i)
    {
      if (i > 0) s.append(", ");
      s.append(keys.get(i)).append(':').append(vals.get(i));
    }
    s.append("]");
    return s.toString();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public MapTypeRef mapType;                 // ExplicitValue[ExplicitKey][...]
  public ArrayList keys = new ArrayList();   // Expr[]
  public ArrayList vals = new ArrayList();   // Expr[]

}