//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import fan.sys.*;
import fanx.compiler.*;

/**
 * MethodParamDef models one parameter (name and type) to a Method
 */
public class MethodParamDef
  extends Node
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public MethodParamDef(TypeRef type, String name)
  {
    super(type.location);
    this.type = type;
    this.name = name;
  }

  public MethodParamDef(ILocation location, Type type, String name)
  {
    super(location);
    this.type = new TypeRef(location, type);
    this.name = name;
  }

  public MethodParamDef(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  public Node walk(Visitor v)
  {
    type = walk(v, type);
    def = walk(v, def);
    return v.visit(this);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public void tree(AstWriter out)
  {
    out.w("MethodParamDef " + name).nl();
    out.indent();
    out.w("type: ");
    type.tree(out);
    if (def != null) { out.w("def: "); def.tree(out); }
    out.unindent();
  }

  public void pretty(AstWriter out)
  {
    out.w(type).w(" ").w(name);
    if (def != null) out.w(" := ").w(def);
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public String name;    // parameter name
  public TypeRef type;   // parameter type
  public Expr def;       // default expression

}