//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   4 Feb 06  Brian Frank  Creation
//
package fanx.compiler.ast;

import fan.sys.*;
import fanx.fcode.*;

/**
 * MethodVar is a variable used in a method - either param or local.
 */
public class MethodVar
{

  public MethodVar(int register, Type type, String name, int flags, Block scope)
  {
    this.register = register;
    this.type     = type;
    this.name     = name;
    this.flags    = flags;
    this.scope    = scope;
  }

  public boolean isParam()
  {
    return (flags & FConst.Param) != 0;
  }

  public String toString()
  {
    return register + "  " + name + ": " + type;
  }

  public void tree(AstWriter out)
  {
    out.w(""+register).w("  ").w(name).w(": ").w(type);
  }

  public int register;
  public Type type;
  public String name;
  public int flags;       // Param
  public Block scope;     // pointer to block which scopes this variable
  public MethodParamDef paramDef;
  public boolean usedInClosure;  // local used by closure within containing method
  public Field cvarsField; // if mapped to a closure variables class

}