//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import java.util.*;
import fanx.compiler.*;

/**
 * RangeLiteralExpr supports creating a Range object populated
 * by a start and end expression.
 */
public class RangeLiteralExpr
  extends Expr
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public RangeLiteralExpr(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// Expr
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return RANGE_LITERAL;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    start = walk(v, start);
    end = walk(v, end);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public void tree(AstWriter out)
  {
    out.w("RangeLiteralExpr ").w(this).nl();
    out.indent();
    out.w("start: "); start.tree(out);
    out.w("end: "); end.tree(out);
    out.unindent();
  }

  public String toString()
  {
    StringBuilder s = new StringBuilder();
    s.append(start);
    s.append(inclusive ? ".." : "...");
    s.append(end);
    return s.toString();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Expr start, end;
  public boolean inclusive;

}