//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.compiler.ast;

import fanx.compiler.*;

/**
 * Return models a return statement.
 */
public class ReturnStmt
  extends Stmt
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public ReturnStmt(ILocation location)
  {
    super(location);
  }

  public ReturnStmt(Expr expr)
  {
    super(expr.location);
    this.expr = expr;
  }

//////////////////////////////////////////////////////////////////////////
// Stmt
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return RETURN;
  }

  public boolean isMethodExit()
  {
    return true;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    expr = walk(v, expr);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    if (expr != null) return "return " + expr  + ";";
    return "return;";
  }

  public void tree(AstWriter out)
  {
    out.w("ReturnStmt").nl();
    out.indent();
    if (expr != null) expr.tree(out);
    out.unindent();
  }

  public void pretty(AstWriter out)
  {
    out.w("return");
    if (expr != null) { out.w(" "); expr.pretty(out); }
    out.w(";").nl();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Expr expr;   // expression to return of null if void return

}