//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   27 Jun 06  Brian Frank  Creation
//
package fanx.compiler.ast;

import java.util.*;
import fanx.compiler.*;

/**
 * Models a try {...} catch (Err e) {...} finally {...} statement.
 */
public class TryStmt
  extends Stmt
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public TryStmt(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// Stmt
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return TRY;
  }

  public boolean isMethodExit()
  {
    if (!block.lastIsMethodExit()) return false;
    for (int i=0; i<catches.size(); ++i)
      if (!((TryCatch)catches.get(i)).block.lastIsMethodExit())
        return false;
    return true;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    block = walk(v, block);
    catches = walk(v, catches);
    finallyBlock = walk(v, finallyBlock);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    return "try";
  }

  public void tree(AstWriter out)
  {
    out.w("TryStmt").nl();
    out.indent();
    out.w("block: "); block.tree(out);
    for (int i=0; i<catches.size(); ++i)
    {
      out.w("catch: ");
      ((TryCatch)catches.get(i)).tree(out);
    }
    out.w("finallyBlock: "); if (finallyBlock != null) finallyBlock.tree(out);
    out.unindent();
  }

  public void pretty(AstWriter out)
  {
    out.w("try").nl();
    block.pretty(out);
    for (int i=0; i<catches.size(); ++i)
      ((TryCatch)catches.get(i)).pretty(out);
    if (finallyBlock != null)
    {
      out.w("finally").nl();
      finallyBlock.pretty(out);
    }
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Expr exception;      // expression which leaves exception on stack
  public Block block;         // body of try block
  public ArrayList catches = new ArrayList();  // TryCatch[]
  public Block finallyBlock;  // body of finally block or null

}