//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   28 Jan 06  Brian Frank  Creation
//
package fanx.compiler.ast;

import fanx.compiler.*;

/**
 * While loop
 */
public class WhileStmt
  extends Stmt
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public WhileStmt(ILocation location)
  {
    super(location);
  }

//////////////////////////////////////////////////////////////////////////
// Stmt
//////////////////////////////////////////////////////////////////////////

  public int id()
  {
    return WHILE;
  }

//////////////////////////////////////////////////////////////////////////
// Tree
//////////////////////////////////////////////////////////////////////////

  protected void walkChildren(Visitor v)
  {
    condition = walk(v, condition);
    block = walk(v, block);
  }

//////////////////////////////////////////////////////////////////////////
// Debug
//////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    return "while (" + condition + ")";
  }

  public void tree(AstWriter out)
  {
    out.w("WhileStmt").nl();
    out.indent();
    out.w("condition: "); condition.tree(out);
    out.w("block: "); block.tree(out);
    out.unindent();
  }

  public void pretty(AstWriter out)
  {
    out.w("while (");
    condition.pretty(out);
    out.w(")").nl();
    block.pretty(out);
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Expr condition;
  public Block block;

}