//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   3 Sep 05  Brian Frank  Creation
//
package fanx.compiler.parser;

import java.lang.reflect.*;
import java.lang.reflect.Field;
import java.util.*;
import fanx.compiler.*;

/**
 * Token models a token in a source file.
 */
public class Token
  implements ILocation
{

//////////////////////////////////////////////////////////////////////////
// Token Ids
//////////////////////////////////////////////////////////////////////////

  public static final int ID               = 0;
  public static final int STR_LITERAL      = 1;
  public static final int CHAR_LITERAL     = 2;
  public static final int INT_LITERAL      = 3;
  public static final int FLOAT_LITERAL    = 4;
  public static final int DURATION_LITERAL = 5;
  public static final int URI_LITERAL      = 6;
  // operators
  public static final int DOT              = 10;  //  .
  public static final int SEMICOLON        = 11;  //  ;
  public static final int COMMA            = 12;  //  ,
  public static final int COLON            = 13;  //  :
  public static final int PLUS             = 14;  //  +
  public static final int MINUS            = 15;  //  -
  public static final int STAR             = 16;  //  *
  public static final int SLASH            = 17;  //  /
  public static final int PERCENT          = 18;  //  %
  public static final int POUND            = 19;  //  #
  public static final int INCREMENT        = 20;  //  ++
  public static final int DECREMENT        = 21;  //  --
  public static final int BANG             = 22;  //  !
  public static final int QUESTION         = 23;  //  ?
  public static final int TILDE            = 24;  //  ~
  public static final int PIPE             = 25;  //  |
  public static final int AMP              = 26;  //  &
  public static final int CARET            = 27;  //  ^
  public static final int DOUBLE_PIPE      = 28;  //  ||
  public static final int DOUBLE_AMP       = 29;  //  &&
  public static final int EQ               = 30;  //  ==
  public static final int NOT_EQ           = 31;  //  !=
  public static final int SAME             = 32;  //  ===
  public static final int NOT_SAME         = 33;  //  !==
  public static final int COMPARE          = 34;  //  <=>
  public static final int LT               = 35;  //  <
  public static final int LTEQ             = 36;  //  <=
  public static final int GT               = 37;  //  >
  public static final int GTEQ             = 38;  //  >=
  public static final int LSHIFT           = 39;  //  <<
  public static final int RSHIFT           = 40;  //  >>
  public static final int LBRACE           = 41;  //  {
  public static final int RBRACE           = 42;  //  }
  public static final int LPAREN           = 43;  //  (
  public static final int RPAREN           = 44;  //  )
  public static final int LBRACKET         = 45;  //  [
  public static final int RBRACKET         = 46;  //  ]
  public static final int DOT_DOT          = 47;  //  ..
  public static final int DOT_DOT_DOT      = 48;  //  ...
  public static final int DEF_ASSIGN       = 49;  //  :=
  public static final int ASSIGN           = 50;  //  =
  public static final int ASSIGN_PLUS      = 51;  //  +=
  public static final int ASSIGN_MINUS     = 52;  //  -=
  public static final int ASSIGN_STAR      = 53;  //  *=
  public static final int ASSIGN_SLASH     = 54;  //  /=
  public static final int ASSIGN_PERCENT   = 55;  //  %=
  public static final int ASSIGN_AMP       = 56;  //  &=
  public static final int ASSIGN_PIPE      = 57;  //  |=
  public static final int ASSIGN_CARET     = 58;  //  ^=
  public static final int ASSIGN_LSHIFT    = 59;  //  <<=
  public static final int ASSIGN_RSHIFT    = 60;  //  >>=
  public static final int ARROW            = 61;  //  ->
  public static final int JALIAS           = 62;  //  jalias:
  public static final int NALIAS           = 63;  //  nalias:
  public static final int JCODE            = 64;  //  jcode:
  public static final int NCODE            = 65;  //  ncode:
  public static final int DOC_COMMENT      = 66;  //  /** comment */
  // keywords
  public static final int ABSTRACT         = 100;  // FIRST_KEYWORD
  public static final int AS               = 101;
  public static final int ASSERT           = 102;
  //public static final int BASE             = 103;
  //public static final int BOOL             = 104;
  //public static final int BOOLEAN          = 105;
  public static final int BREAK            = 106;
  //public static final int BYTE             = 107;
  public static final int CASE             = 108;
  public static final int CATCH            = 109;
  //public static final int CHAR             = 110;
  //public static final int CHECKED          = 111;
  public static final int CLASS            = 112;
  public static final int CONST            = 113;
  public static final int CONTINUE         = 114;
  //public static final int DECIMAL          = 115;
  public static final int DEFAULT          = 116;
  public static final int DELEGATE         = 117;
  public static final int DO               = 118;
  //public static final int DOUBLE           = 119;
  //public static final int DUCK           = ;
  public static final int ELSE             = 120;
  public static final int ENUM             = 121;
  public static final int EVENT            = 122;
  public static final int EXPLICIT         = 123;
  public static final int EXTENDS          = 124;
  public static final int EXTERN           = 125;
  public static final int FALSE            = 126;
  public static final int FINAL            = 127;
  public static final int FINALLY          = 128;
  public static final int FIXED            = 129;
  //public static final int FLOAT            = 130;
  public static final int FOR              = 131;
  public static final int FOREACH          = 132;
  public static final int GOTO             = 133;
  public static final int IF               = 134;
  public static final int IMPLICIT         = 135;
  //public static final int IN             = 136;
  public static final int IMPLEMENTS       = 136;
  //public static final int IMPORT           = 137;
  public static final int INTSTANCEOF      = 138;
  //public static final int INT              = 139;
  public static final int INTERNAL         = 140;
  public static final int INTERFACE        = 141;
  public static final int IS               = 142;
  public static final int LOCK             = 143;
  //public static final int LONG             = 144;
  public static final int MIXIN            = 145;
  //public static final int NAMESPACE        = 145;
  public static final int NATIVE           = 146;
  public static final int NEW              = 147;
  public static final int NULL             = 148;
  public static final int OBJECT           = 149;
  public static final int OPERATOR         = 150;
  //public static final int OUT            = 151;
  public static final int OVERRIDE         = 152;
  public static final int PACKAGE          = 153;
  //public static final int PARAMS           = 154;
  public static final int PRIVATE          = 155;
  public static final int PROTECTED        = 156;
  public static final int PUBLIC           = 157;
  public static final int READONLY         = 158;
  public static final int REF              = 159;
  public static final int RETURN           = 160;
  //public static final int SBYTE            = 161;
  public static final int SEALED           = 162;
  //public static final int SHORT            = 163;
  public static final int SIZEOF           = 164;
  public static final int STACKALLOC       = 165;
  public static final int STATIC           = 166;
  //public static final int STR              = 167;
  public static final int STRICTFP         = 168;
  //public static final int STRING           = 169;
  public static final int STRUCT           = 170;
  public static final int SUPER            = 171;
  public static final int SWITCH           = 172;
  public static final int SYNCHRONIZED     = 173;
  public static final int THIS             = 174;
  public static final int THROW            = 175;
  public static final int THROWS           = 176;
  public static final int TRANSIENT        = 177;
  public static final int TRUE             = 178;
  public static final int TRY              = 179;
  public static final int TYPEOF           = 180;
  //public static final int UINT             = 181;
  //public static final int ULONG            = 182;
  public static final int UNCHECKED        = 183;
  public static final int UNSAFE           = 184;
  //public static final int USHORT           = 185;
  public static final int USING            = 186;
  public static final int VIRTUAL          = 187;
  public static final int VOLATILE         = 188;
  public static final int VOID             = 189;
  public static final int WHILE            = 190;  // LAST_KEYWORD
  // misc
  public static final int EOF              = 191;

  // potential keywords: master list in compiler::Token

  public static final int _FIRST_KEYWORD  = ABSTRACT;
  public static final int _LAST_KEYWORD   = WHILE;

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  public Token(int type, Object val)
  {
    this.type = type;
    this.val  = val;
  }

  public Token(int type)
  {
    this.type = type;
  }

  public Token()
  {
  }

//////////////////////////////////////////////////////////////////////////
// ILocation
//////////////////////////////////////////////////////////////////////////

  public final Object file()   { return file; }
  public final int line()      { return line; }
  public final int col()       { return col; }
  public final String format() { return CompileException.format(this); }

//////////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////////

  public boolean isId()
  {
    return type == ID;
  }

  public boolean isLiteral()
  {
    switch (type)
    {
      case STR_LITERAL:
      case CHAR_LITERAL:
      case INT_LITERAL:
      case FLOAT_LITERAL:
        return true;
      default:
        return false;
    }
  }

  public boolean isKeyword()
  {
    return _FIRST_KEYWORD <= type && type <= _LAST_KEYWORD;
  }

  public boolean isAssign()
  {
    switch (type)
    {
      case Token.ASSIGN:
      case Token.ASSIGN_PLUS:
      case Token.ASSIGN_MINUS:
      case Token.ASSIGN_STAR:
      case Token.ASSIGN_SLASH:
      case Token.ASSIGN_PERCENT:
      case Token.ASSIGN_AMP:
      case Token.ASSIGN_PIPE:
      case Token.ASSIGN_CARET:
      case Token.ASSIGN_LSHIFT:
      case Token.ASSIGN_RSHIFT:
        return true;
      default:
        return false;
    }
  }

  public boolean equals(Object o)
  {
    if (!(o instanceof Token)) return false;

    Token t = (Token)o;

    if (type != t.type) return false;

    if (val != null)
    {
      if (t.val == null) return false;
      if (!val.equals(t.val)) return false;
    }
    else
    {
      if (t.val!= null) return false;
    }

    return true;
  }

  public String typeToString()
  {
    return toString(type);
  }

  public static String toString(int type)
  {
    if (type < 0) return "Token[" + type + "]?";

    String t = ""+names[type];
    switch (type)
    {
      case ID:             return "identifier";
      case STR_LITERAL:    return "str literal";
      case CHAR_LITERAL:   return "char literal";
      case INT_LITERAL:    return "int literal";
      case FLOAT_LITERAL:  return "float literal";
      case DOT:            return ".";
      case SEMICOLON:      return ";";
      case COMMA:          return ",";
      case COLON:          return ":";
      case PLUS:           return "+";
      case MINUS:          return "-";
      case STAR:           return "*";
      case SLASH:          return "/";
      case PERCENT:        return "%";
      case INCREMENT:      return "++";
      case DECREMENT:      return "--";
      case BANG:           return "!";
      case QUESTION:       return "?";
      case TILDE:          return "~";
      case PIPE:           return "|";
      case AMP:            return "&";
      case CARET:          return "^";
      case DOUBLE_PIPE:    return "||";
      case DOUBLE_AMP:     return "&&";
      case EQ:             return "==";
      case SAME:           return "===";
      case LT:             return "<";
      case LTEQ:           return "<=";
      case GT:             return ">";
      case GTEQ:           return ">=";
      case LSHIFT:         return "<<";
      case RSHIFT:         return ">>";
      case LBRACE:         return "{";
      case RBRACE:         return "}";
      case LPAREN:         return "(";
      case RPAREN:         return ")";
      case LBRACKET:       return "[";
      case RBRACKET:       return "]";
      case DOT_DOT:        return "..";
      case DOT_DOT_DOT:    return "...";
      case DEF_ASSIGN:     return ":=";
      case ASSIGN:         return "=";
      case ASSIGN_PLUS:    return "+=";
      case ASSIGN_MINUS:   return "-+";
      case ASSIGN_STAR:    return "*=";
      case ASSIGN_SLASH:   return "/=";
      case ASSIGN_PERCENT: return "%=";
      case ASSIGN_AMP:     return "&=";
      case ASSIGN_PIPE:    return "|=";
      case ASSIGN_CARET:   return "^=";
      case ASSIGN_LSHIFT:  return "<<=";
      case ASSIGN_RSHIFT:  return ">>=";
      case ARROW:          return "->";
      default:             return t.toLowerCase();
    }
  }

  public String toString()
  {
    String v = ""+val;
    switch (type)
    {
      case ID:               return v;
      case STR_LITERAL:      return '"' + v + '"';
      case CHAR_LITERAL:     return '\'' + v + '\'';
      case INT_LITERAL:      return v;
      case FLOAT_LITERAL:    return v + "f";
      case DURATION_LITERAL: return v + "ns";
      case JALIAS:           return "jalias:";
      case NALIAS:           return "nalias:";
      case JCODE:            return "jcode:";
      case NCODE:            return "ncode:";
      default:               return toString(type);
    }
  }

//////////////////////////////////////////////////////////////////////////
// Introspection
//////////////////////////////////////////////////////////////////////////

  static String[] names;
  static HashMap keywords = new HashMap();
  static
  {
    try
    {
      // use reflection to map fields to names array
      Field[] fields = Token.class.getFields();
      names = new String[EOF+1];
      for (int i=0; i<fields.length; ++i)
      {
        Field f = fields[i];
        String name = f.getName();
        if (f.getType() == int.class && name.charAt(0) < 'a' && name.charAt(0) != '_')
        {
          int id = f.getInt(null);
          if (names[id] != null) throw new Exception("ERR: dup const " + id);
          names[id] = name;
        }
      }

      // populate hashmap of keywords
      for (int i=ABSTRACT; i<=WHILE; ++i)
      {
        if (names[i] == null) continue;
        String keyword = names[i].toLowerCase();
        keywords.put(keyword, new Integer(i));
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  };

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public int type;         // type constant
  public Object val;       // String, Character, Long (Int/Duration), or Double
  public String file;      // source file
  public int line;         // line number of token
  public int col;          // column number of token
  public boolean newline;  // have we processed one or more newlines since the last token

}