//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   20 Jun 06  Brian Frank  Creation
//
package fanx.compiler.steps;

import java.util.*;
import fan.sys.*;
import fanx.compiler.*;
import fanx.compiler.Compiler;
import fanx.compiler.parser.*;
import fanx.compiler.ast.*;

/**
 * CheckMethodParamDefs is used to process all the parameter
 * default expressions for all the methods.  Primarily what
 * we are looking for here is default expressions which use
 * default expressions before it which require us to insert
 * a store instruction.
 */
public class CheckMethodParamDefs
  extends VisitorStep
{

  public CheckMethodParamDefs(Compiler compiler)
  {
    super(compiler);
  }

  public void run()
  {
    for (int i=0; i<compiler.classDefs.length; ++i)
      processClass(compiler.classDefs[i]);
  }

  void processClass(ClassDef cls)
  {
    MethodDef[] methods = cls.methods();
    for (int i=0; i<methods.length; ++i)
      processMethod(cls, methods[i]);
  }

  void processMethod(ClassDef cls, MethodDef m)
  {
    // unless there are two or more defaults don't bother
    int num = m.params.size();
    if (num <= 1 || ((MethodParamDef)m.params.get(num-2)).def == null)
      return;

    // ok, let's take a lookie
    MethodParamDef[] params = (MethodParamDef[])m.params.toArray(new MethodParamDef[m.params.size()]);

    // if a def expr calculates a local used after it,
    // then we need to insert a store (local set)
    for (int i=0; i<num-1; ++i)
    {
      if (params[i].def == null) continue;
      if (usedInSuccDef(params, i))
      {
        MethodParamDef param = params[i];
        MethodVar var = (MethodVar)m.vars.get(i);
        if (!param.name.equals(var.name)) throw err("invalid state", param);
        ILocation loc = param.def;
        param.def = new BinaryExpr(loc, new Token(Token.ASSIGN),
          new LocalVarExpr(loc, var), param.def);
      }
    }
  }

  boolean usedInSuccDef(MethodParamDef[] params, int index)
  {
    this.name = params[index].name;
    for (int i=index+1; i<params.length; ++i)
    {
      this.used = false;
      walk(params[i].def);
      if (this.used) return true;
    }
    return false;
  }

  public Node visit(Node node)
  {
    if (node instanceof LocalVarExpr)
    {
      LocalVarExpr local = (LocalVarExpr)node;
      if (name.equals(local.var.name)) used = true;
    }
    return node;
  }

  String name;
  boolean used;

}