//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   3 Sep 05  Brian Frank  Creation
//
package fanx.compiler.steps;

import java.io.*;
import java.util.*;
import fanx.util.*;
import fanx.compiler.*;
import fanx.compiler.Compiler;
import fanx.compiler.ast.*;
import fanx.compiler.parser.*;

/**
 * ParseString a String as abstract syntax tree compilation units
 * containing one or more ClassDefs:
 *   String -> Compiler.units & Compiler.classDefs
 */
public class ParseString
  extends CompilerStep
{

  public ParseString(Compiler compiler, String source, String filename)
  {
    super(compiler);
    this.source = source;
    this.filename = filename;
  }

  public void run()
  {
    // parse into one compilation unit
    char[] buf = source.toCharArray();
    CompilationUnit unit = new Parser(compiler, buf, filename).parse();
    compiler.units = new CompilationUnit[] { unit };
    compiler.classDefs = (ClassDef[])unit.classes.toArray(new ClassDef[unit.classes.size()]);
  }

  String source;
  String filename;

}