//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   26 Dec 05  Brian Frank  Creation
//
package fanx.compiler.steps;

import java.io.*;
import java.util.*;
import fan.sys.*;
import fanx.compiler.*;
import fanx.compiler.Compiler;
import fanx.compiler.ast.*;

/**
 * ResolveImports maps every Import node in each CompilationUnit to a pod
 * and ensures that it exists and that no imports are duplicated.
 */
public class ResolveImports
  extends CompilerStep
{

  public ResolveImports(Compiler compiler)
  {
    super(compiler);
  }

  public void run()
  {
    for (int i=0; i<compiler.units.length; ++i)
    {
      CompilationUnit unit = compiler.units[i];
      HashMap dups = new HashMap();

      for (int j=0; j<unit.imports.size(); ++j)
      {
        Import imp = (Import)unit.imports.get(j);
        String key = imp.podName;

        // check for duplicate imports
        if (dups.get(key) != null)
          throw err("Duplicate import: " + key, imp);
        dups.put(key, imp);

        // resolve the import
        resolve(imp);
      }
    }
  }

  private void resolve(Import imp)
  {
    try
    {
      imp.pod = Pod.find(imp.podName, true, null);
    }
    catch (Throwable e)
    {
      throw err("Pod not found: " + imp.podName, imp, e);
    }
  }

}