//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   26 Dec 05  Brian Frank  Creation
//
package fanx.emit;

import fan.sys.*;
import fanx.fcode.*;
import fanx.util.*;

/**
 * FPodEmit translates FPod fcode to Java bytecode as a class called <podName>.$Pod.
 * The pod class itself defines all the constants used by it's types.
 */
public class FPodEmit
  extends Emitter
  implements FConst
{

//////////////////////////////////////////////////////////////////////////
// Factory
//////////////////////////////////////////////////////////////////////////

  public static Class emitAndLoad(FPod pod)
  {
    try
    {
      Class cls;
      if (Sys.usePrecompiledOnly)
      {
        cls = Class.forName("fan." + pod.podName + ".$Pod");
      }
      else
      {
        FPodEmit emit = emit(pod);
        cls = FanClassLoader.loadClass(emit.className.replace('/', '.'), emit.classFile);
      }
      initFields(pod, cls);
      return cls;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      throw new RuntimeException(e.toString());
    }
  }

  public static FPodEmit emit(FPod pod)
  {
    FPodEmit emit = new FPodEmit(pod);
    emit.classFile = emit.emit();
    return emit;
  }

  private static void initFields(FPod pod, Class cls)
    throws Exception
  {
    for (int i=0; i<pod.ints.size(); ++i)
      cls.getField("I"+i).set(null, pod.ints.get(i));
    for (int i=0; i<pod.floats.size(); ++i)
      cls.getField("F"+i).set(null, pod.floats.get(i));
    for (int i=0; i<pod.strs.size(); ++i)
      cls.getField("S"+i).set(null, pod.strs.get(i));
    for (int i=0; i<pod.durations.size(); ++i)
      cls.getField("D"+i).set(null, pod.durations.get(i));
    for (int i=0; i<pod.uris.size(); ++i)
      cls.getField("U"+i).set(null, pod.uris.get(i));
  }

  private FPodEmit(FPod pod)
  {
    this.pod = pod;
  }

//////////////////////////////////////////////////////////////////////////
// Emit
//////////////////////////////////////////////////////////////////////////

  /**
   * Emit to bytecode classfile.
   */
  private Box emit()
  {
    init("fan/" + pod.podName + "/$Pod", "java/lang/Object", new String[0], EmitConst.PUBLIC | EmitConst.FINAL);

    // generate constant fields other types will reference, we don't
    // initialize them, rather we do that later via reflection
    for (int i=0; i<pod.ints.size(); ++i)
      emitField("I" + i, "Lfan/sys/Int;", EmitConst.PUBLIC | EmitConst.STATIC);
    for (int i=0; i<pod.floats.size(); ++i)
      emitField("F" + i, "Lfan/sys/Float;", EmitConst.PUBLIC | EmitConst.STATIC);
    for (int i=0; i<pod.strs.size(); ++i)
      emitField("S" + i, "Lfan/sys/Str;", EmitConst.PUBLIC | EmitConst.STATIC);
    for (int i=0; i<pod.durations.size(); ++i)
      emitField("D" + i, "Lfan/sys/Duration;", EmitConst.PUBLIC | EmitConst.STATIC);
    for (int i=0; i<pod.uris.size(); ++i)
      emitField("U" + i, "Lfan/sys/Uri;", EmitConst.PUBLIC | EmitConst.STATIC);

    return pack();
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  public Box classFile;
  FPod pod;

}