//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   2 Jul 06  Brian Frank  Creation
//
package fanx.fcode;

import java.io.*;
import java.util.*;
import fan.sys.*;

/**
 * FDoc is used to read/write a fandoc text file.  The fandoc file
 * format is an extremely simple plan text format with left justified
 * type/slot qnames, followed by the fandoc content indented two spaces.
 */
public class FDoc
{

  /**
   * Write a fandoc item to the specified output stream.
   */
  public static void write(PrintWriter out, String key, String doc)
  {
    if (doc == null) return;
    out.print(key);
    out.print("\n");
    out.print("  ");
    for (int i=0; i<doc.length(); ++i)
    {
      int ch = doc.charAt(i);
      if (ch == '\r') throw new IllegalStateException();
      out.print((char)ch);
      if (ch == '\n') out.print("  ");
    }
    out.print("\n\n");
  }

  /**
   * Read a fandoc file and store the doc strings.
   */
  public static void read(InputStream in)
    throws IOException
  {
    BufferedReader r = new BufferedReader(new InputStreamReader(in, "UTF-8"));
    String line;
    String key = null;
    StringBuilder s = new StringBuilder();
    while ((line = r.readLine()) != null)
    {
      if (line.startsWith("  ")) { s.append(line.substring(2)).append('\n'); continue; }
      if (line.length() == 0 && key != null)
      {
        if (key.indexOf('.') < 0)
          Type.find(key, true).doc = Str.make(s.toString());
        else
          Slot.find(key, true).doc = Str.make(s.toString());
        s = new StringBuilder();
        key = null;
      }
      else
      {
        key = line;
      }
    }
  }

}