//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   29 Jan 06  Brian Frank  Creation
//
package fanx.test;

import java.lang.reflect.*;
import fan.sys.*;

/**
 * IsAsTest
 */
public class IsAsTest
  extends CompileTest
{

//////////////////////////////////////////////////////////////////////////
// Main
//////////////////////////////////////////////////////////////////////////

  public void doRun()
    throws Exception
  {
    verifyIs();
    verifyAs();
    verifyFits();
    verifyList();
  }

//////////////////////////////////////////////////////////////////////////
// Is
//////////////////////////////////////////////////////////////////////////

  void verifyIs()
    throws Exception
  {
    verify("Bool f() { return true is Bool }",  null, Bool.True);
    verify("Bool f() { return 5 is Bool }",  null, Bool.False);
    verify("Bool f() { return type is Type}",  null, Bool.True);
    verify("Bool f() { return type.name is Type}",  null, Bool.False);
    verify("Bool f() { return type.name is Str}",  null, Bool.True);
    verify("Bool f() { return type.name.size is Int}",  null, Bool.True);
    verify("Bool f(Obj o) { return o is Int}",  bools(false), Bool.False);
    verify("Bool f(Obj o) { return o is Int}",  ints(7), Bool.True);
  }

//////////////////////////////////////////////////////////////////////////
// As
//////////////////////////////////////////////////////////////////////////

  void verifyAs()
    throws Exception
  {
    verify("Str f(Obj x) { return x as Str }", strs("foo"), Str.make("foo"));
    verify("Str f(Obj x) { return x as Str }", ints(4), null);
    verify("Str f(Obj x) { return x as Str }", new Obj[] { null }, null);
    verify("Str f(Obj x) { return x.type.method(\"toStr\").call1(x) as Str }", ints(2), Str.make("2"));
    verify("Int f(Obj x) { return x.type.method(\"toStr\").call1(x) as Int }", ints(2), null);
  }

//////////////////////////////////////////////////////////////////////////
// Fits
//////////////////////////////////////////////////////////////////////////

  void verifyFits()
    throws Exception
  {
    verify("Bool f() { return true.type.fits(Bool.type) }",  null, Bool.True);
    verify("Bool f() { return 5.type.fits(Bool.type) }",  null, Bool.False);
    verify("Bool f() { return 5ms.type.fits(Obj.type) }",  null, Bool.True);
  }

//////////////////////////////////////////////////////////////////////////
// List
//////////////////////////////////////////////////////////////////////////

  void verifyList()
    throws Exception
  {
    // is Str[,]
    verify("Bool f() { return Str[,] is Bool  }",   null, Bool.False);
    verify("Bool f() { return Str[,] is List  }",   null, Bool.True);
    verify("Bool f() { return Str[,] is Obj[] }",   null, Bool.True);
    verify("Bool f() { return Str[,] is Str[] }",   null, Bool.True);
    verify("Bool f() { return Str[,] is Int[] }",   null, Bool.False);
    verify("Bool f() { return Str[,] is Str[][] }", null, Bool.False);

    // as Str[,]
    Obj x = new List(Sys.StrType);
    Object[] a = { x };
    verify("Bool    f(Obj x) { return x as Bool  }",   a, null);
    verify("List    f(Obj x) { return x as List  }",   a, x);
    verify("Obj[]   f(Obj x) { return x as Obj[] }",   a, x);
    verify("Str[]   f(Obj x) { return x as Str[] }",   a, x);
    verify("Int[]   f(Obj x) { return x as Int[] }",   a, null);
    verify("Str[][] f(Obj x) { return x as Str[][] }", a, null);

    // is [Str[,]]
    verify("Bool f() { return [Str[,]] is Bool  }",      null, Bool.False);
    verify("Bool f() { return [Str[,]] is List  }",      null, Bool.True);
    verify("Bool f() { return [Str[,]] is List[]  }",    null, Bool.True);
    verify("Bool f() { return [Str[,]] is Str[]  }",     null, Bool.False);
    verify("Bool f() { return [Str[,]] is Str[][]  }",   null, Bool.True);
    verify("Bool f() { return [Str[,]] is Obj[][]  }",   null, Bool.True);
    verify("Bool f() { return [Str[,]] is Int[][]  }",   null, Bool.False);
    verify("Bool f() { return [Str[,]] is Str[][][] }",  null, Bool.False);

    // as [Str[,]]
    x = new List(Sys.StrType.toListOf());
    a = new Obj[] { x };
    verify("Bool      f(Obj x) { return x as Bool  }",      a, null);
    verify("List      f(Obj x) { return x as List  }",      a, x);
    verify("List[]    f(Obj x) { return x as List[]  }",    a, x);
    verify("Str[]     f(Obj x) { return x as Str[]  }",     a, null);
    verify("Str[][]   f(Obj x) { return x as Str[][]  }",   a, x);
    verify("Obj[][]   f(Obj x) { return x as Obj[][]  }",   a, x);
    verify("Int[][]   f(Obj x) { return x as Int[][]  }",   a, null);
    verify("Str[][][] f(Obj x) { return x as Str[][][] }",  a, null);
  }

}