//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//
package fanx.test;

import java.lang.reflect.*;
import java.lang.reflect.Method;
import fan.sys.*;
import fan.sys.Float;

/**
 * SlotDefTest
 */
public class SlotDefTest
  extends CompileTest
{

//////////////////////////////////////////////////////////////////////////
// Main
//////////////////////////////////////////////////////////////////////////

  public void doRun()
    throws Exception
  {
    play();
    verifySimple();
    verifyManyArgCtor();
    verifyMultipleCtors();
    verifyProtections();
    verifyClassInit();
    verifyTypeInference();
  }

//////////////////////////////////////////////////////////////////////////
// Play
//////////////////////////////////////////////////////////////////////////

  String play  =
    "class Foo\n" +
    "{\n" +
    "  static Foo f() { return make; }\n" +
    "  Int a := 6\n" +
    "}";

  public void play()
    throws Exception
  {
// TODO: don't handle calling default make()
if (true) return;
    Class cls = compileToClass(play);
    Object foo = invoke(cls, "f");
    verify(get(foo, "a"), Int.make(6));
  }

//////////////////////////////////////////////////////////////////////////
// Simple
//////////////////////////////////////////////////////////////////////////

  String simple =
    "class Foo\n" +
    "{\n" +

    "  new make(Int x)\n" +
    "  {\n" +
    "    this.x = x;\n" +
    "  }\n" +

    "  Int x\n" +
    "  Int y := 3\n" +
    "  const static Int z := 4\n" +

    "  static Foo f() { return make(2); }\n" +
    "}";

  public void verifySimple()
    throws Exception
  {
    Class cls = compileToClass(simple);
    Object o = findMethod(cls, "f").invoke(null, (Object[])null);
      verify(get(o, "x")   == Int.make(2));
      verify(get(o, "y")   == Int.make(3));
      verify(get(cls, "z") == Int.make(4));
  }

//////////////////////////////////////////////////////////////////////////
// verifyManyArgCtor
//////////////////////////////////////////////////////////////////////////

  String manyArgCtor =
    "class Foo\n" +
    "{\n" +

    "  new make(Int a, Int b, Int c, Int d, Int e, Int f, Int g)\n" +
    "  {\n" +
    "    this.a = a\n" +
    "    this.b = b\n" +
    "    this.c = c\n" +
    "    this.d = d\n" +
    "    this.e = e\n" +
    "    this.f = f\n" +
    "    this.g = g\n" +
    "  }\n" +

    "  Int a\n" +
    "  Int b\n" +
    "  Int c\n" +
    "  Int d\n" +
    "  Int e\n" +
    "  Int f\n" +
    "  Int g\n" +

    "  static Foo what() { return make(1, 2, 3, 4, 5, 6, 7); }\n" +
    "}";

  public void verifyManyArgCtor()
    throws Exception
  {
    Class cls = compileToClass(manyArgCtor);
    Object o = findMethod(cls, "what").invoke(null, (Object[])null);
      verify(get(o, "a")   == Int.make(1));
      verify(get(o, "b")   == Int.make(2));
      verify(get(o, "c")   == Int.make(3));
      verify(get(o, "d")   == Int.make(4));
      verify(get(o, "e")   == Int.make(5));
      verify(get(o, "f")   == Int.make(6));
      verify(get(o, "g")   == Int.make(7));
  }

//////////////////////////////////////////////////////////////////////////
// MultipleCtors
//////////////////////////////////////////////////////////////////////////

  String multipleCtors =
    "class Foo\n" +
    "{\n" +

    "  new make()\n" +
    "  {\n" +
    "    s2 = s2 + \" overwritten\" \n" +
    "  }\n" +

    "  new makeArgs(Int i0, Str s0)\n" +
    "  {\n" +
    "    this.i0 = i0;\n" +
    "    this.s0 = s0;\n" +
    "    s2 = s2 + \" overwritten\" \n" +
    "  }\n" +

    "  static Int zero() { return 0; }\n" +
    "  static Int one()  { return 1; }\n" +
    "  static Foo make0()                { return make }\n" +
    "  static Foo make1()                { return Foo.make }\n" +
    "  static Foo make2(Int i0, Str s0)  { return makeArgs(i0, s0); }\n" +
    "  static Foo make3(Int i0, Str s0)  { return (Foo.makeArgs(i0, s0)); }\n" +
    "  static Str makeAndGet(Str s0)     { return makeArgs(-1, s0).s0; }\n" +
    "  Int i1PreInc()  { return ++i1; }\n" +
    "  Int i1PostInc() { return i1++; }\n" +
    "  Int fooPreInc()  { return ++foo.i1; }\n" +
    "  Int fooPostInc() { return foo.i1++; }\n" +

    "  Int i0;\n" +
    "  Int i1 := 7;\n" +
    "  Float r0 := 7.0;\n" +  // auto-casts
    "  Float r1 := 7.0;\n" +
    "  Str s0;\n" +
    "  Str s1 := \"hello\";\n" +
    "  Str s2 := \"s2\";\n" +
    "  Foo foo;\n" +
    "  \n" +
    "  const static Int  si := 3\n" +
    "  const static Float sr := 7.0\n" +  // auto-cast
    "  const static Str ss\n" +      // default to null
    "  const static Int sx\n" +      // compute in static {}
    "  \n" +
    "  static\n" +
    "  {\n" +
    "    sx = si + 6\n" +
    "  }\n" +
    "}\n";

  public void verifyMultipleCtors()
    throws Exception
  {
    Class cls = compileToClass(multipleCtors);

    // static fields
    verify(get(cls, "si").equals(Int.make(3)));
    verify(get(cls, "sr").equals(Float.make(7)));
    verify(get(cls, "ss") == null);
    verify(get(cls, "sx").equals(Int.make(9)));

    // static methods
    verify(invoke(cls, "zero").equals(Int.make(0)));
    verify(invoke(cls, "one").equals(Int.make(1)));

    // instance fields
    Object x = findMethod(cls, "make").invoke(null, (Object[])null);
    Object y = findMethod(cls, "make").invoke(null, (Object[])null);
    verifyDefInit(x);
    verify(invoke(x, "i1PreInc").equals(Int.make(8))); verify(get(x, "i1").equals(Int.make(8)));
    verify(invoke(x, "i1PostInc").equals(Int.make(8))); verify(get(x, "i1").equals(Int.make(9)));
    set(x, "foo", y);
    verify(invoke(x, "fooPreInc").equals(Int.make(8)));
      verify(get(x, "i1").equals(Int.make(9)));
      verify(get(y, "i1").equals(Int.make(8)));
    verify(invoke(x, "fooPostInc").equals(Int.make(8)));
      verify(get(x, "i1").equals(Int.make(9)));
      verify(get(y, "i1").equals(Int.make(9)));

    // instance fields
    x = findMethod(cls, "makeArgs").invoke(null, new Object[] { Int.make(88), Str.make("lombardy") });
    verifyInit(x, 88, "lombardy");

    // factory method
    x = invoke(cls, "make0");
    verifyDefInit(x);
    x = invoke(cls, "make1");
    verifyDefInit(x);
    x = invoke(cls, "make2", new Obj[] { Int.make(62), Str.make("fan rocks") });
    verifyInit(x, 62, "fan rocks");
    x = invoke(cls, "make3", new Obj[] { Int.make(63), Str.make("fan really rocks") });
    verifyInit(x, 63, "fan really rocks");
    x = invoke(cls, "makeAndGet", new Obj[] { Str.make("Haley dog!") });
    verify(x.equals(Str.make("Haley dog!")));
  }

  void verifyDefInit(Object x)
    throws Exception
  {
    verify(get(x, "i0") == null);
    verify(get(x, "i1").equals(Int.make(7)));
    verify(get(x, "s0") == null);
    verify(get(x, "s1").equals(Str.make("hello")));
    verify(get(x, "s2"), Str.make("s2 overwritten"));
  }

  void verifyInit(Object x, int i0, String s0)
    throws Exception
  {
    verify(get(x, "i0").equals(Int.make(i0)));
    verify(get(x, "i1").equals(Int.make(7)));
    verify(get(x, "s0").equals(Str.make(s0)));
    verify(get(x, "s1").equals(Str.make("hello")));
    verify(get(x, "s2").equals(Str.make("s2 overwritten")));
  }

  void verifyMethodFlags(Class cls, String name, int jmod)
    throws Exception
  {
    verifyMethodFlags(cls, name, jmod, false);
  }

  void verifyMethodFlags(Class cls, String name, int jmod, boolean not)
    throws Exception
  {
    Method m = findMethod(cls, name);
    int actual = m.getModifiers();
    // System.out.println("   " + Modifier.toString(actual) + " ?= " + Modifier.toString(jmod));
    if (not)
      verify((actual & jmod) == 0);
    else
      verify((actual & jmod) != 0);
  }

//////////////////////////////////////////////////////////////////////////
// Protection Scope
//////////////////////////////////////////////////////////////////////////

  String protections =
    "class Protection\n" +
    "{\n" +
    "            Void mDefault()   {}\n" +
    "  public    Void mPublic()    {}\n" +
    "  protected Void mProtected() {}\n" +
    "  internal  Void mInternal()  {}\n" +
    "  private   Void mPrivate()   {}\n" +

    "}\n";

  public void verifyProtections()
    throws Exception
  {
    Class cls = compileToClass(protections);

    // protection scope methods
    verifyMethodFlags(cls, "mDefault",   Modifier.PUBLIC);
    verifyMethodFlags(cls, "mDefault",   Modifier.STATIC, true);
    verifyMethodFlags(cls, "mPublic",    Modifier.PUBLIC);
    verifyMethodFlags(cls, "mPublic",    Modifier.STATIC, true);
    verifyMethodFlags(cls, "mProtected", Modifier.PUBLIC); // protected+internal closures
    verifyMethodFlags(cls, "mInternal",  Modifier.PUBLIC | Modifier.PROTECTED | Modifier.PRIVATE, true);
    verifyMethodFlags(cls, "mPrivate",   Modifier.PUBLIC | Modifier.PROTECTED | Modifier.PRIVATE, true); // internal closures
  }

//////////////////////////////////////////////////////////////////////////
// Static Class Initialization
//////////////////////////////////////////////////////////////////////////

  String classInit =
    "class ClassInit\n" +
    "{\n" +

    "  const static Int a := 1 \n" +
    "  const static Int b := 2 \n" +
    "  const static Int c := a + b \n" +
    "  const static Int d := a + b \n" +
    "  static { d = -1 }\n" +
    "  const static Int e := d\n" +
    "  const static Int f\n" +
    "  static { f = e + 9 }\n" +

    "}\n";

  static int a = 1;
  static int b = 2;
  static int c = a + b;
  static int d = a + b;
  static { d = -1; }
  static int e = d;
  static int f;
  static { f = e  + 9; }

  public void verifyClassInit()
    throws Exception
  {
    Class cls = compileToClass(classInit);

    verify(get(cls, "a").equals(Int.make(a)));
    verify(get(cls, "b").equals(Int.make(b)));
    verify(get(cls, "c").equals(Int.make(c)));
    verify(get(cls, "d").equals(Int.make(d)));
    verify(get(cls, "e").equals(Int.make(e)));
    verify(get(cls, "f").equals(Int.make(f)));
  }

//////////////////////////////////////////////////////////////////////////
// TypeInference
//////////////////////////////////////////////////////////////////////////

  String typeInference =
    "class TypeInference \n" +
    "{\n" +

    // some static methods
    " static Bool b() { return true }\n" +
    " static Int  i() { return 5 }\n" +

    // basic literals
    "  const static lb := true \n" +
    "  const static li := 3 \n" +
    "  const static lr := 6.9 \n" +
    "  const static ls := \"inference rules!\" \n" +
    "  const static lt := 5ns \n" +
    "  //static lsa := [ \"a\", \"b\" ] \n" +
    "  //static lta := Duration[] \n" +

    // math expressions
    "  const static m1 := 3 * 4 \n" +
    "  const static m2 := 3.7 + 4.0 \n" +

    // call expression chains
    "  const static call0 := 0xab.toHex() \n" +
    "  const static call1 := 0xab.toHex().size() \n" +

    // my methods, my fields (doesn't work yet, type inference before bind)
    /*
    "  static methodb := b() \n" +
    "  static methodi := i() \n" +
    */

    // local var type inference
    "static Obj local0() { x := \"inferred\"; return x }\n" +
    "static Obj local1() { x := \"foo\"; y := x; return y }\n" +
    "static Obj local2() { x := ls; return x }\n" +

    "}\n";

  public void verifyTypeInference()
    throws Exception
  {
    Class cls = compileToClass(typeInference);

    // literals
    verify(get(cls, "lb"),  Bool.True);
    verify(get(cls, "li"),  Int.make(3));
    verify(get(cls, "lr"),  Float.make(6.9));
    verify(get(cls, "ls"),  Str.make("inference rules!"));
    verify(get(cls, "lt"),  Duration.make(5));
// TODO
//    verify(get(cls, "lsa"), new List(String.class).add(Str.make("a")).add(Str.make("b")));
//    verify(get(cls, "lta"), new List(Duration.class));

    // math
    verify(get(cls, "m1"), Int.make(12));
    verify(get(cls, "m2"), Float.make(7.7));

    // call chains
    verify(get(cls, "call0"), Str.make("ab"));
    verify(get(cls, "call1"), Int.make(2));

    // my methods (doesn't work, type inference before bind)
    /*
    verify(get(cls, "methodb"), new Boolean(true));
    verify(get(cls, "methodi"), Int.make(5));
    */

    // local type inference
    verify(invoke(cls, "local0"), Str.make("inferred"));
    verify(invoke(cls, "local1"), Str.make("foo"));
    verify(invoke(cls, "local2"), Str.make("inference rules!"));
  }

//////////////////////////////////////////////////////////////////////////
// TODO
//////////////////////////////////////////////////////////////////////////

  // TODO: Default constructor 'make' conflicts with another slot
}