//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   13 Sep 06  Andy Frank  Creation
//

using System;

namespace Fan.Sys
{
  ///
  /// Bool is a boolean value: true or false.
  ///
  public sealed class Bool : FanObj
  {

  //////////////////////////////////////////////////////////////////////////
  // Construction
  //////////////////////////////////////////////////////////////////////////

    public static Bool Parse(Str s) { return Parse(s, Bool.True); }
    public static Bool Parse(Str s, Bool check)
    {
      if (s.val == "true") return True;
      if (s.val == "false") return False;
      if (!check.val) return null;
      throw ArgErr.Make("Invalid Bool: '" + s + "'").val;
    }

    public static Bool Make(bool b)  { return b ? True : False; }
    public static Bool Make(int b) { return b != 0 ? True : False; }

    private Bool(bool val)
    {
      this.val = val;
      this.str = val ? Str.Make("true") : Str.Make("false");
    }

  //////////////////////////////////////////////////////////////////////////
  // Identity
  //////////////////////////////////////////////////////////////////////////

    public override bool Equals(Object obj)
    {
      return val == (obj as Bool).val;
    }

    public override Bool Equals(Obj obj)
    {
      return this == obj ? Bool.True : Bool.False;
    }

    public override int GetHashCode()
    {
      return val ? 1231 : 1237;
    }

    public override Int Hash()
    {
      return val ? Int.Make(1231) : Int.Make(1237);
    }

    public override Type Type()
    {
      return Sys.BoolType;
    }

  //////////////////////////////////////////////////////////////////////////
  // Operators
  //////////////////////////////////////////////////////////////////////////

    public Bool Not()
    {
      return val ? False : True;
    }

    public Bool Amp(Bool b)
    {
      return val & b.val ? True : False;
    }

    public Bool Pipe(Bool b)
    {
      return val | b.val ? True : False;
    }

    public Bool Caret(Bool b)
    {
      return val ^ b.val ? True : False;
    }

  //////////////////////////////////////////////////////////////////////////
  // Conversion
  //////////////////////////////////////////////////////////////////////////

    public override Str ToStr()
    {
      return str;
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    public static readonly Bool True  = new Bool(true);
    public static readonly Bool False = new Bool(false);

    public readonly bool val;
    public readonly Str str;
  }
}