//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   14 Jan 06  Andy Frank  Creation
//

using System.Collections;
using System.Text;

namespace Fan.Sys
{
  /// <summary>
  /// Depend.
  /// </summary>
  public sealed class Depend : FanObj
  {

  //////////////////////////////////////////////////////////////////////////
  // Construction
  //////////////////////////////////////////////////////////////////////////

    public static Depend Parse(Str str)
    {
      try
      {
        return new Parser(str.val).Parse();
      }
      catch (System.Exception)
      {
        throw ArgErr.Make("Invalid Depend: '" + str + "'").val;
      }
    }

    private Depend(Str name, Constraint[] constraints)
    {
      this.name = name;
      this.constraints = constraints;
    }

  //////////////////////////////////////////////////////////////////////////
  // Parser
  //////////////////////////////////////////////////////////////////////////

    class Parser
    {
      internal Parser(string str)
      {
        this.str = str;
        this.len = str.Length;
        //Consume();
      }

      internal Depend Parse()
      {
        /*
        name = name();
        constraints.add(constraint());
        while (cur == ',')
        {
          consume();
          consumeSpaces();
          constraints.add(constraint());
        }
        if (pos <= len) throw new RuntimeException();
        return new Depend(name, (Constraint[])constraints .toArray(new Constraint[constraints.size()]));
        */
        return new Depend(null, null);
      }
      /*
      private Str name()
      {
        StringBuilder s = new StringBuilder();
        while (cur != ' ')
        {
          if (cur < 0) throw new RuntimeException();
          s.append((char)cur);
          consume();
        }
        consumeSpaces();
        if (s.length() == 0) throw new RuntimeException();
        return Str.make(s.toString());
      }

      private Constraint constraint()
      {
        Constraint c = new Constraint();
        c.version = version();
        consumeSpaces();
        if (cur == '+')
        {
          c.isPlus = true;
          consume();
          consumeSpaces();
        }
        else if (cur == '-')
        {
          consume();
          consumeSpaces();
          c.endVersion = version();
          consumeSpaces();
        }
        return c;
      }

      private Version version()
      {
        List segs = new List(Sys.IntType, 4);
        int seg = consumeDigit();
        while (true)
        {
          if ('0' <= cur && cur <= '9')
          {
            seg = seg*10 + consumeDigit();
          }
          else
          {
            segs.add(Int.pos(seg));
            seg = 0;
            if (cur != '.') break;
            else consume();
          }
        }
        return new Version(segs);
      }

      private int consumeDigit()
      {
        if ('0' <= cur && cur <= '9')
        {
          int digit = cur - '0';
          consume();
          return digit;
        }
        throw new RuntimeException();
      }

      private void consumeSpaces()
      {
        while(cur == ' ') consume();
      }

      private void consume()
      {
        if (pos < len)
        {
          cur = str.charAt(pos++);
        }
        else
        {
          cur = -1;
          pos = len+1;
        }
      }
      */

      //int cur;
      //int pos;
      int len;
      string str;
      //Str name;
      ArrayList constraints = new ArrayList(4);
    }

  //////////////////////////////////////////////////////////////////////////
  // Identity
  //////////////////////////////////////////////////////////////////////////

    public override Bool Equals(Obj obj)
    {
      if (obj is Depend)
        return ToStr().Equals(obj.ToStr());
      else
        return Bool.False;
    }

    public override int GetHashCode()
    {
      return ToStr().GetHashCode();
    }

    public override Int Hash()
    {
      return ToStr().Hash();
    }

    public override Type Type()
    {
      return Sys.DependType;
    }

    public override Str ToStr()
    {
      if (str == null)
      {
        StringBuilder s = new StringBuilder();
        s.Append(name.val).Append(' ');
        for (int i=0; i<constraints.Length; i++)
        {
          if (i > 0) s.Append(',');
          Constraint c = constraints[i];
          s.Append(c.version);
          if (c.isPlus) s.Append('+');
          if (c.endVersion != null) s.Append('-').Append(c.endVersion);
        }
        str = Str.Make(s.ToString());
      }
      return str;
    }

  //////////////////////////////////////////////////////////////////////////
  // Methods
  //////////////////////////////////////////////////////////////////////////

    public Str Name()
    {
      return name;
    }

    public Int Size()
    {
      return Int.Pos(constraints.Length);
    }

    public Version Version() { return Version(Int.Zero); }
    public Version Version(Int index)
    {
      return constraints[(int)index.val].version;
    }

    public Bool IsPlus() { return IsPlus(Int.Zero); }
    public Bool IsPlus(Int index)
    {
      return constraints[(int)index.val].isPlus ? Bool.True : Bool.False;
    }

    public Bool IsRange() { return IsRange(Int.Zero); }
    public Bool IsRange(Int index)
    {
      return constraints[(int)index.val].endVersion != null ? Bool.True : Bool.False;
    }

    public Version EndVersion() { return EndVersion(Int.Zero); }
    public Version EndVersion(Int index)
    {
      return constraints[(int)index.val].endVersion;
    }

    public Bool Match(Version v)
    {
      /*
      for (int i=0; i<constraints.length; ++i)
      {
        Constraint c = constraints[i];
        if (c.isPlus)
        {
          // versionPlus
          if (c.version.compare(v).val <= 0)
            return Bool.True;
        }
        else if (c.endVersion != null)
        {
          // versionRange
          if (c.version.compare(v).val <= 0 &&
              (c.endVersion.compare(v).val >= 0 || match(c.endVersion, v)))
            return Bool.True;
        }
        else
        {
          // versionSimple
          if (match(c.version, v))
            return Bool.True;
        }
      }
      */
      return Bool.False;
    }

    private static bool Match(Version a, Version b)
    {
      //if (a.segments().sz() > b.segments().sz()) return false;
      //for (int i=0; i<a.segments().sz(); ++i)
      //  if (a.segment(i) != b.segment(i))
      //    return false;
      return true;
    }

  //////////////////////////////////////////////////////////////////////////
  // Constraint
  //////////////////////////////////////////////////////////////////////////

    class Constraint
    {
      internal Version version = null;
      internal bool isPlus = false;
      internal Version endVersion = null;
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    private readonly Str name;
    private readonly Constraint[] constraints;
    private Str str;

  }
}