//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   14 Sep 06  Andy Frank  Creation
//

using System;

namespace Fan.Sys
{
  ///
  /// FanObj is the root class of all classes in Fan - it is the class
  /// representation of Obj which manifests itself as both an interface
  /// called Obj and this class.
  ///
  public class FanObj : Obj
  {

  //////////////////////////////////////////////////////////////////////////
  // .NET
  //////////////////////////////////////////////////////////////////////////

    public override int GetHashCode()
    {
      long hash = Hash().val;
      //return (int)(hash ^ (hash >>> 32));
      return (int)(hash ^ (hash >> 32));
    }

    public override bool Equals(Object obj)
    {
      if (GetType() != obj.GetType()) return false;
      return Equals((Obj)obj).val;
    }

    public override string ToString()
    {
      return ToStr().val;
    }

  //////////////////////////////////////////////////////////////////////////
  // Identity
  //////////////////////////////////////////////////////////////////////////

    public virtual Bool Equals(Obj obj)
    {
      return this == obj ? Bool.True : Bool.False;
    }

    public virtual Int Compare(Obj obj)
    {
      return ToStr().Compare(obj.ToStr());
    }

    public virtual Int Hash()
    {
      return Int.Make(base.GetHashCode());
    }

    public virtual Str ToStr()
    {
      return Str.Make(base.ToString());
    }

    public virtual Type Type()
    {
      return Sys.ObjType;
    }

    public virtual Obj Trap(Str name, List args)
    {
      Slot slot = Type().Slot(name, Bool.True);
      if (slot is Method)
      {
        Method m = (Method)slot;
        return m.CallOn(this, args);
      }
      else
      {
        Field f = (Field)slot;
        int argSize = args.Sz();
        if (argSize == 0)
        {
          return f.Get(this);
        }

        if (argSize == 1)
        {
          Obj val = args.Get(0);
          f.Set(this, val);
          return val;
        }

        throw ArgErr.Make("Invalid number of args to get or set field '" + name + "'").val;
      }
    }

    /*
    public Obj trapUri(Uri uri)
    {
      // sanity checks
      List path = uri.path();
      if (path == null) throw ArgErr.make("Path is null: '" + uri + "'").val;

      // if path is empty, return this
      if (path.sz() == 0) return this;

      // get next level of path
      Str nextName = (Str)path.first();
      Obj obj = null;
      try
      {
        if (emptyList == null) emptyList = new List(Sys.ObjType).ro();
        obj = trap(nextName, emptyList);
      }
      catch(UnknownSlotErr.Val e)
      {
      }
      if (obj == null) return null;

      // recurse
      return obj.trapUri(uri.tail());
    }
    private static List emptyList;
    */

  //////////////////////////////////////////////////////////////////////////
  // Utils
  //////////////////////////////////////////////////////////////////////////

    public static void Echo(Obj obj)
    {
      System.Console.WriteLine(obj);
    }
  }
}