//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   27 Sep 06  Andy Frank  Creation
//

using System;
using System.Reflection;

namespace Fan.Sys
{
  /// <summary>
  /// Field is a slot which "stores" a value.
  /// </summary>
  public class Field : Slot
  {

  //////////////////////////////////////////////////////////////////////////
  // Constructor
  //////////////////////////////////////////////////////////////////////////

    public Field(Type parent, Str name, int flags, Facets facets, Type fieldType)
      : base(parent, name, flags, facets)
    {
      this.fieldType = fieldType;
    }

  //////////////////////////////////////////////////////////////////////////
  // Signature
  //////////////////////////////////////////////////////////////////////////

    public override Type Type()  { return Sys.FieldType;  }

    public Type FieldType() { return fieldType; }

    public Method Getter() { return getter; }

    public Method Setter() { return setter; }

    public override Str Signature() { return Str.Make(fieldType.ToStr().val + " " + Str.Make(name.val)); }

  //////////////////////////////////////////////////////////////////////////
  // Reflection
  //////////////////////////////////////////////////////////////////////////

    public Obj Get() { return Get(null); }
    public Obj Get(Obj instance)
    {
      if (getter != null)
      {
        return getter.Invoke(instance, (Object[])null);
      }

      try
      {
        parent.Finish();
        return (Obj)reflect.GetValue(instance);
      }
      catch (Exception e)
      {
        if (reflect == null)
          throw Err.Make("Field not mapped to java.lang.reflect correctly").val;

        throw Err.Make(e).val;
      }
    }

    public void Set(Obj instance, Obj value)
    {
      if (setter != null)
      {
        setter.Invoke(instance, new Obj[] { value });
        return;
      }

      try
      {
        parent.Finish();
        reflect.SetValue(instance, value);
      }
      catch (Exception e)
      {
        if (reflect == null)
          throw Err.Make("Field not mapped to java.lang.reflect correctly").val;

        throw Err.Make(e).val;
      }
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    internal readonly Type fieldType;
    internal Method getter;
    internal Method setter;
    internal FieldInfo reflect;

  }
}