//
// Copyright (c) 2007, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   16 Jan 07  Andy Frank  Creation
//

namespace Fan.Sys
{
  /// <summary>
  /// File represents a file or directory in a file system.
  /// <summary>
  public abstract class File : FanObj
  {

  //////////////////////////////////////////////////////////////////////////
  // Construction
  //////////////////////////////////////////////////////////////////////////

    public static File Make(Uri uri)
    {
      return null; //new LocalFile(uri);
    }

    public static File OS(Str osPath)
    {
      return null; //new LocalFile(new java.io.File(osPath.val));
    }

    public static File CreateTemp() { return CreateTemp(null, null, null); }
    public static File CreateTemp(Str prefix) { return CreateTemp(prefix, null, null); }
    public static File CreateTemp(Str prefix, Str suffix) { return CreateTemp(prefix, suffix, null); }
    public static File CreateTemp(Str prefix, Str suffix, File dir)
    {
      if (prefix == null || prefix.val.Length == 0) prefix = Str.Make("fan");
      if (prefix.val.Length == 1) prefix = Str.Make(prefix.val + "xx");
      if (prefix.val.Length == 2) prefix = Str.Make(prefix.val + "x");

      if (suffix == null) suffix = Str.Make(".tmp");

      //java.io.File d = null;
      //if (dir != null)
      //{
      //  if (!(dir instanceof LocalFile)) throw IOErr.make("Dir is not on local file system: " + dir).val;
      //  d = ((LocalFile)dir).file;
      //}

      try
      {
        return null; //new LocalFile(java.io.File.createTempFile(prefix.val, suffix.val, d));
      }
      catch (System.IO.IOException e)
      {
        throw IOErr.Make(e).val;
      }
    }

    protected File(Uri uri)
    {
      this.uri = uri;
    }

  //////////////////////////////////////////////////////////////////////////
  // Identity
  //////////////////////////////////////////////////////////////////////////

    public override Bool Equals(Obj obj)
    {
      if (obj is File)
      {
        return uri.Equals(((File)obj).uri);
      }
      return Bool.False;
    }

    public override int GetHashCode() { return uri.GetHashCode(); }

    public override Int Hash() { return uri.Hash(); }

    public override Str ToStr() { return uri.ToStr(); }

    public override Type Type() { return Sys.FileType; }

  //////////////////////////////////////////////////////////////////////////
  // Naming
  //////////////////////////////////////////////////////////////////////////

    public Uri Uri()
    {
      return uri;
    }

    public Bool IsDir()
    {
      return uri.IsDir();
    }

    public List Path()
    {
      return uri.Path();
    }

    public Str PathStr()
    {
      return uri.PathStr();
    }

    public Str Name()
    {
      return uri.Name();
    }

    public Str Basename()
    {
      return uri.Basename();
    }

    public Str Ext()
    {
      return uri.Ext();
    }

  //////////////////////////////////////////////////////////////////////////
  // Access
  //////////////////////////////////////////////////////////////////////////

    public abstract Bool Exists();

    public abstract Int Size();

    public abstract Time Modified();
    public abstract void Modified(Time time);

    public abstract Str OSPath();

    public abstract File Parent();

    public abstract List List();

    public abstract File Normalize();

    public abstract File Plus(Uri uri);

  //////////////////////////////////////////////////////////////////////////
  // Management
  //////////////////////////////////////////////////////////////////////////

    public abstract File Create();

    public abstract void Delete();

    public abstract File DeleteOnExit();

  //////////////////////////////////////////////////////////////////////////
  // IO
  //////////////////////////////////////////////////////////////////////////

    public InStream In() { return In(defaultBufSize); }
    public abstract InStream In(Int bufSize);

    public OutStream Out() { return Out(Bool.False, defaultBufSize); }
    public OutStream Out(Bool append) { return Out(append, defaultBufSize); }
    public abstract OutStream Out(Bool append, Int bufSize);

    private static readonly Int defaultBufSize = Int.Make(4096);

    //public Buf readAllBuf()
    //{
    //  return In(defaultBufSize).readAllBuf();
    //}

    //public List readAllLines()
    //{
    //  return In(defaultBufSize).readAllLines();
    //}

    //public Str readAllStr() { return readAllStr(Bool.True); }
    //public Str readAllStr(Bool normalizeNewlines)
    //{
    //  return In(defaultBufSize).readAllStr(normalizeNewlines);
    //}

    public Map ReadProps()
    {
      return In(defaultBufSize).ReadProps();
    }

    public void WriteProps(Map props)
    {
      Out(Bool.False, defaultBufSize).WriteProps(props, Bool.True);
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    readonly Uri uri;

  }
}