//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   27 Sep 06  Andy Frank  Creation
//

using System;
using System.IO;

namespace Fan.Sys
{
  /// <summary>
  /// IndexErr.
  /// </summary>
  public class IndexErr : Err
  {

  //////////////////////////////////////////////////////////////////////////
  // C# Convenience
  //////////////////////////////////////////////////////////////////////////

    public new static IndexErr Make(string msg)  { return Make(Str.Make(msg)); }
    public static IndexErr Make(Int index)       { return Make(Str.Make(index.ToString())); }
    public static IndexErr Make(Range index) { return Make(Str.Make(index.ToString())); }

  //////////////////////////////////////////////////////////////////////////
  // Fan Constructors
  //////////////////////////////////////////////////////////////////////////

    public new static IndexErr Make() { return Make((Str)null, (Err)null); }
    public new static IndexErr Make(Str msg) { return Make(msg, null); }
    public new static IndexErr Make(Str msg, Err cause)
    {
      IndexErr err = new IndexErr();
      Make_(err, msg, cause);
      return err;
    }

    public static void Make_(IndexErr self) { Make_(self, null);  }
    public static void Make_(IndexErr self, Str msg) { Make_(self, msg, null); }
    public static void Make_(IndexErr self, Str msg, Err cause) { Err.Make_(self, msg, cause); }

  //////////////////////////////////////////////////////////////////////////
  // C# Constructors
  //////////////////////////////////////////////////////////////////////////

    public IndexErr(Err.Val val) : base(val) {}
    public IndexErr() : base(new IndexErr.Val()) {}
    public IndexErr(Exception actual) : base(new IndexErr.Val(), actual) {}

  //////////////////////////////////////////////////////////////////////////
  // Identity
  //////////////////////////////////////////////////////////////////////////

    public override Type Type() { return Sys.IndexErrType; }

  //////////////////////////////////////////////////////////////////////////
  // Val - C# Exception Type
  //////////////////////////////////////////////////////////////////////////

    public new class Val : Err.Val {}

  }
}