//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   27 Sep 06  Andy Frank  Creation
//

namespace Fan.Sys
{
  /// <summary>
  /// ListType is the GenericType for Lists: Foo[] -> V = Foo
  /// </summary>
  public class ListType : GenericType
  {

  //////////////////////////////////////////////////////////////////////////
  // Constructor
  //////////////////////////////////////////////////////////////////////////

    internal ListType(Type v) : base(Sys.ListType)
    {
      this.v = v;
    }

  //////////////////////////////////////////////////////////////////////////
  // Type
  //////////////////////////////////////////////////////////////////////////

    public override Int Hash()
    {
      return Signature().Hash();
    }

    public override Bool Equals(Obj obj)
    {
      if (obj is ListType)
      {
        return v.Equals(((ListType)obj).v);
      }
      return Bool.False;
    }

    public override Type Base()
    {
      return Sys.ListType;
    }

    public override Str Signature()
    {
      if (sig == null)
      {
        sig = Str.Make(v.Signature().val + "[]");
      }
      return sig;
    }

    public override bool Is(Type type)
    {
      if (type is ListType)
      {
        ListType t = (ListType)type;
        return v.Is(t.v);
      }
      return base.Is(type);
    }

    internal override Map MakeParams()
    {
      return new Map(Sys.StrType, Sys.TypeType)
        .Set(Str.ascii['V'], v)
        .Set(Str.ascii['L'], this).RO();
    }

  //////////////////////////////////////////////////////////////////////////
  // GenericType
  //////////////////////////////////////////////////////////////////////////

    public override Type GetRawType()
    {
      return Sys.ListType;
    }

    public override bool IsGenericParameter()
    {
      return v.IsGenericParameter();
    }

    protected override Type DoParameterize(Type t)
    {
      if (t == Sys.VType) return v;
      if (t == Sys.LType) return this;
      throw new System.InvalidOperationException(t.ToString());
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    public readonly Type v;
    private Str sig;

  }
}