//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   28 Jan 07  Andy Frank  Creation
//

namespace Fan.Sys
{
  /// <summary>
  /// LocalFile represents a file or directory in the local file system.
  /// </summary>
  public class LocalFile //: File
  {
/*
  //////////////////////////////////////////////////////////////////////////
  // Construction
  //////////////////////////////////////////////////////////////////////////

    public static Uri ToUri(System.IO.File file)
    {
      string path = file.GetPath();
      path = path.replace('\\', '/');
      if (file.isDirectory())
      {
        // add trailing slash if not present
        if (path.length() == 0 || path.charAt(path.length()-1) != '/')
          path = path + "/";
      }

      // deal with Windoze drive name
      if (path.length() > 2 && path.charAt(1) == ':' && path.charAt(0) != '/')
        path = "/" + path;

      Uri uri = Uri.parse(path);
      return uri;
    }

    public LocalFile(Uri uri)
    {
      this(uri, new System.IO.File(uri.pathStr().val));
    }

    public LocalFile(System.IO.File file)
    {
      this(ToUri(file), file);
    }

    public LocalFile(Uri uri, System.IO.File file) : base(uri)
    {
      this.file = file;
      if (file.Exists())
      {
        if (file.IsDirectory())
        {
          if (!uri.isDir().val)
            throw IOErr.Make("Must use trailing slash for dir: " + uri).val;
        }
        else
        {
          if (uri.IsDir().val)
            throw IOErr.Make("Cannot use trailing slash for file: " + uri).val;
        }
      }
    }

  //////////////////////////////////////////////////////////////////////////
  // Obj
  //////////////////////////////////////////////////////////////////////////

    public Type type() { return Sys.LocalFileType; }

  //////////////////////////////////////////////////////////////////////////
  // File
  //////////////////////////////////////////////////////////////////////////

    public Bool exists()
    {
      return file.exists() ? Bool.True : Bool.False;
    }

    public Int size()
    {
      if (file.isDirectory()) return null;
      return Int.pos(file.length());
    }

    public Time modified()
    {
      return Time.java(file.lastModified());
    }

    public void modified(Time time)
    {
      file.setLastModified(time.java());
    }

    public Str osPath()
    {
      return Str.make(file.getPath());
    }

    public File parent()
    {
      Uri parent = uri.parent();
      if (parent == null) return null;
      return new LocalFile(parent);
    }

    public List list()
    {
      System.IO.File[] list = file.listFiles();
      int len = list == null ? 0 : list.length;
      List acc = new List(Sys.FileType, len);
      for (int i=0; i<len; ++i)
        acc.add(new LocalFile(list[i]));
      return acc;
    }

    public File normalize()
    {
      try
      {
        return new LocalFile(file.getCanonicalFile());
      }
      catch (java.io.IOException e)
      {
        throw IOErr.make(e).val;
      }
    }

    public File plus(Uri uri)
    {
      return make(this.uri.plus(uri));
    }

  //////////////////////////////////////////////////////////////////////////
  // File Management
  //////////////////////////////////////////////////////////////////////////

    public File create()
    {
      if (isDir().val)
        createDir();
      else
        createFile();
      return this;
    }

    private void createFile()
    {
      if (file.exists())
      {
        if (file.isDirectory())
          throw IOErr.make("Already exists as dir: " + file).val;
      }
      try
      {
        System.IO.FileOutputStream out = new System.IO.FileOutputStream(file);
        out.close();
      }
      catch (java.io.IOException e)
      {
        throw IOErr.make(e).val;
      }
    }

    private void createDir()
    {
      if (file.exists())
      {
        if (!file.isDirectory())
          throw IOErr.make("Already exists as file: " + file).val;
      }
      else
      {
        if (!file.mkdirs())
          throw IOErr.make("Cannot createDir: " + file).val;
      }
    }

    public void delete()
    {
      if (!exists().val) return;

      if (file.isDirectory())
      {
        List kids = list();
        for (int i=0; i<kids.sz(); ++i)
          ((File)kids.get(i)).delete();
      }

      if (!file.delete())
        throw IOErr.make("Cannot delete: " + file).val;
    }

    public File deleteOnExit()
    {
      file.deleteOnExit();
      return this;
    }

  //////////////////////////////////////////////////////////////////////////
  // IO
  //////////////////////////////////////////////////////////////////////////

    public InStream in(Int bufSize)
    {
      try
      {
        int bs = (int)bufSize.val;
        if (bs == 0)
          return new SysInStream(new System.IO.FileInputStream(file));
        else
          return new SysInStream(new java.io.BufferedInputStream(new System.IO.FileInputStream(file), bs));
      }
      catch (java.io.IOException e)
      {
        throw IOErr.make(e).val;
      }
    }

    public OutStream out(Bool append, Int bufSize)
    {
      try
      {
        int bs = (int)(int)bufSize.val;
        if (bs == 0)
          return new SysOutStream(new System.IO.FileOutputStream(file, append.val));
        else
          return new SysOutStream(new java.io.BufferedOutputStream(new System.IO.FileOutputStream(file, append.val), bs));
      }
      catch (java.io.IOException e)
      {
        throw IOErr.make(e).val;
      }
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    final System.IO.File file;
*/
  }
}