//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   28 Sep 06  Andy Frank  Creation
//

using System;
using System.Collections;

namespace Fan.Sys
{
  /// <summary>
  /// Map is a hashmap of key value pairs.
  /// </summary>
  public sealed class Map : FanObj
  {

  //////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////

    public static Map Make(Type type)
    {
      //return new Map((MapType)type);
      return new Map(type, null);
    }

    public Map(Type k, Type v)
    {
      //this.type = new MapType(k, v);
      this.type = k;
    }

    public Map(Type type) //MapType type)
    {
      this.type = type;
    }

  //////////////////////////////////////////////////////////////////////////
  // Identity
  //////////////////////////////////////////////////////////////////////////

    //public override Type Type()
    //{
    //  return type;
    //}

  //////////////////////////////////////////////////////////////////////////
  // Methods
  //////////////////////////////////////////////////////////////////////////

    public Bool IsEmpty()
    {
      return map.Count == 0 ? Bool.True : Bool.False;
    }

    public Int Size()
    {
      return Int.Pos(map.Count);
    }

    public Obj Get(Obj key)
    {
      return (Obj)map[key];
    }

    /*
    public final Bool containsKey(Obj key)
    {
      return Bool.make(map.containsKey(key));
    }
    */

    /*
    public final List keys()
    {
      return new List(type.k, map.keySet());
    }

    public final List values()
    {
      return new List(type.v, map.values());
    }
    */

    public Map Set(Obj key, Obj val)
    {
      Modify();
      map[key] = val;
      return this;
    }

    public Obj Remove(Obj key)
    {
      Modify();
      Obj obj = (Obj)map[key];
      map.Remove(key);
      return obj;
    }

    public void Clear()
    {
      Modify();
      map.Clear();
    }

    /*
    public Bool Equals(Obj that)
    {
      if (that instanceof Map)
      {
        return Bool.make(type.equals(that.type()).val &&
                         map.equals(((Map)that).map));
      }
      return Bool.False;
    }
    */

    public override Int Hash()
    {
      return Int.Make(map.GetHashCode());
    }

    /*
    public final Str toStr()
    {
      if (map.size() == 0) return Str.make("[:]");
      StringBuilder s = new StringBuilder(32+map.size()*32);
      s.append("[");
      boolean first = true;
      Iterator it = map.keySet().iterator();
      while (it.hasNext())
      {
        Object key = it.next();
        Object val = map.get(key);
        if (!first) s.append(", ");
        else first = false;
        s.append(key).append(':').append(val);
      }
      s.append("]");
      return Str.make(s.toString());
    }
    */

  //////////////////////////////////////////////////////////////////////////
  // Iterators
  //////////////////////////////////////////////////////////////////////////

    /*
    public final void each(Method m)
    {
      Iterator it = map.keySet().iterator();
      while (it.hasNext())
      {
        Obj key = (Obj)it.next();
        Obj val = (Obj)map.get(key);
        m.call2(val, key);
      }
    }

    public final Obj find(Method m)
    {
      Iterator it = map.keySet().iterator();
      while (it.hasNext())
      {
        Obj key = (Obj)it.next();
        Obj val = (Obj)map.get(key);
        if (m.call2(val, key) == Bool.True)
          return val;
      }
      return null;
    }

    public final Map findAll(Method m)
    {
      Map acc = new Map(type);
      Iterator it = map.keySet().iterator();
      while (it.hasNext())
      {
        Obj key = (Obj)it.next();
        Obj val = (Obj)map.get(key);
        if (m.call2(val, key) == Bool.True)
          acc.set(key, val);
      }
      return acc;
    }

    public final Map exclude(Method m)
    {
      Map acc = new Map(type);
      Iterator it = map.keySet().iterator();
      while (it.hasNext())
      {
        Obj key = (Obj)it.next();
        Obj val = (Obj)map.get(key);
        if (m.call2(val, key) == Bool.False)
          acc.set(key, val);
      }
      return acc;
    }

    public Obj reduce(Obj reduction, Method m)
    {
      Iterator it = map.keySet().iterator();
      while (it.hasNext())
      {
        Obj key = (Obj)it.next();
        Obj val = (Obj)map.get(key);
        reduction = m.call3(reduction, val, key);
      }
      return reduction;
    }

    public Map map(Map acc, Method m)
    {
      Iterator it = map.keySet().iterator();
      while (it.hasNext())
      {
        Obj key = (Obj)it.next();
        Obj val = (Obj)map.get(key);
        acc.set(key, m.call2(val, key));
      }
      return acc;
    }
    */

  //////////////////////////////////////////////////////////////////////////
  // Readonly
  //////////////////////////////////////////////////////////////////////////

    public Bool IsRW()
    {
      return isReadonly ? Bool.False : Bool.True;
    }

    public Bool IsRO()
    {
      return isReadonly ? Bool.True : Bool.False;
    }

    public Map RW()
    {
      if (!isReadonly) return this;

      Map rw = new Map(type);
      rw.map = (Hashtable)map.Clone();
      rw.isReadonly  = false;
      rw.readonlyMap = this;
      return rw;
    }

    public Map RO()
    {
      if (isReadonly) return this;
      if (readonlyMap == null)
      {
        Map ro = new Map(type);
        ro.map = map;
        ro.isReadonly = true;
        readonlyMap   = ro;
      }
      return readonlyMap;
    }

    private void Modify()
    {
      // if readonly then throw readonly exception
      if (isReadonly)
        throw ReadonlyErr.Make("Map is readonly").val;

      // if we have a cached readonlyMap, then detach
      // it so it remains immutable
      if (readonlyMap != null)
      {
        readonlyMap.map = (Hashtable)map.Clone();
        readonlyMap = null;
      }
    }


  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    //private MapType type;
    private Type type;
    private Hashtable map = new Hashtable();
    private bool isReadonly;
    private Map readonlyMap;

  }
}