//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   27 Sep 06  Andy Frank  Creation
//

using System;
using System.IO;

namespace Fan.Sys
{
  /// <summary>
  /// NullErr.
  /// </summary>
  public class NullErr : Err
  {

  //////////////////////////////////////////////////////////////////////////
  // C# Convenience
  //////////////////////////////////////////////////////////////////////////

    public new static NullErr Make(string msg)  { return Make(Str.Make(msg)); }

  //////////////////////////////////////////////////////////////////////////
  // Fan Constructors
  //////////////////////////////////////////////////////////////////////////

    public new static NullErr Make() { return Make((Str)null, (Err)null); }
    public new static NullErr Make(Str msg) { return Make(msg, null); }
    public new static NullErr Make(Str msg, Err cause)
    {
      NullErr err = new NullErr();
      Make_(err, msg, cause);
      return err;
    }

    public static void Make_(NullErr self) { Make_(self, null);  }
    public static void Make_(NullErr self, Str msg) { Make_(self, msg, null); }
    public static void Make_(NullErr self, Str msg, Err cause) { Err.Make_(self, msg, cause); }

  //////////////////////////////////////////////////////////////////////////
  // C# Constructors
  //////////////////////////////////////////////////////////////////////////

    public NullErr(Err.Val val) : base(val) {}
    public NullErr() : base(new NullErr.Val()) {}
    public NullErr(Exception actual) : base(new NullErr.Val(), actual) {}

  //////////////////////////////////////////////////////////////////////////
  // Identity
  //////////////////////////////////////////////////////////////////////////

    public override Type Type() { return Sys.NullErrType; }

  //////////////////////////////////////////////////////////////////////////
  // Val - C# Exception Type
  //////////////////////////////////////////////////////////////////////////

    public new class Val : Err.Val {}

  }
}