//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   14 Sep 06  Andy Frank  Creation
//

using System;
using System.Collections;
using System.IO;
using Fanx.Fcode;
using Fanx.Util;

namespace Fan.Sys
{
  ///
  /// Sys provides static access to the system's environment.
  ///
  public class Sys
  {

  //////////////////////////////////////////////////////////////////////////
  // Constructor
  //////////////////////////////////////////////////////////////////////////

    static void make()
    {
      // TODO
    }

  //////////////////////////////////////////////////////////////////////////
  // Environment
  //////////////////////////////////////////////////////////////////////////

    //public static List Args() { return args.RO(); }

    //public static Map Env() { return env; }

    //public static Str HostName() { return hostName; }

    //public static Str UserName() { return userName; }

    //public static fan.sys.File homeDir() { return homeDir; }

    public static void exit() { exit(Int.Zero); }
    public static void exit(Int status) { System.Environment.Exit((int)status.val); }

    /*
    public static InStream  in()  { return StdIn; }
    public static OutStream out() { return StdOut; }
    public static OutStream err() { return StdErr; }
    */

  //////////////////////////////////////////////////////////////////////////
  // Pods
  //////////////////////////////////////////////////////////////////////////

    public static Method FindMethod(Str qname) { return (Method)FindSlot(qname.val, true); }
    public static Method FindMethod(Str qname, Bool check) { return (Method)FindSlot(qname.val, check.val); }
    public static Method FindMethod(String qname, bool check) { return (Method)FindSlot(qname, check); }

    public static Field FindField(Str qname) { return (Field)FindSlot(qname.val, true); }
    public static Field FindField(Str qname, Bool check) { return (Field)FindSlot(qname.val, check.val); }
    public static Field FindField(String qname, bool check) { return (Field)FindSlot(qname, check); }

    public static Slot FindSlot(Str qname) { return FindSlot(qname.val, true); }
    public static Slot FindSlot(Str qname, Bool check) { return FindSlot(qname.val, check.val); }
    public static Slot FindSlot(String qname, bool check)
    {
      String typeName, slotName;
      try
      {
        int dot = qname.IndexOf('.');
        typeName = qname.Substring(0, dot);
        slotName = qname.Substring(dot+1);
      }
      catch (Exception)
      {
        throw Err.Make("Invalid slot qname \"" + qname + "\", use <pod>::<type>.<slot>").val;
      }
      Type type = FindType(typeName, check);
      if (type == null) return null;
      return type.Slot(slotName, check);
    }

    public static Type FindType(Str sig) { return TypeParser.Load(sig.val, true, null); }
    public static Type FindType(string sig) { return TypeParser.Load(sig, true, null); }
    public static Type FindType(Str sig, Bool check) { return TypeParser.Load(sig.val, check.val, null); }
    public static Type FindType(string sig, bool check) { return TypeParser.Load(sig, check, null); }
    public static Type FindType(string podName, string typeName, bool check)
    {
      Pod pod = FindPod(podName, check, null);
      if (pod == null) return null;
      return pod.FindType(typeName, check);
    }

    public static Pod FindPod(Str name) { return FindPod(name.val, true, null); }
    public static Pod FindPod(Str name, Bool check) { return FindPod(name.val, check.val, null); }
    public static Pod FindPod(string name, bool check, FPod fpod)
    {
      try
      {
        lock (podsByName)
        {
          Pod pod = (Pod)podsByName[name];
          if (pod == null)
          {
            // if fpod is non-null, then we are "creating" this pod in
            // memory direct from the compiler, otherwise we need to
            // find the pod zip file and load it's meta-data
            if (fpod == null) fpod = ReadFPod(name);

            // create the pod and register it
            pod = new Pod(Str.Make(name), fpod);
            podsByName[name] = pod;
          }
          return pod;
        }
      }
      catch (UnknownPodErr.Val e)
      {
        if (!check) return null;
        throw e;
      }
      catch (Exception e)
      {
        Console.WriteLine(e);
        Console.WriteLine(e.StackTrace);
        if (!check) return null;
        throw UnknownPodErr.Make(name, e).val;
      }
    }

    /*
    public static Pod LoadPod(Stream input)
    {
      FPod fpod = null;
      try
      {
        fpod = new FPod(null, null);
        fpod.readFully(new ZipInputStream(SysInStream.java(in)));
      }
      catch(Exception e)
      {
        throw Err.make(e).val;
      }

      String name = fpod.podName;
      synchronized(podsByName)
      {
        // check for duplicate pod name
        if (podsByName.get(name) != null)
          throw Err.make("Duplicate pod name: " + name).val;

        // create Pod and add to master table
        Pod pod = new Pod(fan.sys.Str.make(name), fpod);
        podsByName.put(name, pod);
        return pod;
      }
    }
    */

    public static FPod ReadFPod(string name)
    {
      // check that pod file even exists
      string podPath = PodsDir + "/" + name + ".pod";
      if (!System.IO.File.Exists(podPath)) throw UnknownPodErr.Make(name).val;

      // verify case since Windoze is case insensitive
      // TODO

      // read in the FPod tables
      FPod fpod = new FPod(name, FStore.OpenToRead(podPath));
      fpod.Read();
      return fpod;
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    private static Hashtable podsByName = new Hashtable();
    public static readonly string HomeDir;
    public static readonly string PodsDir;
    public static readonly Pod SysPod;

    // are we using the bootstrap stub versions of sys types
    public static bool isStub;

    // if true then we disable FanClassLoader and all classes
    // must be available precompiled into bytecode from the
    // system's classloader (or webapp, etc); default is false
    public static bool usePrecompiledOnly = false;  // TODO

  //////////////////////////////////////////////////////////////////////////
  // Boot
  //////////////////////////////////////////////////////////////////////////

    static Sys()
    {
      try
      {
        // get fan home directory
        HomeDir = Environment.GetEnvironmentVariable("fan_home");
        if (HomeDir == null)
          throw new Exception("Missing fan_home environment variable.");
        if (!Directory.Exists(HomeDir))
          throw new Exception("Invalid fan_home dir: " + HomeDir);
        PodsDir = HomeDir + "/lib/fan";

        // load sys pod
        SysPod = new Pod("sys");
        podsByName["sys"] = SysPod;
        try
        {
          SysPod.Load(ReadFPod("sys"));
          isStub = false;
        }
        catch (Exception e)
        {
          Console.WriteLine("WARNING: Using stub for 'sys' pod");
          Console.WriteLine(e.StackTrace);
          Sys.isStub = true;
        }

        // types
        ObjType = Builtin("Obj", null);

        // basic primitives
        NumType      = Builtin("Num",      ObjType);
        BoolType     = Builtin("Bool",     ObjType);
        DurationType = Builtin("Duration", ObjType);
        IntType      = Builtin("Int",      NumType);
        EnumType     = Builtin("Enum",     ObjType);
        FloatType    = Builtin("Float",    NumType);
        ListType     = Builtin("List",     ObjType);
        MapType      = Builtin("Map",      ObjType);
        PodType      = Builtin("Pod",      ObjType);
        RangeType    = Builtin("Range",    ObjType);
        StrType      = Builtin("Str",      ObjType);
        StrBufType   = Builtin("StrBuf",   ObjType);
        TestType     = Builtin("Test",     ObjType);
        TimeType     = Builtin("Time",     ObjType);
        TypeType     = Builtin("Type",     ObjType);
        UriType      = Builtin("Uri",      ObjType);
        VoidType     = Builtin("Void",     ObjType);

        // reflection
        SlotType        = Builtin("Slot",        ObjType);
        FieldType       = Builtin("Field",       SlotType);
        MethodType      = Builtin("Method",      SlotType);
        MethodParamType = Builtin("MethodParam", ObjType);
        FacetsType      = Builtin("Facets",      ObjType);

        // IO
        FileType        = Builtin("File",        ObjType);

        // utils
        //ProcessType     = Builtin("Process",  ObjType);
        DependType      = Builtin("Depend",   ObjType);
        VersionType     = Builtin("Version",  ObjType);

        // exceptions
        ErrType            = Builtin("Err",            ObjType);
        ArgErrType         = Builtin("ArgErr",         ErrType);
        IndexErrType       = Builtin("IndexErr",       ErrType);
        IOErrType          = Builtin("IOErr",          ErrType);
        NullErrType        = Builtin("NullErr",        ErrType);
        ReadonlyErrType    = Builtin("ReadonlyErr",    ErrType);
        UnknownPodErrType  = Builtin("UnknownPodErr",  ErrType);
        UnknownSlotErrType = Builtin("UnknownSlotErr", ErrType);
        UnknownTypeErrType = Builtin("UnknownTypeErr", ErrType);
        UnsupportedErrType = Builtin("UnsupportedErr", ErrType);

        // generic types
        GenericParameterTypes['A'] = AType = new Type(SysPod, "A", 0, null);  // A-H Params
        GenericParameterTypes['B'] = BType = new Type(SysPod, "B", 0, null);
        GenericParameterTypes['C'] = CType = new Type(SysPod, "C", 0, null);
        GenericParameterTypes['D'] = DType = new Type(SysPod, "D", 0, null);
        GenericParameterTypes['E'] = EType = new Type(SysPod, "E", 0, null);
        GenericParameterTypes['F'] = FType = new Type(SysPod, "F", 0, null);
        GenericParameterTypes['G'] = GType = new Type(SysPod, "G", 0, null);
        GenericParameterTypes['H'] = HType = new Type(SysPod, "H", 0, null);
        GenericParameterTypes['K'] = KType = new Type(SysPod, "K", 0, null);  // Key
        GenericParameterTypes['L'] = LType = new Type(SysPod, "L", 0, null);  // Variegated List
        GenericParameterTypes['M'] = MType = new Type(SysPod, "M", 0, null);  // Variegated Map
        GenericParameterTypes['R'] = RType = new Type(SysPod, "R", 0, null);  // Return
        GenericParameterTypes['V'] = VType = new Type(SysPod, "V", 0, null);  // Value

        List noMixins = new List(TypeType, 0).RO();
        for (int i=0; i<GenericParameterTypes.Length; i++)
        {
          Type gp = GenericParameterTypes[i];
          if (gp == null) continue;
          gp.baseType = ObjType;
          gp.mixins = noMixins;
        }
      }
      catch(Exception e)
      {
        Console.WriteLine("ERROR: Sys.static");
        Console.WriteLine(e.ToString());
        throw e;
      }
    }

  //////////////////////////////////////////////////////////////////////////
  // Built-in Types
  //////////////////////////////////////////////////////////////////////////

    // the Eve of all types
    public static readonly Type ObjType;

    // basic primitives
    public static readonly Type NumType;
    public static readonly Type BoolType;
    public static readonly Type DurationType;
    public static readonly Type IntType;
    public static readonly Type EnumType;
    public static readonly Type FloatType;
    public static readonly Type ListType;
    public static readonly Type MapType;
    public static readonly Type PodType;
    public static readonly Type RangeType;
    public static readonly Type StrType;
    public static readonly Type StrBufType;
    public static readonly Type TestType;
    public static readonly Type TimeType;
    public static readonly Type TypeType;
    public static readonly Type UriType;
    public static readonly Type VoidType;

    // reflection
    public static readonly Type SlotType;
    public static readonly Type FieldType;
    public static readonly Type MethodType;
    public static readonly Type MethodParamType;
    public static readonly Type FacetsType;

    // IO
    public static readonly Type FileType;

    // utils
    //public static readonly Type ProcessType;
    public static readonly Type DependType;
    public static readonly Type VersionType;

    // exceptions
    public static readonly Type ErrType;
    public static readonly Type ArgErrType;
    public static readonly Type IndexErrType;
    public static readonly Type IOErrType;
    public static readonly Type NullErrType;
    public static readonly Type ReadonlyErrType;
    public static readonly Type UnknownPodErrType;
    public static readonly Type UnknownSlotErrType;
    public static readonly Type UnknownTypeErrType;
    public static readonly Type UnsupportedErrType;

    // generic parameter types used with generic types List, Map, and Method
    public static readonly Type[] GenericParameterTypes = new Type[256];
    public static readonly Type AType, BType, CType, DType, EType, FType, GType,
                             HType, KType, LType, MType, RType, VType;

    public static Type GenericParameterType(string name)
    {
      if (name.Length == 1 && name[0] < GenericParameterTypes.Length)
        return GenericParameterTypes[name[0]];
      else
        return null;
    }

    static Type Builtin(string name, Type baseType)
    {
      return SysPod.FindType(name, true);
      //if (SysPod.fpod != null) return SysPod.FindType(name, true);
      //return SysPod.Stub(Type.Stub(SysPod, name, baseType));
    }
  }
}