//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   4 Oct 06  Andy Frank  Creation
//

using System.Collections;
using Fan.Sys;
using Fanx.Fcode;

namespace Fanx.Emit
{
  /// <summary>
  /// FClassEmit emits a normal class type.
  /// </summary>
  public class FClassEmit : FTypeEmit
  {

  //////////////////////////////////////////////////////////////////////////
  // Constructor
  //////////////////////////////////////////////////////////////////////////

    public FClassEmit(Emitter emitter, Type parent, FType type)
      : base(emitter, parent, type) {}

  //////////////////////////////////////////////////////////////////////////
  // Overrides
  //////////////////////////////////////////////////////////////////////////

    protected override string Base()
    {
      // if the base is a generic instance, then this must be a closure
      // method type (since we can't subclass List or Map).  We subclass
      // from one of the canned Method.Indirect inner classes.
      FTypeRef refer = pod.TypeRef(type.baset);
      if (refer.IsGenericInstance())
      {
        this.methodType = (MethodType)Sys.FindType(refer.sig, true);
        int paramCount = methodType.pars.Length;
        if (paramCount  > Method.MaxIndirectParams)
          return "Fan.Sys.Method/IndirectX";
        else
          return "Fan.Sys.Method/Indirect" + paramCount;
      }
      else
      {
        string baset = Nname(type.baset);
        if (baset == "Fan.Sys.Obj") return "Fan.Sys.FanObj";
        return baset;
      }
    }

    protected override void EmitType()
    {
      emitter.EmitClass(Base(), className, PERWAPI.TypeAttr.Public);

      // generate private static Type $Type; set in clinit
      typeField = emitter.classDef.AddField(
        PERWAPI.FieldAttr.Private | PERWAPI.FieldAttr.Static,
        "$Type", emitter.FindType("Fan.Sys.Type"));

      // generate Type() instance method
      PERWAPI.MethodDef m = emitter.classDef.AddMethod(
        PERWAPI.MethAttr.Public | PERWAPI.MethAttr.Virtual, PERWAPI.ImplAttr.IL,
        "Type", emitter.FindType("Fan.Sys.Type"), new PERWAPI.Param[0]);
      m.AddCallConv(PERWAPI.CallConv.Instance);
      emitter.AddToMethodMap(className, "Type", new string[0], m);

      PERWAPI.CILInstructions code = m.CreateCodeBuffer();
      code.FieldInst(PERWAPI.FieldOp.ldsfld, typeField);
      code.Inst(PERWAPI.Op.ret);

      /*
      // generate peer field if native
      if (isNative)
        peerField = emitField("peer", "L" + className + "Peer;", EmitConst.PUBLIC);
      */

      // Create ctor emit objects first so we can reference them
      // .ctor
      ctor = emitter.classDef.AddMethod(
        PERWAPI.MethAttr.Public | PERWAPI.MethAttr.HideBySig | PERWAPI.MethAttr.SpecialRTSpecialName,
        PERWAPI.ImplAttr.IL, ".ctor", emitter.FindType("System.Void"), new PERWAPI.Param[0]);
      ctor.AddCallConv(PERWAPI.CallConv.Instance);
      emitter.AddToMethodMap(selfName, ".ctor", new string[0], ctor);
      // .cctor
      cctor = emitter.classDef.AddMethod(
        PERWAPI.MethAttr.Private | PERWAPI.MethAttr.Static | PERWAPI.MethAttr.HideBySig | PERWAPI.MethAttr.SpecialRTSpecialName,
        PERWAPI.ImplAttr.IL, ".cctor", emitter.FindType("System.Void"), new PERWAPI.Param[0]);
      emitter.AddToMethodMap(selfName, ".cctor", new string[0], cctor);
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

  }
}