//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   27 Sep 06  Andy Frank  Creation
//

using System;

namespace Fanx.Fcode
{
  /// <summary>
  /// FMethod is the read/write fcode representation of sys::Method.
  /// </summary>
  public class FMethod : FSlot
  {

    public FMethodVar[] pars()
    {
      FMethodVar[] temp = new FMethodVar[paramCount];
      Array.Copy(vars, temp, paramCount);
      return temp;
    }

    public int MaxLocals()
    {
      int max = paramCount + localCount;
      if ((flags & FConst.Static) == 0) max++;
      return max;
    }

    public FMethod Read(FStore.Input input)
    {
      base.ReadCommon(input);
      ret = input.U2();
      inheritedRet = input.U2();
      paramCount = input.U1();
      localCount = input.U1();
      vars = new FMethodVar[paramCount+localCount];
      for (int i=0; i<vars.Length; i++)
        vars[i] = new FMethodVar().Read(input);
      code = FBuf.Read(input);
      base.ReadFacets(input);
      return this;
    }

    public int ret;             // type qname index
    public int inheritedRet;    // type qname index
    public FMethodVar[] vars;   // parameters and local variables
    public int paramCount;      // number of params in vars
    public int localCount;      // number of locals in vars
    public FBuf code;           // method executable code

  }
}