//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   12 Sep 06  Andy Frank  Creation
//

using System;
using System.IO;
using Fan.Sys;

namespace Fanx.Fcode
{
  ///
  /// FType is the read fcode representation of sys::Type.
  ///
  public class FType
  {
  //////////////////////////////////////////////////////////////////////////
  // Constructor
  //////////////////////////////////////////////////////////////////////////

    public FType(FPod pod)
    {
      this.pod = pod;
      this.facets = FFacet.None;
    }

  //////////////////////////////////////////////////////////////////////////
  // Lookup
  //////////////////////////////////////////////////////////////////////////

    public FSlot Slot(string name)
    {
      for (int i=0; i<fields.Length; i++)
        if (pod.Name(fields[i].name) == name)
          return fields[i];
      for (int i=0; i<methods.Length; i++)
        if (pod.Name(methods[i].name) == name)
          return methods[i];
      throw UnknownSlotErr.Make(name).val;
    }

    public FFacet Facet(string name)
    {
      return FFacet.Facet(pod, facets, name);
    }

  //////////////////////////////////////////////////////////////////////////
  // Meta IO
  //////////////////////////////////////////////////////////////////////////

    public FType ReadMeta(FStore.Input input)
    {
      selft  = input.U2();
      baset  = input.U2();
      mixins = new int[input.U2()];
      for (int i=0; i<mixins.Length; i++) mixins[i] = input.U2();
      flags  = input.U4();
      return this;
    }

  //////////////////////////////////////////////////////////////////////////
  // Body IO
  //////////////////////////////////////////////////////////////////////////

    public string Filename()
    {
      return pod.Name(pod.TypeRef(selft).typeName) + ".fcode";
    }

    public void Read()
    {
      Read(pod.store.Read(Filename()));
    }

    public void Read(FStore.Input input)
    {
// TODO
int fieldCount = input.U2();
if (fieldCount == 0x0FC0)
{
  input.U2();
  input.U4();
  fieldCount = input.U2();
}
fields = new FField[fieldCount];

//      fields = new FField[input.U2()];
      for (int i=0; i<fields.Length; i++)
        fields[i] = new FField().Read(input);

      methods = new FMethod[input.U2()];
      for (int i=0; i<methods.Length; i++)
        methods[i] = new FMethod().Read(input);

      facets = new FFacet[input.U2()];
      for (int i=0; i<facets.Length; i++)
        facets[i] = new FFacet().Read(input);

      hollow = false;
      input.Close();
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    public bool hollow = true;  // have we only read meta-data
    public FPod pod;            // parent pod
    public int selft;           // self typeRef index
    public int flags;           // bitmask
    public int baset;           // base typeRef index
    public int[] mixins;        // mixin TypeRef indexes
    public FField[] fields;     // fields
    public FMethod[] methods;   // methods
    public FFacet[] facets;     // facets on type itself
  }
}