//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   15 Sep 05  Brian Frank  Creation
//

using Fan.Sys;

namespace Fanx.Test
{
  /// <summary>
  /// LiteralExprTest
  /// </summary>
  public class LiteralExprTest : CompileTest
  {

  //////////////////////////////////////////////////////////////////////////
  // Main
  //////////////////////////////////////////////////////////////////////////

    public override void Run()
    {
      VerifyBoolLiterals();
      VerifyIntLiterals();
      VerifyCharLiterals();
      VerifyFloatLiterals();
      VerifyStrLiterals();
      VerifyDurationLiterals();
      VerifyUriLiterals();
      VerifyTypeLiterals();
    }

  //////////////////////////////////////////////////////////////////////////
  // Boolean Literals
  //////////////////////////////////////////////////////////////////////////

    void VerifyBoolLiterals()
    {
      Verify("Bool f() { return true }",  Bool.True);
      Verify("Bool f() { return false }", Bool.False);
      Verify("Bool f() { return null; }", null);
    }

  //////////////////////////////////////////////////////////////////////////
  // Int Literals
  //////////////////////////////////////////////////////////////////////////

    void VerifyIntLiterals()
    {
      VerifyIntLiteral(System.UInt32.MinValue);
      VerifyIntLiteral(System.UInt32.MinValue+1);
      VerifyIntLiteral(System.UInt32.MinValue+2);
      VerifyIntLiteral(-32769);
      VerifyIntLiteral(-32768);
      VerifyIntLiteral(-32767);
      VerifyIntLiteral(-32766);
      VerifyIntLiteral(-129);
      VerifyIntLiteral(-128);
      VerifyIntLiteral(-127);
      VerifyIntLiteral(-126);
      VerifyIntLiteral(-100);
      VerifyIntLiteral(-2);
      VerifyIntLiteral(-1);
      VerifyIntLiteral(0);
      VerifyIntLiteral(1);
      VerifyIntLiteral(2);
      VerifyIntLiteral(3);
      VerifyIntLiteral(4);
      VerifyIntLiteral(5);
      VerifyIntLiteral(6);
      VerifyIntLiteral(100);
      VerifyIntLiteral(126);
      VerifyIntLiteral(127);
      VerifyIntLiteral(128);
      VerifyIntLiteral(129);
      VerifyIntLiteral(254);
      VerifyIntLiteral(255);
      VerifyIntLiteral(256);
      VerifyIntLiteral(257);
      VerifyIntLiteral(32766);
      VerifyIntLiteral(32767);
      VerifyIntLiteral(32768);
      VerifyIntLiteral(32769);
      VerifyIntLiteral(65535);
      VerifyIntLiteral(65536);
      VerifyIntLiteral(65537);
      VerifyIntLiteral(65538);
      VerifyIntLiteral(123456);
      VerifyIntLiteral(1234567);
      VerifyIntLiteral(System.UInt32.MaxValue-2);
      VerifyIntLiteral(System.UInt32.MaxValue-1);
      VerifyIntLiteral(System.UInt32.MaxValue);
      //VerifyIntLiteral(0xabcd0123fedc4567L);
      //VerifyIntLiteral(Long.MIN_VALUE);
      //VerifyIntLiteral(Long.MAX_VALUE);
      Verify("Int f() { return null; }", null);
    }

    void VerifyIntLiteral(long val)
    {
      Verify("Int f() { return " + val + "; }", Int.Make(val));
      //Verify("static Int f() { return 0x" + Long.toHexString(val) + "; }", Int.make(val));
    }

  //////////////////////////////////////////////////////////////////////////
  // Char Literals
  //////////////////////////////////////////////////////////////////////////

    void VerifyCharLiterals()
    {
      Verify("Int f() { return 'a'; }",      Int.Make('a'));
      Verify("Int f() { return '_'; }",      Int.Make('_'));
      Verify("Int f() { return '\n'; }",     Int.Make('\n'));
      Verify("Int f() { return '\0'; }",     Int.Make('\0'));
      Verify("Int f() { return '\''; }",     Int.Make('\''));
      Verify("Int f() { return '\uabcd'; }", Int.Make('\uabcd'));
    }

  //////////////////////////////////////////////////////////////////////////
  // Float Literals
  //////////////////////////////////////////////////////////////////////////

    void VerifyFloatLiterals()
    {
      Verify("Float f() { return 0.0; }",       Float.Make(0));
      Verify("Float f() { return 2.0; }",       Float.Make(2));
      Verify("Float f() { return +1.0; }",      Float.Make(1));
      Verify("Float f() { return -1.0; }",      Float.Make(-1));
      Verify("Float f() { return 0.005; }",     Float.Make(0.005));
      Verify("Float f() { return -1000.003; }", Float.Make(-1000.003));
      Verify("Float f() { return -4.5e33; }",   Float.Make(-4.5e33));
      Verify("Float f() { return null; }",      null);
    }

  //////////////////////////////////////////////////////////////////////////
  // Str Literals
  //////////////////////////////////////////////////////////////////////////

    void VerifyStrLiterals()
    {
      Verify("Str f() { return \"a\"; }", Str.Make("a"));
      Verify("Str f() { return \"hello world\"; }", Str.Make("hello world"));
      Verify("Str f() { return null; }", null);
    }

  //////////////////////////////////////////////////////////////////////////
  // Duration Literals
  //////////////////////////////////////////////////////////////////////////

    void VerifyDurationLiterals()
    {
      long ms = 1000L*1000L;
      long sec = 1000L*ms;
      long min = 60L*sec;
      long hr  = 60L*min;
      Verify("Duration f() { return 0ns; }",      Duration.Make(0));
      Verify("Duration f() { return 1ns; }",      Duration.Make(1));
      Verify("Duration f() { return -1ns; }",     Duration.Make(-1));
      Verify("Duration f() { return 1_999ns; }",  Duration.Make(1999));
      Verify("Duration f() { return -2ms; }",     Duration.Make(-2*ms));
      Verify("Duration f() { return 0.5sec; }",   Duration.Make(sec/2L));
      Verify("Duration f() { return 24hr; }",     Duration.Make(24L*hr));
    }

  //////////////////////////////////////////////////////////////////////////
  // Uri Literals
  //////////////////////////////////////////////////////////////////////////

    void VerifyUriLiterals()
    {
      Verify("Uri f() { return `http://www.google.com`; }", Uri.Parse("http://www.google.com"));
      Verify("Uri f() { return null; }", null);
    }

  //////////////////////////////////////////////////////////////////////////
  // Type Literals
  //////////////////////////////////////////////////////////////////////////

    void VerifyTypeLiterals()
    {
      // verify basic sys types
      Verify("Type f() { return Bool.type }",  Sys.BoolType);
      Verify("Type f() { return Int.type }",   Sys.IntType);
      Verify("Type f() { return Str.type; }",   Sys.StrType);
      Verify("Type f() { return Type.type }",  Sys.TypeType);

      // verify variegated list literal
      //verify("static Type f() { return Str[].type }",  Sys.StrType.toListOf());
      //verify("static Type f() { return Int[][].type }",  Sys.IntType.toListOf().toListOf());

      // verify non-sys non-variegated type
      //Type t = CompileToType("class HappyDays { static Type f() { return HappyDays.type } }");
      //verify(t.method("f", true).call(null) == t);

      // verify multiple usages of same type literal
      //t = CompileToType(
      //  "class Coolio {\n" +
      //  "  static Type f() { return Coolio.type }\n" +
      //  "  static Type g() { return Coolio.type }\n" +
      //  "  static Type h() { return Coolio[].type }\n" +
      //  "  static Type i() { return Coolio[].type }\n" +
      //  "  static Type j() { return Obj.type }\n" +
      //  "}");
      //verify(t.method("f", true).call(null) == t);
      //verify(t.method("g", true).call(null) == t);
      //verify(t.method("h", true).call(null) == t.toListOf());
      //verify(t.method("i", true).call(null) == t.toListOf());
      //verify(t.method("j", true).call(null) == Sys.ObjType);
    }

  }
}