//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   4 Jan 06  Andy Frank  Creation
//

using System.Collections;
using Fan.Sys;

namespace Fanx.Tools
{
  /// <summary>
  /// Fant .NET runtime.
  /// </summary>
  ///
  public class Fant : Tool
  {

  //////////////////////////////////////////////////////////////////////////
  // Test
  //////////////////////////////////////////////////////////////////////////

    public int Test(string[] patterns, bool verbose)
    {
      long t1 = System.Environment.TickCount;
      for (int i=0; i<patterns.Length; i++)
        Test(patterns[i], verbose);
      long t2 = System.Environment.TickCount;

      System.Console.WriteLine();
      System.Console.WriteLine("Time: " + (t2-t1) + "ms");
      System.Console.WriteLine();
      System.Console.WriteLine("***");
      System.Console.WriteLine("*** " + (failures == 0 ? "All tests passed!" : failures + " FAILURES") + " [" + testCount + " tests, " + methodCount  + " methods, " + totalVerifyCount + " verifies] ");
      System.Console.WriteLine("***");
      return failures;
    }

    public void Test(string pattern, bool verbose)
    {
      // TODO
      /*
      if (pattern == "*")
      {
        List pods = Pod.List();
        for (int i=0; i<pods.Sz(); i++)
        {
          Pod pod = (Pod)pods.Get(i);
          Test(pod.Name().val, verbose);
        }
        return;
      }
      */

      // TODO
      /*
      if (pattern == "sys" || pattern.StartsWith("sys::"))
      {
        WriteLine("");
        if (pattern == "sys")
          pattern = "";
        else
          pattern = pattern.Substring(5);
        if (!Fanx.Test.Test.Test(pattern)) failures++;
        totalVerifyCount += Fanx.Test.Test.totalVerified;
        return;
      }
      */

      string[] s = pattern.Split(new char[] { ':', '.' });
      string podName = s[0];
      string testName = (s.Length > 2) ? s[2] : "*";
      string methodName = (s.Length > 3) ? s[3] : "*";

      WriteLine("");

      Pod pod = Pod.Find(podName, true, null);
      Type[] tests = Tests(pod, testName);
      for (int i=0; i<tests.Length; i++)
      {
        Method[] methods = Methods(tests[i], methodName);
        for (int j=0; j<methods.Length; j++)
        {
          WriteLine("-- Run:  " + methods[j] + "...");
          int verifyCount = RunTest(tests[i], methods[j]);
          if (verifyCount < 0) { failures++; continue; }
          WriteLine("   Pass: " + methods[j] + " [" + verifyCount + "]");
          methodCount++;
          totalVerifyCount += verifyCount;
        }
        testCount++;
      }
    }

    private Type[] Tests(Pod pod, string testName)
    {
      // named test
      if (testName != "*") return new Type[] { pod.FindType(testName, true) };

      // all types which subclass Test
      List all = pod.Types();
      ArrayList acc = new ArrayList();
      for (int i=0; i<all.Sz(); i++)
      {
        Type x = (Type)all.Get(i);
        // TODO - is?
        //if (x.Is(Sys.TestType) && !x.IsAbstract().val) acc.Add(x);
        acc.Add(x);
      }
      return (Type[])acc.ToArray(System.Type.GetType("Fan.Sys.Type"));
    }

    private Method[] Methods(Type type, string methodName)
    {
      // named test
      if (methodName != "*") return new Method[] { type.Method(methodName, true) };

      // all methods which start with "test"
      Method[] all = type.MethodArr();
      ArrayList acc = new ArrayList();
      for (int i=0; i<all.Length; i++)
        if (all[i].Name().val.StartsWith("test") && !all[i].IsAbstract().val) acc.Add(all[i]);
      return (Method[])acc.ToArray(System.Type.GetType("Fan.Sys.Method"));
    }

    private int RunTest(Type type, Method method)
    {
      Method setup    = type.Method("setup", true);
      Method teardown = type.Method("teardown", true);

      try
      {
        Fan.Sys.Test test = (Fan.Sys.Test)type.Make();
        List args = new List(Sys.ObjType, new Obj[] {test});
        setup.Call(args);
        method.Call(args);
        teardown.Call(args);
        return test.verifyCount;
      }
      catch (System.Exception e)
      {
        System.Console.WriteLine();
        System.Console.WriteLine("TEST FAILED");
        if (e is Err.Val)
          ((Err.Val)e).Err().Trace();
        else
        {
          System.Console.WriteLine(e);
          System.Console.WriteLine(e.StackTrace);
        }
        return -1;
      }
    }

  //////////////////////////////////////////////////////////////////////////
  // Tool Methods
  //////////////////////////////////////////////////////////////////////////

    public override int Run() //string[] args)
    {
      try
      {
        //Fanx.Test.Test.RunTests(null);
        //Test(new string[] { "sysTest::BoolTest.testEquals" }, true);
        Test(new string[] { "test::StrTest" }, true);

        //Type type = Type.Find("sysTest::BoolTest");
        //Type type = Type.Find("test::BoolTest");
        //Method[] m = Methods(type, "*");
        //for (int i=0; i<m.Length; i++)
        //  System.Console.WriteLine("[" + i + "] " + m[i]);
        //  RunTest(type, m[i]);
      }
      catch (System.Exception e)
      {
        System.Console.WriteLine(e);
        System.Console.WriteLine(e.StackTrace);
        return -1;
      }

      return 0;

      // TODO
      //Help();
      //return -1;
    }

    void Help()
    {
      WriteLine("Fan Test");
      WriteLine("Usage:");
      WriteLine("  fant [options] <pod> [pod]*");
      WriteLine("  fant [options] <pod>::<test>");
      WriteLine("  fant [options] <pod>::<test>.<method>");
      WriteLine("Options:");
      WriteLine("  -help, -h, -?  print usage help");
      WriteLine("  -v             verbose mode");
    }

    void WriteLine(string s)
    {
      System.Console.WriteLine(s);
    }

  //////////////////////////////////////////////////////////////////////////
  // Fields
  //////////////////////////////////////////////////////////////////////////

    int testCount = 0;
    int methodCount = 0;
    int totalVerifyCount = 0;
    int failures = 0;
  }
}
