//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   4 Jan 06  Andy Frank  Creation
//

using System;
using System.IO;
using System.Reflection;

namespace Fanx.Tools
{
  public abstract class Tool
  {

  //////////////////////////////////////////////////////////////////////////
  // Overrides
  //////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Run this tool.
    /// </summary>
    public abstract int Run(); //string[] args);

  //////////////////////////////////////////////////////////////////////////
  // Fan Assembly Loading
  //////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Register the assembly resolver.
    /// </summary>
    protected Tool()
    {
      //AppDomain domain = AppDomain.CurrentDomain;
      //domain.AssemblyResolve += new ResolveEventHandler(ResolveAssembly);
    }

    /// <summary>
    /// Get the assembly name from the specified fullName.  The fullName
    /// has the following format:
    ///
    /// <name>, Version=<major.minor.build.patch>, Culture=<...>, PublicKeyToken=<...>
    ///
    /// </summary>
    public static string GetAssemblyName(string fullName)
    {
      int sep = fullName.IndexOf(',');
      if (sep == -1)
        return fullName;
      else
        return fullName.Substring(0, sep);
    }

    /// <summary>
    /// Resolve an assembly that cannot be found in
    /// the current AppDomain.
    /// </summary>
    public static Assembly ResolveAssembly(object sender, ResolveEventArgs args)
    {
      string asmName = GetAssemblyName(args.Name);
      AppDomain domain = AppDomain.CurrentDomain;
      string homeDir = Environment.GetEnvironmentVariable("fan_home");
      if (homeDir == null)
        throw new Exception("Missing fan_home environment variable.");
      if (!Directory.Exists(homeDir))
        throw new Exception("Invalid fan_home directory: " + homeDir);

      string libDir = Path.Combine(homeDir, "lib/net");
      string dll = Path.Combine(libDir, asmName + ".dll");

      FileInfo f = new FileInfo(dll);
      if (!f.Exists) return null;
      BinaryReader fIn = new BinaryReader(f.OpenRead());
      byte[] asm = fIn.ReadBytes((int)f.Length);
      fIn.Close();
      if (asm.Length != f.Length)
        throw new Exception("Could not read " + dll + ": " + asm.Length + " != " + f.Length);

      Assembly result = domain.Load(asm);
      return result;
    }

  }
}