//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   31 Oct 06  Andy Frank  Creation
//

using Object = System.Object;
using Fan.Sys;

namespace Fanx.Util
{
  /// <summary>
  /// OpUtil provides static methods used to implement special fcode opcodes.
  /// </summary>
  public class OpUtil
  {

  //////////////////////////////////////////////////////////////////////////
  // Comparisions -> boolean
  //////////////////////////////////////////////////////////////////////////

    public static bool CompareEQz(Obj a, Obj b)
    {
      if (a == null) return (b == null);
      if (b == null) return false;
      return a.Equals(b).val;
    }

    public static bool CompareNEz(Obj a, Obj b)
    {
      if (a == null) return (b != null);
      if (b == null) return true;
      return !a.Equals(b).val;
    }

    public static bool CompareLTz(Obj a, Obj b)
    {
      if (a == null) return (b != null);
      if (b == null) return false;
      return a.Compare(b).val < 0;
    }

    public static bool CompareLEz(Obj a, Obj b)
    {
      if (a == null) return true;
      if (b == null) return false;
      return a.Compare(b).val <= 0;
    }

    public static bool CompareGEz(Obj a, Obj b)
    {
      if (a == null) return (b == null);
      if (b == null) return true;
      return a.Compare(b).val >= 0;
    }

    public static bool CompareGTz(Obj a, Obj b)
    {
      if (a == null) return false;
      if (b == null) return true;
      return a.Compare(b).val > 0;
    }

  //////////////////////////////////////////////////////////////////////////
  // Comparisions -> Bool
  //////////////////////////////////////////////////////////////////////////

    public static Bool CompareEQ(Obj a, Obj b)
    {
      if (a == null) return (b == null) ? Bool.True : Bool.False;
      if (b == null) return Bool.False;
      return a.Equals(b);
    }

    public static Bool CompareNE(Obj a, Obj b)
    {
      if (a == null) return (b == null) ? Bool.False : Bool.True;
      if (b == null) return Bool.True;
      return a.Equals(b).Not();
    }

    public static Int Compare(Obj a, Obj b)
    {
      if (a == null) return (b == null) ? Int.EQ : Int.LT;
      if (b == null) return Int.GT;
      return a.Compare(b);
    }

    public static Bool CompareLT(Obj a, Obj b)
    {
      if (a == null) return (b == null) ? Bool.False : Bool.True;
      if (b == null) return Bool.False;
      return a.Compare(b).val < 0 ? Bool.True : Bool.False;
    }

    public static Bool CompareLE(Obj a, Obj b)
    {
      if (a == null) return Bool.True;
      if (b == null) return Bool.False;
      return a.Compare(b).val <= 0 ? Bool.True : Bool.False;
    }

    public static Bool CompareGE(Obj a, Obj b)
    {
      if (a == null) return (b == null) ? Bool.True : Bool.False;
      if (b == null) return Bool.True;
      return a.Compare(b).val >= 0 ? Bool.True : Bool.False;
    }

    public static Bool CompareGT(Obj a, Obj b)
    {
      if (a == null) return Bool.False;
      if (b == null) return Bool.True;
      return a.Compare(b).val > 0 ? Bool.True : Bool.False;
    }

    public static Bool CompareSame(Object a, Object b) // need to use Object for mixins
    {
      return a == b ? Bool.True : Bool.False;
    }

    public static Bool CompareNotSame(Object a, Object b) // need to use Object for mixins
    {
      return a != b ? Bool.True : Bool.False;
    }

    public static Bool CompareNull(Object a) // need to use Object for mixins
    {
      return a == null ? Bool.True : Bool.False;
    }

    public static Bool CompareNotNull(Object a) // need to use Object for mixins
    {
      return a != null ? Bool.True : Bool.False;
    }

  //////////////////////////////////////////////////////////////////////////
  // Is/As
  //////////////////////////////////////////////////////////////////////////

    /*
    public static Bool Is(Obj instance, Type type)
    {
      if (instance == null) return Bool.False;
      return instance.Type().Fits(type);
    }

    public static Obj As(Obj instance, Type type)
    {
      if (instance == null) return null;
      return instance.Type().Is(type) ? instance : null;
    }
    */

  }
}