//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   11 Dec 05  Brian Frank  Creation
//

**
** Duration represents a relative duration of time with nanosecond precision.
**
const final class Duration
{

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  **
  ** Get the current value of the system timer.  This method returns
  ** a relative time unrelated to system or wall-clock time.  Typically
  ** it is the number of nanosecond ticks which have elapsed since system
  ** startup.
  **
  static Duration now()

  **
  ** Create a Duration which represents the specified number of nanosecond ticks.
  **
  static Duration make(Int ticks)

  **
  ** Private constructor.
  **
  private new privateMake()

//////////////////////////////////////////////////////////////////////////
// Obj Overrides
//////////////////////////////////////////////////////////////////////////

  **
  ** Return true if same number nanosecond ticks.
  **
  override Bool equals(Obj obj)

  **
  ** Compare based on nanosecond ticks.
  **
  override Int compare(Obj obj)

  **
  ** Return ticks().
  **
  override Int hash()

  **
  ** Return this.
  **
  override Str toStr()

//////////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////////

  **
  ** Return number of nanosecond ticks.
  **
  Int ticks()

  **
  ** Negative of this.  Shortcut is -a.
  **
  Duration negate()

  **
  ** Multiply this with b.  Shortcut is a*b.
  **
  Duration star(Float b)

  **
  ** Divide this by b.  Shortcut is a/b.
  **
  Duration slash(Float b)

  **
  ** Return remainder of this divided by b.  Shortcut is a%b.
  **
  Duration percent(Float b)

  **
  ** Add this with b.  Shortcut is a+b.
  **
  Duration plus(Duration b)

  **
  ** Subtract b from this.  Shortcut is a-b.
  **
  Duration minus(Duration b)

//////////////////////////////////////////////////////////////////////////
// Conversion
//////////////////////////////////////////////////////////////////////////

  **
  ** Get this duration in milliseconds (there may be a loss of precision).
  **
  Int toMillis()

}