//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   16 Apr 06  Brian Frank  Creation
//

**
** EnumTest
**
class EnumTest : Test
{

//////////////////////////////////////////////////////////////////////////
// Type
//////////////////////////////////////////////////////////////////////////

  Void testType()
  {
    verifyEq(EnumAbc.type.signature, "sysTest::EnumAbc")
    verifyEq(EnumAbc.type.base, Enum.type)
    verifyEq(EnumAbc.type.isEnum, true)
    verifyEq(EnumAbc.type.isClass, false)
    verifyEq(EnumAbc.type.isMixin, false)
    verifyEq(Obj.type.isEnum, false)
    verifyEq(Str.type.isEnum, false)

    verifySame(EnumAbc.A.type, EnumAbc.type)

    verify(EnumAbc.A is EnumAbc)
    verify(EnumAbc.A is Enum)
    verify(EnumAbc.A is Obj)
    verifyFalse((Obj)EnumAbc.A is Int)
  }

//////////////////////////////////////////////////////////////////////////
// Equals
//////////////////////////////////////////////////////////////////////////

  Void testEquals()
  {
    verify(EnumAbc.A == EnumAbc.A)
    verify(EnumAbc.A === EnumAbc.A)
    verify(EnumAbc.A != EnumAbc.B)
    verify(EnumAbc.A != EnumAbc.C)
    verify(EnumAbc.B != EnumAbc.C)
  }

//////////////////////////////////////////////////////////////////////////
// Compare
//////////////////////////////////////////////////////////////////////////

  Void testCompare()
  {
    verify(EnumAbc.A <= EnumAbc.A)
    verify(EnumAbc.A < EnumAbc.B)
    verify(EnumAbc.B < EnumAbc.C)
    verifyFalse(EnumAbc.C < EnumAbc.B)
    verifyFalse(EnumAbc.B < EnumAbc.A)
    verify(EnumAbc.B >= EnumAbc.B)
    verifyFalse(EnumAbc.B > EnumAbc.B)
    verify(EnumAbc.B > EnumAbc.A)

    verifyEq([EnumAbc.B, EnumAbc.C, EnumAbc.A].sort, [EnumAbc.A, EnumAbc.B, EnumAbc.C])
  }

//////////////////////////////////////////////////////////////////////////
// Ordinal
//////////////////////////////////////////////////////////////////////////

  Void testOrdinals()
  {
    verifyEq(EnumAbc.A.ordinal, 0)
    verifyEq(EnumAbc.B.ordinal, 1)
    verifyEq(EnumAbc.C.ordinal, 2)
  }

//////////////////////////////////////////////////////////////////////////
// Names
//////////////////////////////////////////////////////////////////////////

  Void testNames()
  {
    verifyEq(EnumAbc.A.name, "A")
    verifyEq(EnumAbc.B.name, "B")
    verifyEq(EnumAbc.C.name, "C")
  }

//////////////////////////////////////////////////////////////////////////
// To Str
//////////////////////////////////////////////////////////////////////////

  Void testToStr()
  {
    verifyEq(EnumAbc.A.toStr, "A")
    verifyEq(EnumAbc.B.toStr, "B")
    verifyEq(EnumAbc.C.toStr, "C")
  }

//////////////////////////////////////////////////////////////////////////
// Values
//////////////////////////////////////////////////////////////////////////

  Void testValues()
  {
    verifyEq(EnumAbc.values, [EnumAbc.A, EnumAbc.B, EnumAbc.C])
    verifyEq(EnumAbc.values.isRO, true)
  }

//////////////////////////////////////////////////////////////////////////
// Additional Methods
//////////////////////////////////////////////////////////////////////////

  Void testAdditionalMethods()
  {
    verifyEq(EnumAbc.A.negOrdinal, 0)
    verifyEq(EnumAbc.B.negOrdinal, -1)
    verifyEq(EnumAbc.C.negOrdinal, -2)
  }

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  Void testCtor()
  {
    verifyEq(Suits.values, [Suits.clubs, Suits.diamonds, Suits.hearts, Suits.spades])
    verifyEq(Suits.clubs.ordinal,    0)
    verifyEq(Suits.diamonds.ordinal, 1)
    verifyEq(Suits.hearts.ordinal,   2)
    verifyEq(Suits.spades.ordinal,   3)

    verifyEq(Suits.clubs.color,    "black")
    verifyEq(Suits.diamonds.color, "red")
    verifyEq(Suits.hearts.color,   "red")
    verifyEq(Suits.spades.color,   "black")
  }

//////////////////////////////////////////////////////////////////////////
// Reflection
//////////////////////////////////////////////////////////////////////////

  Void testReflection()
  {
    // enum has fields
    verify(EnumAbc.type.slot("A").isField)
    verifyEq(EnumAbc.type.field("A").name, "A")
    verifyEq(EnumAbc.type.field("A").getter, null)
    verifyEq(EnumAbc.type.field("A").setter, null)

    // get
    verifyEq(EnumAbc.type.field("A").get(null), EnumAbc.A)
    verifyEq(EnumAbc.type.field("B")[null], EnumAbc.B)
  }

}

internal enum EnumAbc
{
  A, B, C

  Int negOrdinal() { return -ordinal }
}

enum Suits
{
  clubs("black"),
  diamonds("red"),
  hearts("red"),
  spades("black");

  private new make(Str color) { this.color = color; }

  const Str color;
}

enum Foo
{
  one,
  two,
  three
}