//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   30 Jan 06  Brian Frank  Creation
//

**
** ListTest
**
class ListTest : Test
{

//////////////////////////////////////////////////////////////////////////
// Play
//////////////////////////////////////////////////////////////////////////

  Void testPlay()
  {
    x := [0, 1, 2, 3]
    verify(x.type == Int[].type)
    verify(x.type === Int[].type)
    a := x[-1]
    verify(a is Int)
    verify(a.type == Int.type)
    verify(a == 3)
  }

//////////////////////////////////////////////////////////////////////////
// Equals
//////////////////////////////////////////////////////////////////////////

  Void testEquals()
  {
    verifyEq([0, 1], [0, 1])
    verifyEq([0, 1].hash, [0, 1].hash)
    verifyNotEq([0, 1], null)
    verifyNotEq(null, [0, 1])
    verifyNotEq([1, 0], [0, 1])
    verifyNotEq([0, 1], [0, 1, 2])
    verifyNotEq([0, 1, 2], [0, 1])
    verifyNotEq([0, 1, 2], "string")
  }

//////////////////////////////////////////////////////////////////////////
// Is Operator
//////////////////////////////////////////////////////////////////////////

  Void testIsExplicit()
  {
    // Obj[]
    Obj a := Obj[,]
    verify(a is Obj)
    verify(a is Obj[])
    verifyFalse(a is Str)
    verifyFalse(a is Str[])

    // Str[]
    Obj b := Str[,]
    verify(b is Obj)
    verify(b is Obj[])
    verify(b is Str[])
    verifyFalse(b is Str)
    verifyFalse(b is Int[])

    // Field[]
    Obj c := Field[,]
    verify(c is Obj)
    verify(c is Obj[])
    verify(c is Slot[])
    verify(c is Field[])
  }

  Void testIsInfered()
  {
    // Obj[]
    verify([2,"a"] is Obj)
    verify([3,"b"] is Obj[])
    verifyFalse((Obj)[type,8.0] is Str)
    verifyFalse((Obj)["a",this] is Str[])

    // Int[]
    verify([3] is Obj)
    verify([6] is Obj[])
    verify([3] is Obj)
    verify([4,3] is Int[])
    verify([4,null] is Int[])  // null doesn't count
    verify([null, null, 9] is Int[])  // null doesn't count
    verifyFalse((Obj)[-1,9] is Int)
    verifyFalse((Obj)[4,6,9] is Str[])
  }

//////////////////////////////////////////////////////////////////////////
// As Operator
//////////////////////////////////////////////////////////////////////////

  Void testAsExplicit()
  {
    Obj x := [,];

    Obj o    := x as Obj;    verifySame(o , x)
    Bool b   := x as Bool;   verifySame(b , null)
    Str s    := x as Str;    verifySame(s , null)
    List l   := x as List;   verifySame(l , x)
    Obj[] ol := x as Obj[];  verifySame(ol , x)
    Int[] il := x as Int[];  verifySame(il , null)
    Str[] sl := x as Str[];  verifySame(sl , null)

    x  = ["a", "b"]
    o  = x as Obj;    verifySame(o , x)
    b  = x as Bool;   verifySame(b , null)
    s  = x as Str;    verifySame(s , null)
    l  = x as List;   verifySame(l , x)
    ol = x as Obj[];  verifySame(ol , x)
    il = x as Int[];  verifySame(il , null)
    sl = x as Str[];  verifySame(sl , x)
  }

//////////////////////////////////////////////////////////////////////////
// Reflection
//////////////////////////////////////////////////////////////////////////

  Void testReflect()
  {
    verifyEq([,].type.base,      List.type)
    verifyEq([,].type.base.base, Obj.type)
    verifyEq([,].type.pod.name,  "sys")
    verifyEq([,].type.name,      "List")
    verifyEq([,].type.qname,     "sys::List")
    verifyEq([,].type.signature, "sys::Obj[]")
    verifyEq([,].type.toStr,     "sys::Obj[]")
    verifyEq([,].type.method("isEmpty").returnType, Bool.type)
    verifyEq([,].type.method("get").returnType,   Obj.type)
    verifyEq([,].type.method("add").returnType,   Obj[].type)
    verifyEq([,].type.method("add").params[0].paramType, Obj.type)
    verifyEq([,].type.method("each").params[0].paramType, |Obj a, Int i->Void|.type)
    verifyNotEq([,].type.method("each").params[0].paramType, |Str a, Int i->Void|.type)

    verifyEq([7].type.base,      List.type)
    verifyEq([7].type.base.base, Obj.type)
    verifyEq([7].type.pod.name,  "sys")
    verifyEq([7].type.name,      "List")
    verifyEq([7].type.qname,     "sys::List")
    verifyEq([7].type.signature, "sys::Int[]")
    verifyEq([7].type.toStr,     "sys::Int[]")
    verifyEq([7].type.method("isEmpty").returnType, Bool.type)
    verifyEq([7].type.method("get").returnType,   Int.type)
    verifyEq([7].type.method("add").returnType,   Int[].type)
    verifyEq([7].type.method("add").params[0].paramType, Int.type)
    verifyEq([7].type.method("each").params[0].paramType, |Int a, Int i->Void|.type)
    verifyNotEq([7].type.method("each").params[0].paramType, |Obj a, Int i->Void|.type)

    verifyEq([[8ms]].type.base,      List.type)
    verifyEq([[8ms]].type.base.base, Obj.type)
    verifyEq([[8ms]].type.pod.name,  "sys")
    verifyEq([[8ms]].type.name,      "List")
    verifyEq([[8ms]].type.qname,     "sys::List")
    verifyEq([[8ms]].type.signature, "sys::Duration[][]")
    verifyEq([[8ms]].type.toStr,     "sys::Duration[][]")
    verifyEq([[8ms]].type.method("isEmpty").returnType, Bool.type)
    verifyEq([[8ms]].type.method("get").returnType,   Duration[].type)
    verifyEq([[8ms]].type.method("add").returnType,   Duration[][].type)
    verifyEq([[8ms]].type.method("add").params[0].paramType, Duration[].type)
    verifyEq([8ms].type.method("insert").params[1].paramType, Duration.type)
    verifyEq([8ms].type.method("removeAt").returnType, Duration.type)
    verifyEq([8ms].type.method("each").params[0].paramType, |Duration a, Int i->Void|.type)
    verifyEq([8ms].type.method("map").returnType, List.type)
    verifyNotEq([8ms].type.method("map").returnType, Duration[].type)
    verifyNotEq([8ms].type.method("each").params[0].paramType, |Obj a, Int i->Void|.type)
  }

//////////////////////////////////////////////////////////////////////////
// Items
//////////////////////////////////////////////////////////////////////////

  Void testItems()
  {
    // add, insert, removeAt, get, size
    Obj r;
    list := Int[,]
    list.add(10); verifyEq(list, [10]); verifyEq(list.size, 1);
    list.add(20); verifyEq(list, [10, 20]); verifyEq(list.size, 2);
    list.add(30); verifyEq(list, [10, 20, 30]); verifyEq(list.size, 3);
    list.insert(0, 40); verifyEq(list, [40, 10, 20, 30]); verifyEq(list.size, 4);
    list.insert(1, 50).add(60); verifyEq(list, [40, 50, 10, 20, 30, 60]); verifyEq(list.size, 6);
    list.insert(-1, 70); verifyEq(list, [40, 50, 10, 20, 30, 70, 60]); verifyEq(list.size, 7);
    list.insert(-3, 80); verifyEq(list, [40, 50, 10, 20, 80, 30, 70, 60]); verifyEq(list.size, 8);
    verify(list.removeAt(0) == 40); verifyEq(list, [50, 10, 20, 80, 30, 70, 60]); verifyEq(list.size, 7);
    verify(list.removeAt(6) == 60); verifyEq(list, [50, 10, 20, 80, 30, 70]); verifyEq(list.size, 6);
    list.removeAt(3); verifyEq(list, [50, 10, 20, 30, 70]); verifyEq(list.size, 5);
    list.removeAt(-1); verifyEq(list, [50, 10, 20, 30]); verifyEq(list.size, 4);
    list.removeAt(-4); verifyEq(list, [10, 20, 30]); verifyEq(list.size, 3);
    list.add(40); verifyEq(list, [10, 20, 30, 40]); verifyEq(list.size, 4);
    verify(list.insert(2, 50) === list); verifyEq(list, [10, 20, 50, 30, 40]); verifyEq(list.size, 5);
    list.removeAt(2); verifyEq(list, [10, 20, 30, 40]); verifyEq(list.size, 4);
    verifyEq(list[0], 10); verifyEq(list[-4], 10);
    verifyEq(list[1], 20); verifyEq(list[-3], 20);
    verifyEq(list[2], 30); verifyEq(list[-2], 30);
    verifyEq(list[3], 40); verifyEq(list[-1], 40);
    list[0] = -10; verifyEq(list, [-10, 20, 30, 40]); verifyEq(list.size, 4);
    list[2] = -30; verifyEq(list, [-10, 20, -30, 40]); verifyEq(list.size, 4);
    list[-1] = -40; verifyEq(list, [-10, 20, -30, -40]); verifyEq(list.size, 4);
    list[-3] = -20; verifyEq(list, [-10, -20, -30, -40]); verifyEq(list.size, 4);
    list[-2] = null; verifyEq(list, [-10, -20, null, -40]); verifyEq(list.size, 4);
    list[0] = null; verifyEq(list, [null, -20, null, -40]); verifyEq(list.size, 4);

    // IndexErr - no items
    list = Int[,]
    verifyErr(IndexErr.type) |,| { x:=list[0] }
    verifyErr(IndexErr.type) |,| { x:=list[1] }
    verifyErr(IndexErr.type) |,| { x:=list[-1] }
    verifyErr(IndexErr.type) |,| { x:=list[-2] }

    // IndexErr - one items
    list = [77]
    verifyErr(IndexErr.type) |,| { x:=list[1] }
    verifyErr(IndexErr.type) |,| { x:=list[-2] }
  }

//////////////////////////////////////////////////////////////////////////
// Duplicate
//////////////////////////////////////////////////////////////////////////

  Void testDup()
  {
    a := [0, 1, 2]
    verifyEq(a.size, 3)
    verifyEq(a.type, Int[].type)
    verifyEq(a, [0, 1, 2])

    b := a.dup
    verifyEq(b.size, 3)
    verifyEq(b.type, Int[].type)
    verifyEq(b, [0, 1, 2])

    a[1] = 99
    verifyEq(a, [0, 99, 2])
    verifyEq(b, [0, 1, 2])

    a.clear
    verifyEq(a.size, 0)
    verifyEq(b.size, 3)
    verifyEq(a, Int[,])
    verifyEq(b, [0, 1, 2])
  }

//////////////////////////////////////////////////////////////////////////
// AddAll/InsertAll
//////////////////////////////////////////////////////////////////////////

  Void testInsertAll()
  {
    a := Str[,]
    x := Str[,]

    verifyEq(a.addAll(x), Str[,])
    verifyEq(a.insertAll(-1, x), Str[,])

    a.add("a")
    verifyEq(a.addAll(x), ["a"])
    verifyEq(a.insertAll(0, x), ["a"])

    x.add("x")
    verifyEq(a.addAll(x), ["a", "x"])
    verifyEq(a.insertAll(0, x), ["x", "a", "x"])

    x.add("y")
    verifyEq(a.addAll(x), ["x", "a", "x", "x", "y"])
    verifyEq(a.insertAll(1, x), ["x", "x", "y", "a", "x", "x", "y"])

    a = ["a", "b", "c"]
    verifyEq(a.insertAll(1, a), ["a", "a", "b", "c", "b", "c"])
    verifyEq(a.insertAll(-2, ["x", "y"]), ["a", "a", "b", "c", "x", "y", "b", "c"])
  }

//////////////////////////////////////////////////////////////////////////
// Size/Capacity
//////////////////////////////////////////////////////////////////////////

  Void testSizeCapacity()
  {
    x := Str[,]

    verifyEq(x.size, 0)
    verifyEq(x.capacity, 0)

    x.capacity = 2
    verifyEq(x.size, 0)
    verifyEq(x.capacity, 2)

    x.add("a").add("b")
    verifyEq(x.size, 2)
    verifyEq(x.capacity, 2)
    verifyErr(ArgErr.type) |,| { x.capacity = 1 }

    x.add("c")  // auto-grow
    verifyEq(x.size, 3)
    verifyEq(x.capacity, 10)
    verifyEq(x, ["a", "b", "c"])

    x.capacity = 3 // manual trim
    verifyEq(x.size, 3)
    verifyEq(x.capacity, 3)
    verifyEq(x, ["a", "b", "c"])

    x.size = 4
    verifyEq(x.size, 4)
    verifyEq(x.capacity, 4)
    verifyEq(x, ["a", "b", "c", null])

    x.size = 2
    verifyEq(x.size, 2)
    verifyEq(x.capacity, 2)
    verifyEq(x, ["a", "b"])

    x.size = 5
    verifyEq(x.size, 5)
    verifyEq(x.capacity, 5)
    verifyEq(x, ["a", "b", null, null, null])
    x.add("z")

    verifyEq(x, ["a", "b", null, null, null, "z"])
    verifyEq(x.size, 6)
    verifyEq(x.capacity, 10)

    x.size = 0
    verifyEq(x.size, 0)
    verifyEq(x.capacity, 0)
    verifyEq(x, Str[,])

    x.add("x")
    verifyEq(x.size, 1)
    verifyEq(x.capacity, 10)
    verifyEq(x, ["x"])

    x.size = x.size
    verifyEq(x.size, 1)
    verifyEq(x.capacity, 1)
    verifyEq(x, ["x"])
  }

//////////////////////////////////////////////////////////////////////////
// Slicing
//////////////////////////////////////////////////////////////////////////

  Void testSlicing()
  {
    /* Ruby
    irb(main):001:0> a = [0, 1, 2, 3] => [0, 1, 2, 3]
    irb(main):002:0> a[0..3]   => [0, 1, 2, 3]
    irb(main):003:0> a[0..2]   => [0, 1, 2]
    irb(main):004:0> a[0..1]   => [0, 1]
    irb(main):005:0> a[0..0]   => [0]
    irb(main):006:0> a[0...0]  => []
    irb(main):007:0> a[0...1]  => [0]
    irb(main):008:0> a[0...2]  => [0, 1]
    irb(main):009:0> a[0...3]  => [0, 1, 2]
    irb(main):010:0> a[1..3]   => [1, 2, 3]
    irb(main):011:0> a[1..4]   => [1, 2, 3]
    irb(main):012:0> a[1..5]   => [1, 2, 3]
    irb(main):013:0> a[1..1]   => [1]
    irb(main):014:0> a[1..-1]  => [1, 2, 3]
    irb(main):015:0> a[1..-2]  => [1, 2]
    irb(main):016:0> a[1..-3]  => [1]
    irb(main):017:0> a[1..-4]  => []
    irb(main):018:0> a[1...-1] => [1, 2]
    irb(main):019:0> a[1...-2] => [1]
    irb(main):020:0> a[1...-3] => []
    irb(main):021:0> a[-3..-1] => [1, 2, 3]
    irb(main):022:0> a[-3..-2] => [1, 2]
    irb(main):023:0> a[-3..-3] => [1]
    */

    list := [0, 1, 2, 3]

    verifyEq(list[0..3],  [0, 1, 2, 3])
    verifyEq(list[0..2],  [0, 1, 2])
    verifyEq(list[0..1],  [0, 1])
    verifyEq(list[0..0],  [0])
    verifyEq(list[0...0], Int[,])
    verifyEq(list[0...1], [0])
    verifyEq(list[0...2], [0, 1])
    verifyEq(list[0...3], [0, 1, 2])
    verifyEq(list[0...4], [0, 1, 2, 3])
    verifyEq(list[1..3], [1, 2, 3])
    verifyEq(list[1..1], [1])
    verifyEq(list[1..-1], [1, 2, 3])
    verifyEq(list[1..-2], [1, 2])
    verifyEq(list[1..-3], [1])
    verifyEq(list[1..-4], Int[,])
    verifyEq(list[1...-1], [1, 2])
    verifyEq(list[1...-2], [1])
    verifyEq(list[1...-3], Int[,])
    verifyEq(list[-3..-1], [1, 2, 3])
    verifyEq(list[-3..-2], [1, 2])
    verifyEq(list[-3..-3], [1])
    verifyEq(list[4..-1], Int[,])

    // examples
    ex := [0, 1, 2, 3]
    verifyEq(ex[0..2], [0, 1, 2])
    verifyEq(ex[3..3], [3])
    verifyEq(ex[-2..-1], [2, 3])
    verifyEq(ex[0...2], [0, 1])
    verifyEq(ex[1..-2], [1, 2])

    // errors
    verifyErr(IndexErr.type) |,| { x:=list[0..4] }
    verifyErr(IndexErr.type) |,| { x:=list[0...5] }
    verifyErr(IndexErr.type) |,| { x:=list[2...1] }
    verifyErr(IndexErr.type) |,| { x:=list[3..1] }
    verifyErr(IndexErr.type) |,| { x:=list[-5..-1] }
    verifyErr(IndexErr.type) |,| { x:=list[1..4] }
    verifyErr(IndexErr.type) |,| { x:=list[1..5] }
  }

//////////////////////////////////////////////////////////////////////////
// Remove
//////////////////////////////////////////////////////////////////////////

  Void testRemove()
  {
    list := ["a", "b", "c", null, "a"]
    verifyEq(list.remove("b"), "b");    verifyEq(list, ["a", "c", null, "a"])
    verifyEq(list.remove("a"), "a");    verifyEq(list, ["c", null, "a"])
    verifyEq(list.remove("x"), null);   verifyEq(list, ["c", null, "a"])
    verifyEq(list.remove("a"), "a");    verifyEq(list, ["c", null])
    verifyEq(list.remove(null), null);  verifyEq(list, ["c"])
    verifyEq(list.remove("c"), "c");    verifyEq(list, Str[,])
    verifyEq(list.remove("a"), null);   verifyEq(list, Str[,])
  }

//////////////////////////////////////////////////////////////////////////
// Clear
//////////////////////////////////////////////////////////////////////////

  Void testClear()
  {
    list := ["a", "b", "c"]
    verifyEq(list.size, 3)
    verifyFalse(list.isEmpty)

    list.clear
    verifyEq(list.size, 0)
    verify(list.isEmpty)
  }

//////////////////////////////////////////////////////////////////////////
// Contains/Index
//////////////////////////////////////////////////////////////////////////

  Void testContainsIndex()
  {
    list := ["a", "b", null, "c", null, "b"]

    verifyEq([,].contains(null), false)
    verifyEq([,].contains("a"), false)

    verify(list.contains("a"))
    verifyEq(list.index("a"), 0)

    verify(list.contains("b"))
    verifyEq(list.index("b"), 1)

    verify(list.contains("c"))
    verifyEq(list.index("c"), 3)

    verify(list.contains(null))
    verifyEq(list.index(null), 2)

    verifyFalse(list.contains("d"))
    verifyEq(list.index("d"), null)

    verifyEq(list.index("a", -5), null)
    verifyEq(list.index("a", -6), 0)
    verifyEq(list.index("b", 1), 1)
    verifyEq(list.index("b", 2), 5)
    verifyEq(list.index("b", -1), 5)
    verifyEq(list.index("b", -5), 1)

    verifyEq(list.index(null, 0), 2)
    verifyEq(list.index(null, 2), 2)
    verifyEq(list.index(null, 3), 4)

    verifyErr(IndexErr.type) |,| { list.index("a", 6) }
    verifyErr(IndexErr.type) |,| { list.index("a", -7) }
  }

//////////////////////////////////////////////////////////////////////////
// FirstLast
//////////////////////////////////////////////////////////////////////////

  Void testFirstLast()
  {
    verifyEq([,].first, null)
    verifyEq([,].last, null)

    verifyEq([5].first, 5)
    verifyEq([5].last,  5)

    verifyEq([1,2].first, 1)
    verifyEq([1,2].last,  2)

    verifyEq([1,2,3].first, 1)
    verifyEq([1,2,3].last,  3)
  }

//////////////////////////////////////////////////////////////////////////
// Stack
//////////////////////////////////////////////////////////////////////////

  Void testStack()
  {
    s := Int[,]

    verifyEq(s.peek, null); verifyEq(s.pop,  null)

    s.push(1)
    verifyEq(s.peek, 1);    verifyEq(s.pop,  1)
    verifyEq(s.peek, null); verifyEq(s.pop,  null)

    s.push(1);
    s.push(2)
    verifyEq(s.peek, 2);    verifyEq(s.pop,  2)
    verifyEq(s.peek, 1);    verifyEq(s.pop,  1)
    verifyEq(s.peek, null); verifyEq(s.pop,  null)
  }

//////////////////////////////////////////////////////////////////////////
// Each
//////////////////////////////////////////////////////////////////////////

  Void testEach()
  {
    values  := Int[,]
    indexes := Int[,]

    // empty list
    Int[,].each |Int value, Int index|
    {
      values.add(value)
      indexes.add(index)
    }
    verifyEq(values,  Int[,])
    verifyEq(indexes, Int[,])

    // list of one
    values.clear;
    indexes.clear;
    [ 7 ].each |Int value, Int index|
    {
      values.add(value)
      indexes.add(index)
    }
    verifyEq(values,  [7])
    verifyEq(indexes, [0])

    // list of two
    values.clear;
    indexes.clear;
    [ -9, 0xab ].each |Int value, Int index|
    {
      values.add(value)
      indexes.add(index)
    }
    verifyEq(values,  [-9, 0xab])
    verifyEq(indexes, [0, 1])

    // list of four
    values.clear;
    indexes.clear;
    [ 10, 20, 30, 40 ].each |Int value, Int index|
    {
      values.add(value)
      indexes.add(index)
    }
    verifyEq(values,  [10, 20, 30, 40])
    verifyEq(indexes, [0, 1, 2, 3])
  }

//////////////////////////////////////////////////////////////////////////
// Eachr
//////////////////////////////////////////////////////////////////////////

  Void testEachr()
  {
    values  := Int[,]
    indexes := Int[,]

    // empty list
    Int[,].eachr |Int value, Int index|
    {
      values.add(value)
      indexes.add(index)
    }
    verifyEq(values,  Int[,])
    verifyEq(indexes, Int[,])

    // list of one
    values.clear;
    indexes.clear;
    [ 7 ].eachr |Int value, Int index|
    {
      values.add(value)
      indexes.add(index)
    }
    verifyEq(values,  [7])
    verifyEq(indexes, [0])

    // list of two
    values.clear;
    indexes.clear;
    [ -9, 0xab ].eachr |Int value, Int index|
    {
      values.add(value)
      indexes.add(index)
    }
    verifyEq(values,  [0xab, -9])
    verifyEq(indexes, [1, 0])

    // list of four
    values.clear;
    indexes.clear;
    [ 10, 20, 30, 40 ].eachr |Int value, Int index|
    {
      values.add(value)
      indexes.add(index)
    }
    verifyEq(values,  [40, 30, 20, 10])
    verifyEq(indexes, [3, 2, 1, 0])

    // just value
    values.clear;
    indexes.clear;
    [ 1, 2, 3, 4, 5, 6, 7, 8 ].eachr |Int value|
    {
      values.add(value)
    }
    verifyEq(values,  [8, 7, 6, 5, 4, 3, 2, 1])
  }

//////////////////////////////////////////////////////////////////////////
// Find
//////////////////////////////////////////////////////////////////////////

  Void testFind()
  {
    list := [0, 10, 20, 30, 40, 60]

    // find
    verifyEq(list.find |Int v, Int i->Bool| { return v == 20 }, 20)
    verifyEq(list.find |Int v, Int i->Bool| { return i == 3 }, 30)
    verifyEq(list.find |Int v, Int i->Bool| { return false }, null)
    verifyEq(list.find |Int v->Bool| { return false }, null)
    verifyEq(list.find |->Bool| { return false }, null)

    // typed assign
    Int x := list.find |Int v->Bool| { return v.toStr == "40" }
    verifyEq(x, 40)

    // findAll
    verifyEq(list.findAll|Int v, Int i->Bool| { return v % 20 == 0 }, [0, 20, 40, 60])
    verifyEq(list.findAll|Int v, Int i->Bool| { return i % 2 == 0 },  [0, 20, 40])
    verifyEq(list.findAll|Int v, Int i->Bool| { return false },  Int[,])
    verifyEq(list.findAll|Int v->Bool| { return false },  Int[,])
    verifyEq(list.findAll|->Bool| { return false },  Int[,])

    // findType
    verifyEq(["a", 3, "b", 6sec].findType(Str.type), ["a", "b"])
    verifyEq(["a", 3, "b", 6sec].findType(Str.type).type, Str[].type)
    verifyEq(["a", 3, "b", 6sec, 5f].findType(Num.type), [3, 5f])
    verifyEq(["a", 3, "b", 6sec, 5f].findType(Num.type).type, Num[].type)
    verifyEq([null, "a", 3, "b", null, 5ms].findType(Duration.type), [5ms])
    verifyEq(["a", 3, "b", 6sec, 5f].findType(Obj.type), ["a", 3, "b", 6sec, 5f])

    // exclude
    verifyEq(list.exclude|Int v, Int i->Bool| { return v % 20 == 0 }, [10, 30])
    verifyEq(list.exclude|Int v, Int i->Bool| { return i % 2 == 0 },  [10, 30, 60])
    verifyEq(list.exclude|Int v, Int i->Bool| { return true },  Int[,])
    verifyEq(list.exclude|Int v->Bool| { return true },  Int[,])
    verifyEq(list.exclude|->Bool| { return true },  Int[,])

    // typed assign
    Int[] a := list.findAll |Int v->Bool| { return v.toStr.size == 1 }
    verifyEq(a, [0])
  }

//////////////////////////////////////////////////////////////////////////
// Reduce
//////////////////////////////////////////////////////////////////////////

  Void testReduce()
  {
    list := [3, 4, 5]
    verifyEq(list.reduce(0) |Obj r, Int v->Obj| { return v*2 + (Int)r }, 24)
    verifyEq(list.reduce(10) |Obj r, Int v, Int i->Obj| { return v + (Int)r + i }, 25)
  }

//////////////////////////////////////////////////////////////////////////
// Map
//////////////////////////////////////////////////////////////////////////

  Void testMap()
  {
    list := [3, 4, 5]
    verifyEq(list.map(Int[,]) |Int v->Obj| { return v*2 },  [6, 8, 10])
    verifyEq(list.map([,]) |Int v->Obj| { return null }, [null, null, null])
    verifyEq(list.map(Bool[,]) |Int v, Int i->Obj| { return i%2==0 },  [true, false, true])

    acc := [0, 1, 2]
    list.map(acc) |Int v->Obj| { return v }
    verifyEq(acc, [0, 1, 2, 3, 4, 5])
  }

//////////////////////////////////////////////////////////////////////////
// Any/All
//////////////////////////////////////////////////////////////////////////

  Void testAnyAll()
  {
    // empty
    list := Str[,]
    verifyEq(list.any |Str s->Bool| { return s.size == 3 }, false)
    verifyEq(list.all |Str s->Bool| { return s.size == 3 }, true)

    // all 3
    list = ["foo", "bar"]
    verifyEq(list.any |Str s->Bool| { return s.size == 3 }, true)
    verifyEq(list.all |Str s->Bool| { return s.size == 3 }, true)
    verifyEq(list.any |Str s->Bool| { return s.size == 4 }, false)
    verifyEq(list.all |Str s->Bool| { return s.size == 4 }, false)

    // one 3, one 4
    list = ["foo", "pool"]
    verifyEq(list.any |Str s->Bool| { return s.size == 3 }, true)
    verifyEq(list.all |Str s->Bool| { return s.size == 3 }, false)
    verifyEq(list.any |Str s->Bool| { return s.size == 4 }, true)
    verifyEq(list.all |Str s->Bool| { return s.size == 4 }, false)

    // one 3, one 4 with index
    list = ["foo", "pool"]
    verifyEq(list.any |Str s,Int i->Bool| { return s.size == 3 }, true)
    verifyEq(list.all |Str s,Int i->Bool| { return s.size == 3 }, false)
    verifyEq(list.any |Str s,Int i->Bool| { return s.size == 4 }, true)
    verifyEq(list.all |Str s,Int i->Bool| { return s.size == 4 }, false)
  }

//////////////////////////////////////////////////////////////////////////
// Min/Max
//////////////////////////////////////////////////////////////////////////

  Void testMinMax()
  {
    // empty
    list := Str[,]
    verifyEq(list.min, null)
    verifyEq(list.max, null)
    verifyEq(list.min |Str a, Str b->Int| { return a.size <=> b.size }, null)
    verifyEq(list.max |Str a, Str b->Int| { return a.size <=> b.size }, null)

    // doc example
    list = Str["albatross", "dog", "horse"]
    verifyEq(list.min, "albatross")
    verifyEq(list.max, "horse")
    verifyEq(list.min |Str a, Str b->Int| { return a.size <=> b.size }, "dog")
    verifyEq(list.max |Str a, Str b->Int| { return a.size <=> b.size }, "albatross")

    // with null
    list = Str["a", null, "b"]
    verifyEq(list.min, null)
    verifyEq(list.max, "b")
  }

//////////////////////////////////////////////////////////////////////////
// Unique
//////////////////////////////////////////////////////////////////////////

  Void testUnique()
  {
    verifyEq(Str[,].unique, Str[,])
    verifyEq(["a"].unique, ["a"])
    verifyEq(["a", "b"].unique, ["a", "b"])
    verifyEq(["a", "b", "c"].unique, ["a", "b", "c"])
    verifyEq(["a", "a", "b", "c"].unique, ["a", "b", "c"])
    verifyEq(["a", "b", "a", "c"].unique, ["a", "b", "c"])
    verifyEq(["a", "b", "c", "a"].unique, ["a", "b", "c"])
    verifyEq(["a", null, "b", "c", "a"].unique, ["a", null, "b", "c"])
    verifyEq(["a", null, "b", "b", "c", "a", null, "c", "a", "a"].unique, ["a", null, "b", "c"])
  }

//////////////////////////////////////////////////////////////////////////
// Union
//////////////////////////////////////////////////////////////////////////

  Void testUnion()
  {
    verifyEq([0, 1, 2].union([2]).type, Int[].type)
    verifyEq(Int[,].union([2]), [2])
    verifyEq(Int[6].union(Int[,]), [6])
    verifyEq(Int[0, 1, 2].union(Int[1, 2, 3]), [0, 1, 2, 3])
    verifyEq(Int[0, 1, 2].union(Int[10, 20]), [0, 1, 2, 10, 20])
    verifyEq(Int[0, 1, 2, 1, 2, 0].union(Int[10, 20, 10, 10]), [0, 1, 2, 10, 20])
    verifyEq(Int[null, 0, 1, 2].union(Int[10, null, 20, 2]), [null, 0, 1, 2, 10, 20])
  }

//////////////////////////////////////////////////////////////////////////
// Intersection
//////////////////////////////////////////////////////////////////////////

  Void testIntersection()
  {
    verifyEq([0, 1, 2].intersection([2]).type, Int[].type)
    verifyEq(Int[,].intersection([2]), Int[,])
    verifyEq(Int[6].intersection(Int[,]), Int[,])
    verifyEq([4].intersection([5]), Int[,])
    verifyEq([0, 1, 2].intersection([2]), [2])
    verifyEq([0, 1, 2].intersection([0, 2]), [0,2])
    verifyEq([0, 1, 2].intersection([2, 0]), [0,2])
    verifyEq([0, 1, 2].intersection([0, 1, 2]), [0, 1, 2])
    verifyEq([0, 1, 2].intersection([0, 1, 2, 3]), [0, 1, 2])
    verifyEq([0, 1, 2].intersection([3, 2, 1, 0]), [0, 1, 2])
    verifyEq([0, 1, 2, 3].intersection([5, 3, 1]), [1, 3])
    verifyEq([0, null, 2].intersection([0, 1, 2, 3]), [0, 2])
    verifyEq([0, null, 2].intersection([null, 0, 1, 2, 3]), [0, null, 2])
    verifyEq([0, 1, 2, 2, 1, 1].intersection([2, 2, 1, 0]), [0, 1, 2])
    verifyEq([0, 1, null, 2, 1, null, 1].intersection([2, null, 2, 1, 0]), [0, 1, null, 2])
  }

//////////////////////////////////////////////////////////////////////////
// Sort
//////////////////////////////////////////////////////////////////////////

  Void testSort()
  {
    x := Int[,]
    x.sort
    verifyEq(x, Int[,])

    x = [6, 3, 5, 2, 4, 1]
    x.sort
    verifyEq(x, Int[1, 2, 3, 4, 5, 6])
    x.sort
    verifyEq(x, Int[1, 2, 3, 4, 5, 6])
    x.sortr
    verifyEq(x, Int[6, 5, 4, 3, 2, 1])
    x.sortr
    verifyEq(x, Int[6, 5, 4, 3, 2, 1])

    x = [3, 1, 6, 4, 2, 5]
    x.sort |Int a, Int b->Int| { return a <=> b }
    verifyEq(x, Int[1, 2, 3, 4, 5, 6])
    x.sortr |Int a, Int b->Int| { return a <=> b }
    verifyEq(x, Int[6, 5, 4, 3, 2, 1])

    x = [3, 1, 6, 4, 2, 5]
    names := ["zero", "one", "two", "three", "four", "five", "six" ]
    comparator := |Int a, Int b->Int| { return names[a] <=> names[b] }
    x.sort(comparator)
    verifyEq(x, Int[5, 4, 1, 6, 3, 2])
    x.sortr(comparator)
    verifyEq(x, Int[2, 3, 6, 1, 4, 5])
  }

//////////////////////////////////////////////////////////////////////////
// Reverse
//////////////////////////////////////////////////////////////////////////

  Void testReverse()
  {
    verifyEq(Int[,].reverse, Int[,])
    verifyEq(Int[5].reverse, Int[5])
    verifyEq(Int[1,2].reverse, Int[2,1])
    verifyEq(Int[1,2,3].reverse, Int[3,2,1])
    verifyEq(Int[1,2,3,4].reverse, Int[4,3,2,1])
    verifyEq(Int[1,2,3,4,5].reverse, Int[5,4,3,2,1])
    verifyEq(Int[1,2,3,4,5,6].reverse, Int[6,5,4,3,2,1])
    verifyEq(Int[1,2,3,4,5,6,7].reverse, Int[7,6,5,4,3,2,1])
    verifyEq(Int[1,2,3,4,5,6,7,8].reverse, Int[8,7,6,5,4,3,2,1])
  }

//////////////////////////////////////////////////////////////////////////
// Swap
//////////////////////////////////////////////////////////////////////////

  Void testSwap()
  {
    x := [0, 1, 2, 3, 4]
    verifyEq(x.swap(0, 1),   [1, 0, 2, 3, 4])
    verifyEq(x.swap(-1, -2), [1, 0, 2, 4, 3])
    verifyEq(x.swap(2, -2),  [1, 0, 4, 2, 3])
  }

//////////////////////////////////////////////////////////////////////////
// Str
//////////////////////////////////////////////////////////////////////////

  Void testStr()
  {
    o := [,]
    verifyEq(o.toStr,     "[,]")
    verifyEq(o.join,      "")
    verifyEq(o.join("-"), "")

    s := ["foo"]
    verifyEq(s.toStr,      "[foo]")
    verifyEq(s.join,       "foo")
    verifyEq(s.join("-"),  "foo")
    verifyEq(s.join("; "), "foo")

    s = ["a", "b", "c"]
    verifyEq(s.toStr,      "[a, b, c]")
    verifyEq(s.join,       "abc")
    verifyEq(s.join("-"),  "a-b-c")
    verifyEq(s.join("; "), "a; b; c")
    verifyEq(s.join("-") |Str x->Str| { return "($x)" }, "(a)-(b)-(c)")

    s = [null, "foo", null]
    verifyEq(s.toStr,      "[null, foo, null]")
    verifyEq(s.join,       "nullfoonull")
    verifyEq(s.join("-"),  "null-foo-null")
    verifyEq(s.join("; "), "null; foo; null")
  }

//////////////////////////////////////////////////////////////////////////
// Readonly
//////////////////////////////////////////////////////////////////////////

  Void testReadonly()
  {
    // create rw list
    x := ["a", "b", "c"].trim
    verifyEq(x.isRW, true)
    verifyEq(x.isRO, false)
    verifySame(x.rw, x)

    // get ro list
    r := x.ro
    verifyEq(x.isRW, true)
    verifyEq(x.isRO, false)
    verifySame(x.rw, x)
    verifyEq(r.isRW, false)
    verifyEq(r.isRO, true)
    verifySame(x.ro, r)
    verifySame(x.ro, r)
    verifySame(r.ro, r)
    verifySame(r.ro, r)
    verifyEq(r, x)

    // verify all idempotent methods work
    verifyEq(r.type, Str[].type)
    verifyEq(r.isEmpty, false)
    verifyEq(r.size, 3)
    verifyEq(r.capacity, 3)
    verifyEq(r[0], "a")
    verifyEq(r[1], "b")
    verifyEq(r[2], "c")
    verifyEq(r[0..1], ["a", "b"])
    verifyEq(r.contains("b"), true)
    verifyEq(r.contains("x"), false)
    verifyEq(r.index("c"), 2)
    verifyEq(r.first, "a")
    verifyEq(r.last, "c")
    verifyEq(r.peek, "c")
    verifyEq(r.dup, ["a", "b", "c"])
    r.each |Str s, Int i| { verifyEq(r[i], s) }
    r.eachr |Str s, Int i| { verifyEq(r[i], s) }
    verifyEq(r.find |Str s->Bool| { return s == "b" }, "b")
    verifyEq(r.findAll |Str s->Bool| { return true }, ["a", "b", "c"])
    verifyEq(r.exclude |Str s->Bool| { return s == "c" }, ["a", "b"])
    verifyEq(r.any |Str s->Bool| { return true }, true)
    verifyEq(r.all |Str s->Bool| { return true }, true)
    verifyEq(r.reduce(0) |Obj result, Str ignore->Obj| { return result }, 0)
    verifyEq(r.map(Int[,]) |Str s->Obj| { return s.size}, [1, 1, 1])
    verifyEq(r.min, "a")
    verifyEq(r.max, "c")
    verifyEq(r.unique, ["a", "b", "c"])
    verifyEq(r.union(["a", "d"]), ["a", "b", "c", "d"])
    verifyEq(r.intersection(["a", "d"]), ["a"])
    verifyEq(r.toStr, "[a, b, c]")
    verifyEq(r.join, "abc")

    // verify all modification methods throw ReadonlyErr
    verifyErr(ReadonlyErr.type) |,| { r.size = 10 }
    verifyErr(ReadonlyErr.type) |,| { r.capacity = 10 }
    verifyErr(ReadonlyErr.type) |,| { r[2] = "x" }
    verifyErr(ReadonlyErr.type) |,| { r.add("x") }
    verifyErr(ReadonlyErr.type) |,| { r.addAll(["x"]) }
    verifyErr(ReadonlyErr.type) |,| { r.insert(2, "x") }
    verifyErr(ReadonlyErr.type) |,| { r.insertAll(2, ["x"]) }
    verifyErr(ReadonlyErr.type) |,| { r.remove("a") }
    verifyErr(ReadonlyErr.type) |,| { r.removeAt(5) }
    verifyErr(ReadonlyErr.type) |,| { r.clear }
    verifyErr(ReadonlyErr.type) |,| { r.trim }
    verifyErr(ReadonlyErr.type) |,| { r.pop }
    verifyErr(ReadonlyErr.type) |,| { r.push("x") }
    verifyErr(ReadonlyErr.type) |,| { r.sort }
    verifyErr(ReadonlyErr.type) |,| { r.sortr }
    verifyErr(ReadonlyErr.type) |,| { r.reverse }
    verifyErr(ReadonlyErr.type) |,| { r.swap(0, 1) }

    // verify rw detaches ro
    x.add("d")
    r2 := x.ro
    verifySame(x.ro, r2)
    verifyNotSame(r2, r)
    verifyNotSame(x.ro, r)
    verifyEq(r.isRO, true)
    verifyEq(r.size, 3)
    verifyEq(r, ["a", "b", "c"])
    x.remove("b")
    r3 := x.ro
    verifySame(x.ro, r3)
    verifyNotSame(r2, r3)
    verifyNotSame(r3, r)
    verifyNotSame(r2, r)
    verifyNotSame(x.ro, r)
    verifyEq(r.size, 3)
    verifyEq(r, ["a", "b", "c"])

    // verify ro to rw
    y := r.rw
    verifyEq(y.isRW, true)
    verifyEq(y.isRO, false)
    verifySame(y.rw, y)
    verifySame(y.ro, r)
    verifyEq(y, r)
    verifyEq(r.isRO, true)
    verifyEq(r.size, 3)
    verifyEq(r, ["a", "b", "c"])
    verifyEq(y, ["a", "b", "c"])
    y.sortr
    verifyNotSame(y.ro, r)
    verifyEq(y.size, 3)
    verifyEq(y, ["c", "b", "a"])
    verifySame(y.rw, y)
    verifyEq(r, ["a", "b", "c"])
    y.add("d")
    verifyEq(y.size, 4)
    verifyEq(y, ["c", "b", "a", "d"])
    verifyEq(r.size, 3)
    verifyEq(r, ["a", "b", "c"])
  }

//////////////////////////////////////////////////////////////////////////
// ToImmutable
//////////////////////////////////////////////////////////////////////////

  Void testToImmutable()
  {
    a := ["a"]
    b := ["b"]
    c := ["c"]

    x := [a, b, c]
    xc := x.toImmutable

    y := [x]
    yc := y.toImmutable

    verifyNotSame(x.ro, xc)
    verifyEq(xc.isRO, true)
    verifyEq(xc.isImmutable, true)
    verifySame(xc.toImmutable, xc)
    verifyEq(xc[0], a)
    verifyEq(xc[1], b)
    verifyEq(xc[2], c)
    verifyEq(xc[0].isRO, true)
    verifyEq(xc[1].isRO, true)
    verifyEq(xc[2].isRO, true)

    verifyNotSame(y.ro, yc)
    verifyEq(yc.isRO, true)
    verifyEq(yc.isImmutable, true)
    verifySame(yc.toImmutable, yc)
    verifyEq(yc[0][0].isRO, true)
    verifyEq(yc[0][0].isImmutable, true)

    m := [0:"zero", 99:null, null:"null"]
    z := [m, null]
    zc := z.toImmutable
    verifyEq(zc.type.signature, "[sys::Int:sys::Str][]")
    verify(zc.isRO)
    verify(zc[0].isRO)
    verify(zc[0].isImmutable)
    verifyEq(zc[0], m)
    verify(zc[1] == null)

    xrw := xc.rw
    verifyEq(xrw.isImmutable, false)
    verifyEq(xc.isImmutable, true)
    xrw[0] = ["Z"]
    verifyEq(xc[0], ["a"])
    verifyEq(xrw[0], ["Z"])

    verifyEq([,].isImmutable, false)
    verifyEq([,].ro.isImmutable, false)
    verifyEq([,].toImmutable.isImmutable, true)

    verifyEq([1,2].isImmutable, false)
    verifyEq([1,2].ro.isImmutable, false)
    verifyEq([1,2].toImmutable.isImmutable, true)

    verifyEq([this].isImmutable, false)
    verifyEq([this].ro.isImmutable, false)
    verifyErr(NotImmutableErr.type) |,| { [this].toImmutable }
    verifyErr(NotImmutableErr.type) |,| { [0, this, 2].toImmutable }
    verifyErr(NotImmutableErr.type) |,| { [0, [this], 2].toImmutable }
  }

}