//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   17 Mar 06  Brian Frank  Creation
//

**
** MapTest
**
class MapTest : Test
{

//////////////////////////////////////////////////////////////////////////
// Equal
//////////////////////////////////////////////////////////////////////////

  Void testEquals()
  {
    verifyEq([:], [:])
    verifyEq([0:null], [0:null])
    verifyEq([null:"a"], [null:"a"])
    verifyEq([0:"a"], [0:"a"])

    verifyEq([:].hash, [:].hash)
    verifyEq([0:null].hash, [0:null].hash)
    verifyEq([null:"a"].hash, [null:"a"].hash)
    verifyEq([0:"a"].hash, [0:"a"].hash)

    verifyNotEq([:], [0:"a"])
    verifyNotEq([0:"a"], [:])
    verifyNotEq([0:"a"], [0:null])
    verifyNotEq([null:"a"], [0:"a"])
    verifyNotEq([0:"a"], ["f":"a"])
    verifyNotEq([0:"a"], [0:3])
    verifyNotEq([:], Str:Str[:])
    verifyNotEq([:], null)
    verifyNotEq(null, [:])
    verifyNotEq([0:"0"], [0:"x"])
    verifyNotEq([0:"0"], [0:"x"])
    verifyNotEq([0:"0"], "hello")
  }

//////////////////////////////////////////////////////////////////////////
// Type
//////////////////////////////////////////////////////////////////////////

  Void testType()
  {
    verifyEq(Int:Str.type, Int:Str.type)
    verifyNotEq(Int:Str.type, Int:Obj.type)
    verifyNotEq(Int:Str.type, Obj:Str.type)
    verifyNotEq(Int:Str.type, [:].type)
  }

//////////////////////////////////////////////////////////////////////////
// Is Operator
//////////////////////////////////////////////////////////////////////////

  Void testIsExplicit()
  {
    // Obj:Obj
    Obj a := Obj:Obj[:]
    verify(a is Obj)
    verify(a is Map)
    verify(a is Obj:Obj)
    verify(a is Obj:Obj)
    verifyFalse(a is Str)
    verifyFalse(a is Str:Obj)
    verifyFalse(a is Obj:Str)

    // Int:Str - empty
    Obj b := Str:Int[:]
    verify(b is Obj)
    verify(b is Map)
    verify(b is Obj:Obj)
    verify(b is Obj:Int)
    verify(b is Str:Obj)
    verify(b is Str:Int)
    verifyFalse(b is Str)
    verifyFalse(b is Str[])
    verifyFalse(b is Str:Str)

    // Int:Str - with values
    Obj c := Int:Str[2:"b"]
    verifyFalse(c is Str:Str)

    // Str:Field
    Obj d := Str:Field[:]
    verify(d is Obj)
    verify(d is Str:Obj)
    verify(d is Str:Slot)
    verify(d is Str:Field)
  }

  Void testIsInfered()
  {
    // empty Obj[Obj]
    Obj a := [:]
    verify(a is Obj:Obj)
    verifyEq(a.type, Obj:Obj.type)

    // inferred Obj:Obj
    Obj b := [2:"two", "three":3]
    verify(b is Obj:Obj)
    verifyEq(b.type, Obj:Obj.type)

    // inferred Int:Str
    Obj c := [3:"c"]
    verify(c is Obj)
    verify(c is Int:Str)
    verify(c is Num:Str)
    verify(c is Num:Obj)
    verify(c is Obj:Obj)
    verifyEq(c.type, Int:Str.type)
    verifyEq([null : "c" , 4 : "d"].type, Int:Str.type) // null
    verifyEq([3 : null , 4 : "d"].type, Int:Str.type)   // null

    Obj d := [3:"c"]
    verifyNotEq(d.type, Obj:Str.type)
    verifyNotEq(d.type, Int:Obj.type)
    verifyNotEq(d.type, Sys:Bool.type)
  }

//////////////////////////////////////////////////////////////////////////
// As Operator
//////////////////////////////////////////////////////////////////////////

  Void testAsExplicit()
  {
    Obj x := [:]

    Obj o    := x as Obj;          verifySame(o , x)
    Bool b   := x as Bool;         verifySame(b , null)
    Str s    := x as Str;          verifySame(s , null)
    Map m    := x as Map;          verifySame(m , x)
    Obj:Obj  ol := x as Obj:Obj;   verifySame(ol , x)
    Obj:Int  il := x as Int:Int;   verifySame(il , null)
    Int:Str  sl := x as Int:Str;   verifySame(sl , null)

    x  = [0:"a", 1:"b"]
    o  = x as Obj;       verifySame(o , x)
    b  = x as Bool;      verifySame(b , null)
    s  = x as Str;       verifySame(s , null)
    m  = x as Map;       verifySame(m , x)
    ol = x as Obj:Obj;   verifySame(ol , x)
    il = x as Str:Int;   verifySame(il , null)
    sl = x as Int:Str;   verifySame(sl , x)
  }

//////////////////////////////////////////////////////////////////////////
// Reflection
//////////////////////////////////////////////////////////////////////////

  Void testReflect()
  {
    a := [:]
    verifyEq(a.type.base,      Map.type)
    verifyEq(a.type.base.base, Obj.type)
    verifyEq(a.type.pod.name,  "sys")
    verifyEq(a.type.name,      "Map")
    verifyEq(a.type.qname,     "sys::Map")
    verifyEq(a.type.signature, "[sys::Obj:sys::Obj]")
    verifyEq(a.type.toStr,     "[sys::Obj:sys::Obj]")
    verifyEq(a.type.method("isEmpty").returnType, Bool.type)
    verifyEq(a.type.method("get").returnType ,         Obj.type)
    verifyEq(a.type.method("get").params[0].paramType, Obj.type)
    verifyEq(a.type.method("set").returnType,          Obj:Obj.type)
    verifyEq(a.type.method("set").params[0].paramType, Obj.type)
    verifyEq(a.type.method("set").params[1].paramType, Obj.type)
    verifyEq(a.type.method("each").params[0].paramType, |Obj v, Obj k->Void|.type)
    verifyNotEq(a.type.method("each").params[0].paramType, |Str v, Obj k->Void|.type)

    b := [0:"zero"]
    verifyEq(b.type.base,      Map.type)
    verifyEq(b.type.base.base, Obj.type)
    verifyEq(b.type.pod.name,  "sys")
    verifyEq(b.type.name,      "Map")
    verifyEq(b.type.qname,     "sys::Map")
    verifyEq(b.type.signature, "[sys::Int:sys::Str]")
    verifyEq(b.type.toStr,     "[sys::Int:sys::Str]")
    verifyEq(b.type.method("isEmpty").returnType, Bool.type)
    verifyEq(b.type.method("get").returnType,          Str.type)
    verifyEq(b.type.method("get").params[0].paramType, Int.type)
    verifyEq(b.type.method("set").returnType,          Int:Str.type)
    verifyEq(b.type.method("set").params[0].paramType, Int.type)
    verifyEq(b.type.method("set").params[1].paramType, Str.type)
    verifyEq(b.type.method("keys").returnType, Int[].type)
    verifyEq(b.type.method("values").returnType, Str[].type)
    verifyEq(b.type.method("each").params[0].paramType, |Str v, Int k->Void|.type)
    verifyNotEq(b.type.method("each").params[0].paramType, |Obj v, Int i->Void|.type)

    c := [ArgErr.make:[0,1,2]]
    verifyEq(c.type.base,      Map.type)
    verifyEq(c.type.base.base, Obj.type)
    verifyEq(c.type.pod.name,  "sys")
    verifyEq(c.type.name,      "Map")
    verifyEq(c.type.qname,     "sys::Map")
    verifyEq(c.type.signature, "[sys::ArgErr:sys::Int[]]")
    verifyEq(c.type.toStr,     "[sys::ArgErr:sys::Int[]]")
    verifyEq(c.type.method("isEmpty").returnType, Bool.type)
    verifyEq(c.type.method("get").returnType,          Int[].type)
    verifyEq(c.type.method("get").params[0].paramType, ArgErr.type)
    verifyEq(c.type.method("set").returnType,          ArgErr:Int[].type)
    verifyEq(c.type.method("set").returnType,          [ArgErr:Int[]].type)
    verifyEq(c.type.method("set").params[0].paramType, ArgErr.type)
    verifyEq(c.type.method("set").params[1].paramType, Int[].type)
    verifyEq(c.type.method("keys").returnType, ArgErr[].type)
    verifyEq(c.type.method("values").returnType, Int[][].type)
    verifyEq(c.type.method("each").params[0].paramType, |Int[] v, ArgErr k->Void|.type)
  }

//////////////////////////////////////////////////////////////////////////
// Add/Remove
//////////////////////////////////////////////////////////////////////////

  Void testItems()
  {
    m := Int:Str[:]
    verifyEq(m, Int:Str[:])
    verifyEq(m[0], null)
    verify(m.isEmpty)

    m[5] = "five"
    verifyEq(m, [5:"five"])
    verifyEq(m[0], null)
    verifyEq(m[5], "five")
    verifyEq(m.get(5), "five")
    verifyEq(m.get(3), null)
    verifyEq(m.get(3, "?"), "?")
    verifyFalse(m.isEmpty)

    m[9] = "nine"
    verifyEq(m, [5:"five", 9:"nine"])
    verifyEq(m[0], null)
    verifyEq(m[5], "five")
    verifyEq(m[9], "nine")
    verifyFalse(m.isEmpty)

    m.add(2, "two")
    verifyEq(m, [2:"two", 5:"five", 9:"nine"])
    verifyEq(m[0], null)
    verifyEq(m[2], "two")
    verifyEq(m[5], "five")
    verifyEq(m[9], "nine")
    verifyErr(ArgErr.type) |,| { m.add(2, "err") }
    verifyEq(m[2], "two")

    m[9] = null
    verifyEq(m, [2:"two", 5:"five", 9:null])
    verifyEq(m[0], null)
    verifyEq(m[2], "two")
    verifyEq(m[5], "five")
    verifyEq(m[9], null)
    verifyEq(m.get(0, "?"), "?")
    verifyEq(m.get(2, "?"), "two")
    verifyEq(m.get(9, "?"), "?") // mapped, but null returns def

    m.add(9, "nine") // add overwrites null
    verifyEq(m, [2:"two", 5:"five", 9:"nine"])
    verifyEq(m[0], null)
    verifyEq(m[2], "two")
    verifyEq(m[5], "five")
    verifyEq(m[9], "nine")
    verifyErr(ArgErr.type) |,| { m.add(9, "err") }
    verifyEq(m[9], "nine")
    m[9] = null

    m[null] = "null"
    verifyEq(m, [null:"null", 2:"two", 5:"five", 9:null])
    verifyEq(m[null], "null"); verify(m.containsKey(null))
    verifyEq(m[0], null);      verify(!m.containsKey(0))
    verifyEq(m[2], "two");     verify(m.containsKey(2))
    verifyEq(m[5], "five");    verify(m.containsKey(5))
    verifyEq(m[9], null);      verify(m.containsKey(9))

    m.remove(null)
    verifyEq(m, [2:"two", 5:"five", 9:null])
    verifyEq(m[null], null)
    verifyEq(m[0], null)
    verifyEq(m[2], "two")
    verifyEq(m[5], "five")
    verifyEq(m[9], null)

    m.remove(9)
    verifyEq(m, [2:"two", 5:"five"])
    verifyEq(m[null], null)
    verifyEq(m[0], null)
    verifyEq(m[2], "two")
    verifyEq(m[5], "five")
    verifyEq(m[9], null)

    m.remove(5)
    verifyEq(m, [2:"two"])
    verifyEq(m[null], null)
    verifyEq(m[0], null)
    verifyEq(m[2], "two")
    verifyEq(m[5], null)
    verifyEq(m[9], null)

    m.remove(2)
    verifyEq(m, Int:Str[:])
    verifyEq(m[null], null)
    verifyEq(m[0], null)
    verifyEq(m[2], null)
    verifyEq(m[5], null)
    verifyEq(m[9], null)
    verify(m.isEmpty)

    for (Int i :=0;   i<10000; ++i) m[i] = i.toStr
    for (Int i :=0;   i<10000; ++i) verifyEq(m[i], i.toStr)
    for (Int i :=500; i<10000; ++i) m.remove(i)
    for (Int i :=0;   i<500;   ++i) verifyEq(m[i], i.toStr)
    for (Int i :=500; i<1000;  ++i) verifyEq(m[i], null)

    em := [:]
    verifyErr(NotImmutableErr.type) |,| { em.add(this, "foo") }
    verifyErr(NotImmutableErr.type) |,| { em.set(this, "foo") }
    verifyErr(NotImmutableErr.type) |,| { em[this] = "foo" }
    verify(em.isEmpty)
  }

//////////////////////////////////////////////////////////////////////////
// Duplicate
//////////////////////////////////////////////////////////////////////////

  Void testDup()
  {
    a := ['a':"A", 'b':"B", 'c':"C"]
    verifyEq(a.size, 3)
    verifyEq(a.type, Int:Str.type)
    verifyEq(a, ['a':"A", 'b':"B", 'c':"C"])

    b := a.dup
    verifyEq(b.size, 3)
    verifyEq(b.type, Int:Str.type)
    verifyEq(b, ['a':"A", 'b':"B", 'c':"C"])

    a['a'] = "X"
    verifyEq(a, ['a':"X", 'b':"B", 'c':"C"])
    verifyEq(b, ['a':"A", 'b':"B", 'c':"C"])

    a.clear
    verifyEq(a, Int:Str[:])
    verifyEq(b, ['a':"A", 'b':"B", 'c':"C"])
  }

//////////////////////////////////////////////////////////////////////////
// SetAll / AddAll
//////////////////////////////////////////////////////////////////////////

  Void testSetAddAll()
  {
    m := [2:2ns, 3:3ns, 4:4ns]
    verifyEq(m.setAll([1:10ns, 3:30ns]), [1:10ns, 2:2ns, 3:30ns, 4:4ns])

    m = [2:2ns, 3:3ns, 4:4ns]
    verifyEq(m.addAll([1:10ns, 5:50ns]), [1:10ns, 2:2ns, 3:3ns, 4:4ns, 5:50ns])
    verifyErr(ArgErr.type) |,| { m.addAll([1:10ns, 5:50ns]) }
  }

//////////////////////////////////////////////////////////////////////////
// Clear
//////////////////////////////////////////////////////////////////////////

  Void testClear()
  {
    map := ["a":"A", "b":"B", "c":"C"]
    verifyEq(map.size, 3)
    verifyFalse(map.isEmpty)

    map.clear
    verifyEq(map.size, 0)
    verify(map.isEmpty)

    map["d"] = "D"
    verifyEq(map.size, 1)
    verifyFalse(map.isEmpty)
    verifyEq(map, ["d": "D"])
  }

//////////////////////////////////////////////////////////////////////////
// Keys/Values
//////////////////////////////////////////////////////////////////////////

  Void testKeyValueLists()
  {
    m := [0:"zero"]
    verifyEq(m.keys,   [0])
    verifyEq(m.values, ["zero"])

    m = [0:"zero", 1:"one"]
    verifyEq(m.keys.sort,   [0, 1])
    verifyEq(m.values.sort, ["one", "zero"])

    m = [0:"zero", 1:"one", 2:"two"]
    verifyEq(m.keys.sort,   [0, 1, 2])
    verifyEq(m.values.sort, ["one", "two", "zero"])

    m = [0:"zero", 1:"one", 2:"two", 3:"three"]
    verifyEq(m.keys.sort,   [0, 1, 2, 3])
    verifyEq(m.values.sort, ["one", "three", "two", "zero"])

    m = [0:"zero", 1:"one", 2:"two", 3:"three", 4:"four"]
    verifyEq(m.keys.sort,   [0, 1, 2, 3, 4])
    verifyEq(m.values.sort, ["four", "one", "three", "two", "zero"])

    x := ["a":[0], "b":[0,1], "c":[0,1,2]]
    verifyEq(x.keys.sort,   ["a", "b", "c"])
    verifyEq(x.values.sort |Int[] a, Int[] b->Int| { return a.size <=> b.size },
             [[0], [0,1], [0,1,2]])
  }

//////////////////////////////////////////////////////////////////////////
// Each
//////////////////////////////////////////////////////////////////////////

  Void testEach()
  {
    keys := Int[,]
    vals := Str[,]

    // empty list
    Int:Str[:].each |Str val, Int key|
    {
      vals.add(val)
      keys.add(key)
    }
    verifyEq(keys, Int[,])
    verifyEq(vals, Str[,]);

    // list of one
    keys.clear; vals.clear;
    [0:"zero"].each |Str val, Int key|
    {
      vals.add(val)
      keys.add(key)
    }
    verifyEq(keys, [0])
    verifyEq(vals, ["zero"]);

    // list of two
    keys.clear; vals.clear;
    [0:"zero", 1:"one"].each |Str val, Int key|
    {
      vals.add(val)
      keys.add(key)
    }
    verify(keys.size == 2);
    verify(keys.contains(0)); verify(keys.index(0) == vals.index("zero"));
    verify(keys.contains(1)); verify(keys.index(1) == vals.index("one"));

    // list of ten
    keys.clear; vals.clear;
    [0:"zero", 1:"one", 2:"two", 3:"three", 4:"four",
     5:"five", 6:"six", 7:"seven", 8:"eight", 9:"nine"].each |Str val, Int key|
    {
      vals.add(val)
      keys.add(key)
    }
    verify(keys.size == 10);
    verify(keys.contains(0)); verify(keys.index(0) == vals.index("zero"));
    verify(keys.contains(1)); verify(keys.index(1) == vals.index("one"));
    verify(keys.contains(2)); verify(keys.index(2) == vals.index("two"));
    verify(keys.contains(3)); verify(keys.index(3) == vals.index("three"));
    verify(keys.contains(4)); verify(keys.index(4) == vals.index("four"));
    verify(keys.contains(5)); verify(keys.index(5) == vals.index("five"));
    verify(keys.contains(6)); verify(keys.index(6) == vals.index("six"));
    verify(keys.contains(7)); verify(keys.index(7) == vals.index("seven"));
    verify(keys.contains(8)); verify(keys.index(8) == vals.index("eight"));
    verify(keys.contains(9)); verify(keys.index(9) == vals.index("nine"));
  }

//////////////////////////////////////////////////////////////////////////
// Find
//////////////////////////////////////////////////////////////////////////

  Void testFind()
  {
    map := [0:"zero", 1:"one", 2:"two", 3:"three", 4:"four"]

    // find
    verifyEq(map.find |Str v, Int k->Bool| { return k == 2 }, "two")
    verifyEq(map.find |Str v, Int k->Bool| { return v == "four" }, "four")
    verifyEq(map.find |Str v, Int k->Bool| { return false }, null)
    verifyEq(map.find |Str v->Bool| { return false }, null)
    verifyEq(map.find |->Bool| { return false }, null)

    // typed assign
    Str x := map.find |Str v->Bool| { return v.size == 5 }
    verifyEq(x, "three")

    // findAll
    verifyEq(map.findAll|Str v, Int k->Bool| { return v.size == 3 }, [1:"one", 2:"two"])
    verifyEq(map.findAll|Str v, Int k->Bool| { return k % 2 == 0 },  [0:"zero", 2:"two", 4:"four"])
    verifyEq(map.findAll|Str v, Int k->Bool| { return false },  Int:Str[:])
    verifyEq(map.findAll|Str v->Bool| { return false },  Int:Str[:])
    verifyEq(map.findAll|->Bool| { return false },  Int:Str[:])

    // exclude
    map2 := ["off":0, "slow":50, "fast":100]
    verifyEq(map2.exclude|Int v->Bool| { return v == 0 }, ["slow":50, "fast":100])
    verifyEq(map2.exclude|Int v->Bool| { return true }, Str:Int[:])

    // typed assign
    Int:Str a := map.findAll |Str v->Bool| { return v.size == 4 }
    verifyEq(a, [0:"zero", 4:"four"])
  }

//////////////////////////////////////////////////////////////////////////
// Reduce
//////////////////////////////////////////////////////////////////////////

  Void testReduce()
  {
    map := [0:"zero", 1:"one", 2:"two", 3:"three", ]

    verifyEq(map.reduce(0) |Obj r, Str v, Int k->Obj| { return (Int)r + k }, 6)

    vals := Str[,]
    map.reduce(vals) |Obj r, Str v, Int k->Obj| { return vals.add(v) }
    verifyEq(vals.sort, ["one", "three", "two", "zero"])
  }

//////////////////////////////////////////////////////////////////////////
// Map
//////////////////////////////////////////////////////////////////////////

  Void testMap()
  {
    map := [0:"zero", 1:"one", 2:"two"]

    c := Int:Str[:]
    map.map(c) |Str v->Obj| { return "($v)" }
    verifyEq(c, [0:"(zero)", 1:"(one)", 2:"(two)"])

    c2 := Int:Int[:]
    map.map(c2) |Str v, Int k->Obj| { return k*2 }
    verifyEq(c2, [0:0, 1:2, 2:4])
  }

//////////////////////////////////////////////////////////////////////////
// Readonly
//////////////////////////////////////////////////////////////////////////

  Void testReadonly()
  {
    // create rw map
    x := [0:"a", 1:"b", 2:"c"]
    verifyEq(x.isRW, true)
    verifyEq(x.isRO, false)
    verifySame(x.rw, x)

    // get ro list
    r := x.ro
    verifyEq(x.isRW, true)
    verifyEq(x.isRO, false)
    verifySame(x.rw, x)
    verifyEq(r.isRW, false)
    verifyEq(r.isRO, true)
    verifySame(x.ro, r)
    verifySame(x.ro, r)
    verifySame(r.ro, r)
    verifySame(r.ro, r)
    verifyEq(r, x)

    // verify all idempotent methods work
    verifyEq(r.type, Int:Str.type)
    verifyEq(r.isEmpty, false)
    verifyEq(r.size, 3)
    verifyEq(r[0], "a")
    verifyEq(r[1], "b")
    verifyEq(r[2], "c")
    verifyEq(r.get(3, "?"), "?")
    verifyEq(r.containsKey(2), true)
    verifyEq(r.containsKey(4), false)
    verifyEq(r.keys.sort, [0, 1, 2])
    verifyEq(r.values.sort, ["a", "b", "c"])
    verifyEq(r.dup, [0:"a", 1:"b", 2:"c"])
    r.each |Str v, Int k| { verifyEq(r[k], v) }
    verifyEq(r.find |Str s->Bool| { return s == "b" }, "b")
    verifyEq(r.findAll |Str s->Bool| { return true }, [0:"a", 1:"b", 2:"c"])
    verifyEq(r.toStr, [0:"a", 1:"b", 2:"c"].toStr)
//verifyEq(r.join, "abc")

    // verify all modification methods throw ReadonlyErr
    verifyErr(ReadonlyErr.type) |,| { r[2] = "x" }
    verifyErr(ReadonlyErr.type) |,| { r[3] = "x" }
    verifyErr(ReadonlyErr.type) |,| { r.add(2, "?") }
    verifyErr(ReadonlyErr.type) |,| { r.setAll([1:"yikes!"]) }
    verifyErr(ReadonlyErr.type) |,| { r.addAll([1:"yikes!"]) }
    verifyErr(ReadonlyErr.type) |,| { r.remove(0) }
    verifyErr(ReadonlyErr.type) |,| { r.remove(5) }
    verifyErr(ReadonlyErr.type) |,| { r.clear }

    // verify rw detaches ro
    x[3] = "d"
    r2 := x.ro
    verifySame(x.ro, r2)
    verifyNotSame(r2, r)
    verifyNotSame(x.ro, r)
    verifyEq(r.isRO, true)
    verifyEq(r.size, 3)
    verifyEq(r, [0:"a", 1:"b", 2:"c"])
    verifyNotEq(r, x)
    verifyNotEq(r, r2)
    x.remove(3)
    r3 := x.ro
    verifySame(x.ro, r3)
    verifyNotSame(r2, r3)
    verifyNotSame(r3, r)
    verifyNotSame(r2, r)
    verifyNotSame(x.ro, r)
    verifyEq(r.size, 3)
    verifyEq(r, [0:"a", 1:"b", 2:"c"])

    // verify ro to rw
    y := r.rw
    verifyEq(y.isRW, true)
    verifyEq(y.isRO, false)
    verifySame(y.rw, y)
    verifySame(y.ro, r)
    verifyEq(y, r)
    verifyEq(r.isRO, true)
    verifyEq(r.size, 3)
    verifyEq(r, [0:"a", 1:"b", 2:"c"])
    verifyEq(y, [0:"a", 1:"b", 2:"c"])
    y.clear
    verifyNotSame(y.ro, r)
    verifyEq(y.size, 0)
    verifySame(y.rw, y)
    verifyEq(r, [0:"a", 1:"b", 2:"c"])
    y[-1] = "!"
    verifyEq(y.size, 1)
    verifyEq(y, [-1:"!"])
    verifyEq(r.size, 3)
    verifyEq(r, [0:"a", 1:"b", 2:"c"])
  }

//////////////////////////////////////////////////////////////////////////
// ToImmutable
//////////////////////////////////////////////////////////////////////////

  Void testToImmutable()
  {
    m := [
          [0].toImmutable: [0ns:"zero"],
          [1].toImmutable: [1ns:"one"],
          null: [9ns:"null"],
          [2].toImmutable :null
         ]
    mc := m.toImmutable
    verifyEq(m, mc)
    verifySame(mc.toImmutable, mc)

    verifyNotSame(m, mc)
    verifyNotSame(m.ro, mc)
    verify(mc.isRO)
    verify(mc.isImmutable)
    verifyEq(mc.type.signature, "[sys::Int[]:[sys::Duration:sys::Str]]")
    verifyEq(mc.get([0]), [0ns:"zero"])
    verifyEq(mc.get(null), [9ns:"null"])
    verifyEq(mc.get([2]), null)
    verify(mc.get([0]).isRO)
    verify(mc.get([0]).isImmutable)
    verify(mc.get(null).isRO)
    verify(mc.get(null).isImmutable)
    mc.keys.each |Int[] k| { if (k != null) verify(k.isImmutable) }

    mx := mc.rw
    verifyEq(mx.isImmutable, false)
    verifyEq(mc.isImmutable, true)
    mx[[0].toImmutable] = [7ns:"seven"]
    verifyEq(mc.get([0]), [0ns:"zero"])
    verifyEq(mx.get([0]), [7ns:"seven"])

    verifyEq([0:"zero"].isImmutable, false)
    verifyEq([0:"zero"].ro.isImmutable, false)
    verifyEq([0:"zero"].toImmutable.isImmutable, true)

    verifyEq([0:this].isImmutable, false)
    verifyEq([0:this].ro.isImmutable, false)
    verifyErr(NotImmutableErr.type) |,| { [0:this].toImmutable }
    verifyErr(NotImmutableErr.type) |,| { [0:[this]].toImmutable }
    verifyErr(NotImmutableErr.type) |,| { [4:[8ns:this]].toImmutable }
  }

}