//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   14 Mar 06  Brian Frank  Creation
//

**
** StrTest
**
class StrTest : Test
{

//////////////////////////////////////////////////////////////////////////
// Str Interpolation
//////////////////////////////////////////////////////////////////////////

  Void testInterpolation()
  {
    hello := "hello"
    five  := 5
    x     := 0xab

    verifyEq("\$"[0], '$')
    verifyEq("\$".size, 1)
    verifyEq("$hello",   "hello")
    verifyEq("$five",    "5")
    verifyEq("$x",       "171")
    verifyEq("$x.toHex", "ab")
    verifyEq("$x.toHex.size",   "2")
    verifyEq("$x.toHex.size()", "2()")
    verifyEq("$type",    "sysTest::StrTest")
    verifyEq("x$five",   "x5")
    verifyEq("$five^",   "5^")
    verifyEq("x$five*",  "x5*")
    verifyEq("$five$hello",    "5hello")
    verifyEq("$five $hello",   "5 hello")
    verifyEq("<$five $hello>", "<5 hello>")
    verifyEq("${five}",  "5")
    verifyEq("${hello}", "hello")
    verifyEq("${five + 3}",   "8")
    verifyEq("${hello.size}", "5")
    verifyEq("${(five + 1) * 3 - 3}", "15")
    verifyEq("${((five + 1) * 3 - 3).toHex}", "f")

    y := "$five".toInt
    verifyEq(y.type, Int.type)
    verifyEq(y, 5)
  }

//////////////////////////////////////////////////////////////////////////
// Equals
//////////////////////////////////////////////////////////////////////////

  Void testEquals()
  {
    verify("" == "")
    verify("a" == "a")
    verify("aB" == "aB")
    verify("" != "a")
    verify("bad" != "ass")
    verify("" != null)
    verify(null != "")
    verify("ok" != null)
    verify(null != "no")
    verify("wow" as Obj != 4)
  }

//////////////////////////////////////////////////////////////////////////
// Compare
//////////////////////////////////////////////////////////////////////////

  Void testCompare()
  {
    verify("a" < "b")
    verify(null < "")
    verify(null < " ")
    verifyFalse("abc" < "abc")
    verifyFalse("a" < "A")
    verifyFalse("a" < null)

    verify("3" <= "3")
    verify("3" <= "3")
    verify(null <= "3")
    verifyFalse("6" <= "5")
    verifyFalse("5" <= null)

    verify("abcd" > "abc")
    verify("a" > "")
    verify("ABX" > "ABS")
    verifyFalse(null > "cool")
    verifyFalse("A" > "B")

    verify("ab" >= "aa")
    verify("abc" >= "abc")
    verify("anything" >= null)
    verifyFalse(null >= "")
    verifyFalse(null >= " x ")
    verifyFalse("y" >= "z")

    verifyEq("a" <=> "b", -1)
    verifyEq("a" <=> "a", 0)
    verifyEq("b" <=> "a", 1)
  }

//////////////////////////////////////////////////////////////////////////
// Intern
//////////////////////////////////////////////////////////////////////////

  Void testIntern()
  {
    s := StrBuf.make.addChar('a').addChar('b').addChar('c').toStr
    verifySame("abc".intern, "abc")
    verifyNotSame(s, "abc")
    verifySame(s.intern, "abc".intern)
  }

//////////////////////////////////////////////////////////////////////////
// Size
//////////////////////////////////////////////////////////////////////////

  Void testSize()
  {
    verifyEq("".size, 0)
    verifyEq("".isEmpty, true)
    verifyEq("a".size, 1)
    verifyEq("a".isEmpty, false)
    verifyEq("ab".size, 2)
    verifyEq("ab".isEmpty, false)
  }

//////////////////////////////////////////////////////////////////////////
// Concat
//////////////////////////////////////////////////////////////////////////

  Void testConcat()
  {
    s := "hello number " + 5
    verifyEq(s, "hello number 5");

    s += "!"
    verifyEq(s, "hello number 5!");

    s += null
    verifyEq(s, "hello number 5!null");

    s += ""
    verifyEq(s, "hello number 5!null");

    s = s + " all done"
    verifyEq(s, "hello number 5!null all done");
  }

//////////////////////////////////////////////////////////////////////////
// Get
//////////////////////////////////////////////////////////////////////////

  Void testGet()
  {
    s := "abc\u00fb"
    verifyEq(s.size, 4)

    verifyEq(s[0], 97)
    verifyEq(s[1], 98)
    verifyEq(s[2], 99)
    verifyEq(s[3], 0xfb)

    verifyEq(s[-1], 0xfb)
    verifyEq(s[-2], 'c')
    verifyEq(s[-3], 'b')
    verifyEq(s[-4], 'a')

    // TODO: err on out of index
  }

//////////////////////////////////////////////////////////////////////////
// Slice
//////////////////////////////////////////////////////////////////////////

  Void testSlice()
  {
    /* Ruby
    irb(main):001:0> "abcd"[0..1]    => "ab"
    irb(main):002:0> "abcd"[0..2]    => "abc"
    irb(main):003:0> "abcd"[0..3]    => "abcd"
    irb(main):004:0> "abcd"[0..4]    => "abcd"
    irb(main):005:0> "abcd"[0..5]    => "abcd"
    irb(main):006:0> "abcd"[1..1]    => "b"
    irb(main):007:0> "abcd"[1..2]    => "bc"
    irb(main):008:0> "abcd"[1..3]    => "bcd"
    irb(main):009:0> "abcd"[1..4]    => "bcd"
    irb(main):010:0> "abcd"[0...1]   => "a"
    irb(main):011:0> "abcd"[0...2]   => "ab"
    irb(main):012:0> "abcd"[0...3]   => "abc"
    irb(main):013:0> "abcd"[0...4]   => "abcd"
    irb(main):014:0> "abcd"[0...5]   => "abcd"
    irb(main):015:0> "abcd"[0..-1]   => "abcd"
    irb(main):016:0> "abcd"[0..-2]   => "abc"
    irb(main):017:0> "abcd"[0..-3]   => "ab"
    irb(main):018:0> "abcd"[0..-4]   => "a"
    irb(main):019:0> "abcd"[0..-5]   => ""
    irb(main):020:0> "abcd"[0...-1]  => "abc"
    irb(main):021:0> "abcd"[0...-2]  => "ab"
    irb(main):022:0> "abcd"[0...-3]  => "a"
    irb(main):023:0> "abcd"[1...-3]  => ""
    irb(main):024:0> "abcd"[1...-1]  => "bc"
    irb(main):025:0> "abcd"[-3...-1] => "bc"
    irb(main):026:0> "abcd"[-2...-1] => "c"
    irb(main):027:0> "abcd"[-3...-1] => "bc"
    irb(main):028:0> "abcd"[-1...-1] => ""
    */

    // TODO: not clear how error handling should work...

    s := "abcd"
    verifyEq(s[0..1],    "ab")
    verifyEq(s[0...0],   "")
    verifyEq(s[0..2],    "abc")
    verifyEq(s[0..3],    "abcd")
    verifyEq(s[1..1],    "b")
    verifyEq(s[1..2],    "bc")
    verifyEq(s[1..3],    "bcd")
    verifyEq(s[3..2],    "")
    verifyEq(s[3..3],    "d")
    verifyEq(s[4..-1],   "")
    verifyEq(s[0...1],   "a")
    verifyEq(s[0...2],   "ab")
    verifyEq(s[0...3],   "abc")
    verifyEq(s[0...4],   "abcd")
    verifyEq(s[0..-1],   "abcd")
    verifyEq(s[0..-2],   "abc")
    verifyEq(s[0..-3],   "ab")
    verifyEq(s[0..-4],   "a")
    verifyEq(s[0..-5],   "")
    verifyEq(s[0...-1],  "abc")
    verifyEq(s[0...-2],  "ab")
    verifyEq(s[0...-3],  "a")
    verifyEq(s[1...-3],  "")
    verifyEq(s[1...-1],  "bc")
    verifyEq(s[-3...-1], "bc")
    verifyEq(s[-2...-1], "c")
    verifyEq(s[-3...-1], "bc")
    verifyEq(s[-1...-1], "")

    // examples
    verifyEq("abcd"[0..2],   "abc")
    verifyEq("abcd"[3..3],   "d")
    verifyEq("abcd"[-2..-1], "cd")
    verifyEq("abcd"[0...2],  "ab")
    verifyEq("abcd"[1..-2],  "bc")


    verifyErr(IndexErr.type) |,| { x:=s[0..4] }
    verifyErr(IndexErr.type) |,| { x:=s[1..4] }
    verifyErr(IndexErr.type) |,| { x:=s[3..1] }
    verifyErr(IndexErr.type) |,| { x:=s[3...2] }
    verifyErr(IndexErr.type) |,| { x:=s[0...5] }
  }

//////////////////////////////////////////////////////////////////////////
// StartsWith, EndsWith
//////////////////////////////////////////////////////////////////////////

  Void testStartsWith()
  {
    verify("".startsWith(""))

    verify("x".startsWith(""))
    verify("x".startsWith("x"))
    verifyFalse("x".startsWith("xy"))

    verify("foo".startsWith(""))
    verify("foo".startsWith("f"))
    verify("foo".startsWith("fo"))
    verify("foo".startsWith("foo"))
    verifyFalse("foo".startsWith("fool"))
    verifyFalse("foo".startsWith("F"))
    verifyFalse("foo".startsWith("xyz"))
  }

  Void testEndsWith()
  {
    verify("".endsWith(""))

    verify("x".endsWith(""))
    verify("x".endsWith("x"))
    verifyFalse("x".endsWith("yx"))
    verifyFalse("x".endsWith("!"))

    verify("xyz".endsWith(""))
    verify("xyz".endsWith("z"))
    verify("xyz".endsWith("yz"))
    verify("xyz".endsWith("xyz"))
    verifyFalse("x".endsWith("!"))
    verifyFalse("x".endsWith("wxyz"))
  }

//////////////////////////////////////////////////////////////////////////
// Index
//////////////////////////////////////////////////////////////////////////

  Void testIndex()
  {
    verifyEq("abcd".index("a"), 0)
    verifyEq("abcd".index("b"), 1)
    verifyEq("abcd".index("c"), 2)
    verifyEq("abcd".index("d"), 3)
    verifyEq("abcd".index("e"), null)
    verifyEq("abcd".index("ab"), 0)
    verifyEq("abcd".index("bcd"), 1)
    verifyEq("abcd".index("cd"), 2)
    verifyEq("abcd".index("cdx"), null)
    verifyEq("abcd".index("a", 1), null)
    verifyEq("abcd".index("a", -1), null)
    verifyEq("abcd".index("a", -3), null)
    verifyEq("abcd".index("a", -4), 0)

    verifyEq("billy bob".index("b"), 0)
    verifyEq("billy bob".index("b", 1), 6)
    verifyEq("billy bob".index("b", 7), 8)
    verifyEq("billy bob".index("b", 9), null)

    verifyEq("billy bob".index("b", -1), 8)
    verifyEq("billy bob".index("b", -2), 8)
    verifyEq("billy bob".index("b", -4), 6)

    // fandoc examples
    verifyEq("abcabc".index("b"), 1)
    verifyEq("abcabc".index("b", 1), 1)
    verifyEq("abcabc".index("b", 3), 4)
    verifyEq("abcabc".index("b", -3), 4)
    verifyEq("abcabc".index("x"), null)
  }

  Void testIndexr()
  {
    verifyEq("abcd".indexr("a"), 0)
    verifyEq("abcd".indexr("b"), 1)
    verifyEq("abcd".indexr("c"), 2)
    verifyEq("abcd".indexr("d"), 3)
    verifyEq("abcd".indexr("e"), null)
    verifyEq("abcd".indexr("ab"), 0)
    verifyEq("abcd".indexr("bcd"), 1)
    verifyEq("abcd".indexr("cd"), 2)
    verifyEq("abcd".indexr("cdx"), null)
    verifyEq("abcd".indexr("a", 1), 0)
    verifyEq("abcd".indexr("a", -1), 0)
    verifyEq("abcd".indexr("a", -3), 0)
    verifyEq("abcd".indexr("a", -4), 0)

    verifyEq("bee hee".indexr("ee", -1), 5)
    verifyEq("bee hee".indexr("ee", -2), 5)
    verifyEq("bee hee".indexr("ee", -3), 1)

    // fandoc examples
    verifyEq("abcabc".indexr("b"), 4)
    verifyEq("abcabc".indexr("b", -3), 1)
    verifyEq("abcabc".indexr("b", 0), null)
  }

//////////////////////////////////////////////////////////////////////////
// Contains
//////////////////////////////////////////////////////////////////////////

  Void testContains()
  {
    verify("abcd".contains("a"))
    verify("abcd".contains('a'.toChar))
    verify("abcd".contains("bc"))
    verifyFalse("abcd".contains("x"))
    verifyFalse("abcd".contains("abx"))
  }

//////////////////////////////////////////////////////////////////////////
// Each
//////////////////////////////////////////////////////////////////////////

  Void testEach()
  {
    chars   := Int[,]
    indices := Int[,]

    // empty with just char
    chars.clear; indices.clear
    "".each |Int c| { chars.add(c) }
    verifyEq(chars, Int[,])

    // empty with char and index
    chars.clear; indices.clear
    "".each |Int c, Int index| { chars.add(c); indices.add(index) }
    verifyEq(chars, Int[,])
    verifyEq(indices, Int[,])

    // "x" with just char
    chars.clear; indices.clear
    "x".each |Int c| { chars.add(c) }
    verifyEq(chars, ['x'])

    // "x" with char and index
    chars.clear; indices.clear
    "x".each |Int c, Int index| { chars.add(c); indices.add(index) }
    verifyEq(chars, ['x'])
    verifyEq(indices, [0])

    // "abc" with just char
    chars.clear; indices.clear
    "abc".each |Int c| { chars.add(c) }
    verifyEq(chars, ['a', 'b', 'c'])

    // "abc" with char and index
    chars.clear; indices.clear
    "abc".each |Int c, Int index| { chars.add(c); indices.add(index) }
    verifyEq(chars, ['a', 'b', 'c'])
    verifyEq(indices, [0, 1, 2])
  }

//////////////////////////////////////////////////////////////////////////
// Spaces
//////////////////////////////////////////////////////////////////////////

  Void testSpaces()
  {
    x := ""
    for (Int i := 0; i < 100; ++i)
    {
      verifyEq(Str.spaces(i), x)
      x += " "
    }
    verify(Str.spaces(0)    === Str.spaces(0))
    verify(Str.spaces(4)    === Str.spaces(4))
    verify(Str.spaces(10)   === Str.spaces(10))
    verify(Str.spaces(1000) !== Str.spaces(1000))
  }

//////////////////////////////////////////////////////////////////////////
// Upper/Lower
//////////////////////////////////////////////////////////////////////////

  Void testCaseConv()
  {
    verifyEq("aBcDeFxyZ".upper, "ABCDEFXYZ")
    verifyEq("aBcDeFxyZ".lower, "abcdefxyz")

    // include some Unicode chars - U+0130 is the infamous
    // Turkish dotted I and U+0131 is the lowercase undotted i
    verifyEq("Ab\u03D0\u03F4\u0130\u0131".upper, "AB\u03D0\u03F4\u0130\u0131")
    verifyEq("aB\u03D0\u03F4\u0130\u0131".lower, "ab\u03D0\u03F4\u0130\u0131")

    // capitalize
    verifySame("".capitalize, "")
    verifySame("A".capitalize, "A")
    verifySame("Abc".capitalize, "Abc")
    verifySame("_brian".capitalize, "_brian")
    verifyEq("a".capitalize, "A")
    verifyEq("ab".capitalize, "Ab")
    verifyEq("brian".capitalize, "Brian")

    // decapitalize
    verifySame("".decapitalize, "")
    verifySame("x".decapitalize, "x")
    verifySame("?".decapitalize, "?")
    verifyEq("X".decapitalize, "x")
    verifyEq("Brian".decapitalize, "brian")
    verifyEq("BAR".decapitalize, "bAR")
  }

//////////////////////////////////////////////////////////////////////////
// Just
//////////////////////////////////////////////////////////////////////////

  Void testJust()
  {
    verifySame("abc".justl(2), "abc")
    verifySame("abc".justl(3), "abc")
    verifyEq("abc".justl(4), "abc ")
    verifyEq("abc".justl(5), "abc  ")

    verifySame("abc".justr(2), "abc")
    verifySame("abc".justr(3), "abc")
    verifyEq("abc".justr(4), " abc")
    verifyEq("abc".justr(5), "  abc")
  }

//////////////////////////////////////////////////////////////////////////
// Reverse
//////////////////////////////////////////////////////////////////////////

  Void testReverse()
  {
    verifySame("".reverse, "")
    verifySame("x".reverse, "x")
    verifyEq("ab".reverse, "ba")
    verifyEq("xyz\uabcd".reverse, "\uabcdzyx")
  }

//////////////////////////////////////////////////////////////////////////
// Trim
//////////////////////////////////////////////////////////////////////////

  Void testTrim()
  {
    verifySame("".trim, "")
    verifySame("x".trim, "x")
    verifySame("!!".trim, "!!")
    verifyEq(" ".trim, "")
    verifyEq(" x".trim, "x")
    verifyEq("x ".trim, "x")
    verifyEq(" x ".trim, "x")
    verifyEq(" \n x \r  ".trim, "x")
    verifyEq("\u0019\u0000|foo bar|\n\r\t \u0005\n".trim, "|foo bar|")
  }

//////////////////////////////////////////////////////////////////////////
// Split
//////////////////////////////////////////////////////////////////////////

  Void testSplit()
  {
    // TODO: this is just a temp hack
    verifyEq("a b c".split(" "), ["a", "b", "c"])
    verifyEq("foo=bar".split("="), ["foo", "bar"])
  }

//////////////////////////////////////////////////////////////////////////
// Conversions
//////////////////////////////////////////////////////////////////////////

  Void testConversion()
  {
    verifyEq("true".toBool, true)
    verifyEq("false".toBool, false)
    verifyEq("xxx".toBool(false), null)
    verifyErr(ArgErr.type) |,| { "blah".toBool }

    verifyEq("708".toInt, 708)
    verifyEq("ff".toInt(16), 0xff)
    verifyEq("xxx".toInt(10, false), null)
    verifyErr(ArgErr.type) |,| { "blah".toInt }

    verifyEq("760.5".toFloat(), 760.5)
    verifyEq("xxx".toFloat(false), null)
    verifyErr(ArgErr.type) |,| { "blah".toFloat }

    verifyEq("http://foo/".toUri, `http://foo/`)
  }

//////////////////////////////////////////////////////////////////////////
// TODO
//////////////////////////////////////////////////////////////////////////

  // TODO - need to fix when do const folding optimization
  // TODO - need to check field rvalues

}