//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   3 Jul 06  Brian Frank  Creation
//

**
** TimeTest
**
class TimeTest : Test
{

//////////////////////////////////////////////////////////////////////////
// Equals
//////////////////////////////////////////////////////////////////////////

  Void testEquals()
  {
    // equals
    verifyEq(Time.make(123_456_789), Time.make(123_456_789))
    verifyNotEq(Time.make(123_456_789), Time.make(123_456_780))

    // hash
    verifyEq(Time.make(123_456_789).hash, 123_456_789)
  }

//////////////////////////////////////////////////////////////////////////
// Compare
//////////////////////////////////////////////////////////////////////////

  Void testCompare()
  {
    verify(Time.make(723_456_789) > Time.make(123_456_789))
    verify(Time.make(123_456_789) >= Time.make(123_456_789))
    verify(Time.make(123_456_789) <= Time.make(123_456_789))
    verify(Time.make(123_456_789) <  Time.make(723_456_789))
  }

//////////////////////////////////////////////////////////////////////////
// Month/Weekday
//////////////////////////////////////////////////////////////////////////

  Void testMonth()
  {
    verifyEq(Month.type.qname, "sys::Month")
    verifySame(Month.type.base, Enum.type)
    verifySame(Month.jan.type, Month.type)

    verifyEq(Month.values.isRO, true)
    verifyEq(Month.values.size, 12)
    verifyEq(Month.values.capacity, 12)
    verifyEnum(Month.jan, 0,  "jan", Month.values)
    verifyEnum(Month.feb, 1,  "feb", Month.values)
    verifyEnum(Month.mar, 2,  "mar", Month.values)
    verifyEnum(Month.apr, 3,  "apr", Month.values)
    verifyEnum(Month.may, 4,  "may", Month.values)
    verifyEnum(Month.jun, 5,  "jun", Month.values)
    verifyEnum(Month.jul, 6,  "jul", Month.values)
    verifyEnum(Month.aug, 7,  "aug", Month.values)
    verifyEnum(Month.sep, 8,  "sep", Month.values)
    verifyEnum(Month.oct, 9,  "oct", Month.values)
    verifyEnum(Month.nov, 10, "nov", Month.values)
    verifyEnum(Month.dec, 11, "dec", Month.values)
  }

  Void testWeekday()
  {
    verifyEq(Weekday.type.qname, "sys::Weekday")
    verifySame(Weekday.type.base, Enum.type)
    verifySame(Weekday.sun.type, Weekday.type)

    verifyEq(Weekday.values.isRO, true)
    verifyEq(Weekday.values.size, 7)
    verifyEq(Weekday.values.capacity, 7)
    verifyEnum(Weekday.sun, 0, "sun", Weekday.values)
    verifyEnum(Weekday.mon, 1, "mon", Weekday.values)
    verifyEnum(Weekday.tue, 2, "tue", Weekday.values)
    verifyEnum(Weekday.wed, 3, "wed", Weekday.values)
    verifyEnum(Weekday.thu, 4, "thu", Weekday.values)
    verifyEnum(Weekday.fri, 5, "fri", Weekday.values)
    verifyEnum(Weekday.sat, 6, "sat", Weekday.values)
  }

  Void verifyEnum(Enum e, Int ordinal, Str name, Enum[] values)
  {
    verifyEq(e.ordinal, ordinal)
    verifyEq(e.name, name)
    verifySame(values[ordinal], e)
  }

//////////////////////////////////////////////////////////////////////////
// Math
//////////////////////////////////////////////////////////////////////////

  Void testMath()
  {
    now := Time.now

    yesterday := now + -1days
    tomorrow := now + 1days

    verifyEq(now.ticks - yesterday.ticks, 1days.ticks)
    verifyEq(now.ticks - tomorrow.ticks, -1days.ticks)

    verifyEq(now - yesterday, 1days)
    verifyEq(now - tomorrow, -1days)
  }

}